/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/


package armadilloUI;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.FileWriter;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.text.DateFormat;

/**
 * Provides menu bar playlist listener functionality
 * @author jonathan , kasia
 */
public class AddRemovePlaylistListener implements ActionListener {

    private String whichcase;
    private int playlistID;
    private int[] trackIDs;
    private ArmadilloDB db;
    private PlaylistTable table;
    private ArmadilloUI ui;
    private boolean isAnalysing;


    /**
     * Creates a new action listener, which adds or removes tracks from playlists,
     * used in the main UI menubar.
     * @param whichcase
     * @param playlistID
     * @param trackID
     * @param table
     */
    public AddRemovePlaylistListener(String whichcase, int playlistID, int[] trackID, PlaylistTable table, ArmadilloUI ui) {
        ArmadilloLog.LogOperation("Constructing AddRemovePlaylistListener Object");
        this.playlistID = playlistID;
        this.trackIDs = trackID;
        this.table = table;
        this.whichcase = whichcase;
        this.ui = ui;
    }

    /**
     * action performed event, when fired attempted to add/remove selected tracks
     * in the table to the chosen playlist
     * @param addPlaylistEvent
     */
    public void actionPerformed(ActionEvent addPlaylistEvent) {
        trackIDs = table.getCurrentSelection();
        this.pauseOtherThreads();
        try {
            db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            ArmadilloLog.LogError("AddRemovePlaylistListener cannot access DB - "+ex.toString());
        }

        if (trackIDs.length < 1)
            new DialogUI(8).setVisible(true);
        else
        {
            if (whichcase.equals("add"))
                for (int i=0; i<trackIDs.length; i++) {
                    db.addTrackToPlaylist(playlistID, trackIDs[i], db.getPlaylistContents(playlistID).size());
                }
            else if (whichcase.equals("remove"))
            {
                int[] rows = table.getCurrentRow();
                for (int i=0; i<trackIDs.length; i++)
                    db.removeTrackFromPlaylist(trackIDs[i], playlistID, rows[i]-i);
            }
            table.reload();
        }
        if (isAnalysing)
            ui.resumeAnalysing();
        isAnalysing = false;
    }

    private void pauseOtherThreads()
    {
        // first pause the analysing thread (if its active)
        this.isAnalysing = false;
        if (ui.getbpmThread() != null)
        {
            if (ui.getbpmThread().isAlive())
                this.isAnalysing = ui.getbpmThread().pauseAnalysing(false);
        }
    }
}
