/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * This class contains the UI source code
 */

package armadilloUI;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.Timer;
import javax.swing.Box;
import java.io.File;
import javax.swing.filechooser.*;
import javax.swing.JMenuItem;
import javax.swing.JScrollBar;
import java.awt.event.*;
import javax.swing.table.DefaultTableModel;
import java.util.*;
import org.gstreamer.State;
import javax.swing.UIManager;
import java.awt.Font;
import java.awt.Image;
import java.util.concurrent.atomic.AtomicInteger;
import javax.swing.ImageIcon;
import javax.swing.JPopupMenu;
import javax.swing.*;
import java.awt.*;


/*
 * ArmadilloUI.java
 *
 * Created on 14-Nov-2009, 12:38:12
 */


/**
 * Main UI class, controls the rest of the program
 * @author jonathan, kasia and sup
 */
public class ArmadilloUI extends javax.swing.JFrame {

    private MusicPlayer musicPlayer;
    private TrackObject currentTrack;
    private long duration;
    private long durationSeconds;
    /**
     *playTimer updates the seek bar and scrolling track info every half second
     */
    public Timer playTimer;
    private Timer resizeTimer;
    private boolean resizing=false;    
 
    /**
     * main table in UI, displays contents of playlists
     */
    public DefaultTableModel selectedPlaylistTableModel;
    private javax.swing.JSlider slider;

    //following variables read from confi file
    /**
     * holds the current level of detail of the graph
     */
    public int graphDetail;
    /**
     * holds the current amount of colour in the graph
     */
    public float graphColourSize;
    private String theme;
    private int volume;
    /**
     *holds the current level of detail of the mood analysis
     */
    public int moodDetail;
    private Boolean loopOne;
    private Boolean loopAll;
    private Boolean shuffle;
    private Boolean startingUP = true;

   // private ArrayList<Integer> playedList = new ArrayList();
    //private ArrayList<Integer> newPlaylistIDs = new ArrayList();
    //private int position=0;  //holds the position of the current song in the 'playedList'
    private FileNameExtensionFilter filter;   
    private String TrackText = "            ";
    private ArmadilloDB db;
    private ImportSong is;
    private javax.swing.ButtonGroup skinsButtonGroup = new javax.swing.ButtonGroup();
    
   // private String[] metadata = new String[4];
    private int charlength,startsub,endsub;
    private boolean updown;
    private UIUtilities util;
    private ArmadilloUI UI = this;
    private File[] filesSelected = {};
    private ArrayList<File> filesToImport = new ArrayList<File>();
    //private ArrayList<Integer> fileIDs = new ArrayList<Integer>();
    //buttons for menubar
    private javax.swing.JLabel minMenuItem = new javax.swing.JLabel("  min  ");
    private javax.swing.JLabel maxMenuItem = new javax.swing.JLabel("  max  ");
    private javax.swing.JLabel closeMenuItem = new javax.swing.JLabel("  X");
    //resizing/dragging variables
    private int mouseX = 0;
    private int mouseY = 0;
    private int screenWidth = 0;
    private int screenHeight = 0;
    private int screenX = 0;
    private int screenY = 0;
    private int counter = 0;
    private boolean dragXright;
    private boolean dragXleft;
    private boolean dragYdown;
    private boolean dragYup;
    private boolean empty=true;
    // graph drawing thread
    private Thread gt = null;
    private AtomicInteger gsync = new AtomicInteger(0);
    private int gidx = 0;
    // thread that calculates bpm and other for tracks
    private ImportSongBPMThread bpmthread = null;
    // popup menu for the 'import' progress bar
    private JPopupMenu popupImport, popupStartImport;
    private JMenuItem pauseResumeMenuItem, stopMenuItem, startMenuItem;
    private float oldDec=0;

    /** Creates new form ArmadilloUI
     * @throws Exception
     */
    public ArmadilloUI() throws Exception {
        //util
        ArmadilloLog.LogOperation("Main armadilloUI constructing");
        util = new UIUtilities();
        Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
        //sets decorated/undecorated
        try {
            String decorated = util.config("undecorated");
            if (decorated == null) {
                this.setUndecorated(true);
                util.config("undecorated","true");
            } else {
                boolean decBool = Boolean.parseBoolean(decorated);
                this.setUndecorated(decBool);
            }
        } catch (Exception e) {
            ArmadilloLog.LogError("Error opening config file - "+e.toString());
        }
        //set up config file settings
        initialConfigSettings();
        //connect to db
        try {
            db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            ArmadilloLog.LogError("Error opening DB - "+ex.toString());
        }

        // SETTING UP UI        
        initComponents();
        this.setLocationRelativeTo(null);
        //add mouse listeners
        addMouseListeners();
        //add key listeners
        addKeyListeners();

        addResizeListeners();

        //Set graph's settings
        drawGraph1.setInnerSize(graphColourSize);
 
        //Sets up the timer for the music player
        addMusicPlayer();

        //Setting up file filter
        filter = new FileNameExtensionFilter("Music Files ( .mp3 .wav .ogg .m4a )", "mp3", "wav","ogg","m4a");
        setUpNavTree();

        setUpPlaylistTable();

        setUpPlayingList();
        ImageIcon icon = new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/babyIcon.gif"));
        Image iconImage = icon.getImage();
        this.setIconImage(iconImage);
        //Image FileImg = myFileIcon.getImage();h
        //Image NewFileImg = FileImg.getScaledInstance(20, 20,  java.awt.Image.SCALE_SMOOTH);
        //newIcon = new ImageIcon(newimg);
        //ImageIcon myFileIconNew = new ImageIcon(NewFileImg);
        ImageIcon myIcon = new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/ArmadilloWritingLogo.gif"));
        Image img = myIcon.getImage();
        Image newimg = img.getScaledInstance(88, 22,  java.awt.Image.SCALE_SMOOTH);
        //newIcon = new ImageIcon(newimg);
        ImageIcon myIconNew = new ImageIcon(newimg);
        logoLabel.setIcon(myIconNew);
        //logoLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/logo3copy.gif")));



        //loads the last thing
        selectedPlaylistTable.playContents();
        this.seekBarSlider.setValue(0);
        this.Pause();
        //sets volume
        try {
            String volumeString = util.config("volume");
            if (volumeString == null) {
                util.config("volume","50");
                volSlider.setValue(50);
                musicPlayer.SetVolume(50);
            } else {
                int value = Integer.parseInt(volumeString);
                volSlider.setValue(value);
                musicPlayer.SetVolume(value);
            }
        } catch (Exception e) {
            ArmadilloLog.LogError("Error opening config file - "+e.toString());
        }
    }
    /*******************************************************************/



    private void setUpPlayingList() {
        setListLabelText("0 Tracks");
        playingList.setUp(this);
    }

    private void setUpPlaylistTable() {
        selectedPlaylistTable.setPlayingList(playingList);
        selectedPlaylistTable.isMainPlaylistTable(true);
        selectedPlaylistTable.setUI(this,navTree);
        selectedPlaylistTable.setUp();
    }

    //sets up the navigation tree component
    private void setUpNavTree() {
        navTree.setSelectedTable(selectedPlaylistTable);
        navTree.setPlayingList(playingList);
        navTree.setUIandFilter(this, filter);
        navTree.setUp();
    }

    //adds all resize listeners and tmers to the main ui
    private void addResizeListeners() {
        resizeTimer = new Timer(1000/2, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (resizing) {
                    resizing = false;
                } else {
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR);
                    setCursor(ResizeCursor);
                    resizeTimer.stop();
                }
            }
        });
        //Component listener to tell tables to resize when jframe is resized
        ComponentListener resizeListener = new ComponentListener() {
            public void componentHidden(ComponentEvent e) {}
            public void componentMoved(ComponentEvent e){}
            public void componentShown(ComponentEvent e){}
            public void componentResized(ComponentEvent e) {
                selectedPlaylistTable.packColumnWidth();
            }
        };
        addComponentListener(resizeListener);
    }

    //Sets up the music player and the timer to display the track info
    private void addMusicPlayer() {
        //SETTING UP MUSIC PLAYER
        musicPlayer = new MusicPlayer();        
        musicPlayer.playbin.setState(State.NULL);
        musicPlayer.SetVolume(0);

        playTimer = new Timer(1000/2, new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                long elapsedTime = musicPlayer.GetTimeElapsed();                
                float percentagePlayed = elapsedTime;
                percentagePlayed=percentagePlayed/duration;
                // update graph seekbar
                drawGraph1.seek(percentagePlayed);                
                percentagePlayed = 1000*percentagePlayed;
                // update slider (value between 0-1000)                
                int newSliderValue = Math.round(percentagePlayed);

                //if user is using seek bar to skip do not update                
                if (!seekBarSlider.getValueIsAdjusting()) {                    
                    seekBarSlider.setValue(newSliderValue);                    
                }                

                //MOVES ONTO NEXT SONG IN nowPlayingList IF SONG FINISHES
                if (elapsedTime != 0) {
                    if (elapsedTime >= duration) {
                        playTimer.stop();
                        Play(playingList.getNextTrack());
                    }
                }

                //scrolling track text
                charlength = (trackPanel.getWidth())/8;
                endsub = startsub + charlength;
                if (endsub>TrackText.length()) {
                    endsub = TrackText.length();
                }
                    if (TrackText.length() > charlength) {
                    if (updown) {
                        if (endsub < TrackText.length()) {
                            startsub++;
                            endsub++;
                        } else {
                            updown=false;
                        }
                    } else {
                        if (startsub > 0) {
                            startsub--;
                            endsub--;
                        } else {
                            updown = true;
                        }
                    }
                    trackLabel.setText(UIUtilities.FormatTime(elapsedTime/1000)+"/"+UIUtilities.FormatTime(durationSeconds)+"   " +TrackText.substring(startsub,endsub));
                } else {
                    trackLabel.setText(UIUtilities.FormatTime(elapsedTime/1000)+"/"+UIUtilities.FormatTime(durationSeconds)+"   " +TrackText);
                }
            }
        });
    }


    //This function loads up all the config settings, or sets the to a default
    //if they do not exist in the .config file
    private void initialConfigSettings() {
        try {

            //load from config file
            String configTheme = util.config("skin");
            String configMood = util.config("moodDetail");
            String configGraphDetail = util.config("graphDetail");
            String configGraphColourSize = util.config("graphSize");
            String configVolume = util.config("volume");
            String configShuffle = util.config("shuffle");
            String configLoopOne = util.config("loopOne");
            String configLoopAll = util.config("loopAll");

            //check values exist, if so parse them, if not set default and save in config
            if (configTheme != null) {
              this.theme = configTheme;
            } else {
                //default skin is water
                this.theme = "Water";
                util.config("skin", "Water");
            }
            if (configMood != null) {
                this.moodDetail = Integer.parseInt(configMood);
            } else {
                //default mood detail is lowest, 1 (can be 1-3)
                this.moodDetail = 1;
                util.config("moodDetail", "1");
            }
            if (configGraphDetail != null) {
                this.graphDetail = Integer.parseInt(configGraphDetail);
            } else {
                //default graph detail is lowest, 640 (can be 640,1024,1920)
                this.graphDetail = 640;
                util.config("graphDetail", "640");
            }
            if (configGraphColourSize != null) {
                this.graphColourSize = Float.parseFloat(configGraphColourSize);
            } else {
                //default graph colour size is 0.25 (between 0 and 1)
                this.graphColourSize = 0.25f;
                util.config("graphSize", "0.25");
            }
            if (configVolume != null) {
                this.volume = Integer.parseInt(configVolume);
            } else {
                //default volume is 50 (between 0 and 100)
                this.volume = 50;
                util.config("volume","50");
            }
            if (configShuffle != null) {
                this.shuffle = Boolean.parseBoolean(configShuffle);
            } else {
                //default shuffle is off
                this.shuffle = false;
                util.config("shuffle","false");
            }
            if (configLoopOne != null) {
                this.loopOne = Boolean.parseBoolean(configLoopOne);
            } else {
                //default loop current is off
                this.loopOne = false;
                util.config("loopOne","false");
            }
            if (configLoopAll != null) {
                this.loopAll = Boolean.parseBoolean(configLoopAll);
            } else {
                //default loop all is off
                this.loopAll = false;
                util.config("loopAll","false");
            }
        } catch (Exception e) {
            ArmadilloLog.LogError(("Error reading config file - "+e.toString()));
            new DialogUI(14).setVisible(true);
        }
    }

    //adds all mouse listeners need for this class
    private void addMouseListeners() {
        // create a popup menu for the import thread
        popupImport = new JPopupMenu();
        popupStartImport = new JPopupMenu();
        pauseResumeMenuItem = new JMenuItem("Pause");
        final ArmadilloUI ui = this;
        pauseResumeMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pauseResumeMenuItem(evt);
            }

            private void pauseResumeMenuItem(ActionEvent evt) {

                if (pauseResumeMenuItem.getText().equals("Pause"))
                {
                    bpmthread.pauseAnalysing(true);
                    pauseResumeMenuItem.setText("Resume");
                }
                else
                {                    
                    pauseResumeMenuItem.setText("Pause");
                    resumeAnalysing();
                }
            }
        });
        popupImport.add(pauseResumeMenuItem);
        stopMenuItem = new JMenuItem("Stop");
        stopMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateProgressBarText("Stopping...");
                stopMenuItem(evt);
            }

            private void stopMenuItem(ActionEvent evt) {
                updateProgressBarText("Stopping...");
                if (bpmthread.isAlive())
                {
                    bpmthread.stopAnalysing();
                }
                else
                    updateProgressBar(0);
            }
        });
        popupImport.add(stopMenuItem);
        startMenuItem = new JMenuItem("Analyse remaining tracks");
        startMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateProgressBarText("Gathering songs... ");
                startMenuItem(evt);
            }

            private void startMenuItem(ActionEvent evt) {
                int[] IDs = db.getTracksToAnalyse();
                // start new thread to calculate bpms of the new tracks
                bpmthread = new ImportSongBPMThread("bpmthread", IDs, ui);
            }
        });
        popupStartImport.add(startMenuItem);

         //Add mouselistener to seekbar
        MouseListener seekBarListener = new MouseAdapter() {
            @Override
            public void mouseReleased(MouseEvent click) {
                int mouseXPosition = click.getX();
                int newSeekBarValue = (mouseXPosition*1000)/seekBarSlider.getWidth();
                if (newSeekBarValue >995) {
                    UI.Play(UI.playingList.getNextTrack());
                }
                seekBarSlider.setValue(newSeekBarValue);                
                musicPlayer.Seek((newSeekBarValue*duration)/1000);
                seekBarSlider.setValueIsAdjusting(false);
                if (!playTimer.isRunning()) {
                    UI.Pause();
                }
            }
        };
        seekBarSlider.addMouseListener(seekBarListener);

        MouseListener volBarListener = new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent click) {
                int mousePosition = click.getX();                
                    float val = mousePosition;
                    val = val*100;
                    val = val/88;
                    int intval = Math.round(val);
                    volSlider.setValue(intval);
                    musicPlayer.SetVolume(volSlider.getValue());
                    try {
                        util.config("volume",Integer.toString(intval));
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
            }
        };
        volSlider.addMouseListener(volBarListener);

        MouseListener graphListener = new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent click) {
                int mouseXPosition = click.getX();
                int newSeekBarValue = (mouseXPosition*1000)/drawGraph1.getWidth();
                if (newSeekBarValue >995) {
                    UI.Play(UI.playingList.getNextTrack());
                }
                seekBarSlider.setValue(newSeekBarValue);                
                musicPlayer.Seek((newSeekBarValue*duration)/1000);                
                seekBarSlider.setValueIsAdjusting(false);
                if (!playTimer.isRunning()) {
                    UI.Pause();
                }
            }
        };
        drawGraph1.addMouseListener(graphListener);

                MouseListener menuBarClick = new MouseAdapter() {
            @Override
            @SuppressWarnings("static-access")
            public void mouseClicked(MouseEvent click) {
                if (click.getClickCount() == 2) {
                    if (UI.getExtendedState() == MAXIMIZED_BOTH) {
                        UI.setExtendedState(ArmadilloUI.NORMAL);
                    } else {
                        UI.setExtendedState(MAXIMIZED_BOTH);
                    }

                }
            }

            @Override
            public void mousePressed(MouseEvent click) {
                mouseX = click.getXOnScreen();// - UI.getX() ;
                mouseY = click.getYOnScreen();// - UI.getY() ;
                screenWidth = UI.getX();
                screenHeight = UI.getY();
                java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.MOVE_CURSOR);
                setCursor(ResizeCursor);
            }
            @Override
            public void mouseReleased(MouseEvent click) {
                UI.setLocation(screenWidth + (click.getXOnScreen() - mouseX), screenHeight + (click.getYOnScreen() - mouseY));
                java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR);
                setCursor(ResizeCursor);
            }

        };
        menuBar.addMouseListener(menuBarClick);
          MouseMotionListener menuBarDrag = new MouseAdapter() {
            @Override
            public void mouseDragged(MouseEvent click) {
                if (counter == 3) {
                    counter = 0;
                    UI.setLocation(screenWidth + (click.getXOnScreen() - mouseX), screenHeight + (click.getYOnScreen() - mouseY));
                } else {
                    counter++;
                }

            }
        };
        menuBar.addMouseMotionListener(menuBarDrag);

        //adding window resizing
        MouseMotionListener windowResize = new MouseAdapter() {
            @Override
            public void mouseMoved(MouseEvent click) {
                resizeTimer.start();
                resizing=true;
                if ((click.getXOnScreen() < UI.getX()+UI.getWidth()) && (click.getXOnScreen() > (UI.getX()+UI.getWidth()-4))) {
                    dragXright = true;
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.E_RESIZE_CURSOR);
                    setCursor(ResizeCursor);
                } else {
                    dragXright = false;
                }
                if ((click.getXOnScreen() > UI.getX()) && (click.getXOnScreen() < (UI.getX()+4))) {
                    dragXleft = true;
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.W_RESIZE_CURSOR);
                    setCursor(ResizeCursor);
                } else {
                    dragXleft = false;
                }
                if ((click.getYOnScreen() < UI.getY()+UI.getHeight()) && (click.getYOnScreen() > (UI.getY()+UI.getHeight()-4))) {
                    dragYdown = true;
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.S_RESIZE_CURSOR);
                    setCursor(ResizeCursor);
                } else {
                    dragYdown = false;
                }
                if ((click.getYOnScreen() > UI.getY()) && (click.getYOnScreen() < (UI.getY()+4))) {
                    dragYup = true;
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.W_RESIZE_CURSOR);
                    setCursor(ResizeCursor);
                } else {
                    dragYup = false;
                }

                if (dragXright && dragYdown) {
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.SE_RESIZE_CURSOR);
                    setCursor(ResizeCursor);
                } else if (dragXright && dragYup) {
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.NE_RESIZE_CURSOR);
                    setCursor(ResizeCursor);
                } else if (dragXleft && dragYup) {
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.NW_RESIZE_CURSOR);
                    setCursor(ResizeCursor);
                } else if (dragXleft && dragYdown) {
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.SW_RESIZE_CURSOR);
                    setCursor(ResizeCursor);
                }
                if (((!dragXleft) && (!dragXright)) && ((!dragYup) && (!dragYdown))) {
                    java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR);
                    setCursor(ResizeCursor);
                }
            }

            @Override
            public void mouseDragged(MouseEvent click) {
                if (dragXright && dragYdown) {
                    UI.setSize(screenWidth+click.getXOnScreen()-mouseX, screenHeight+click.getYOnScreen() - mouseY);
                } else if (dragXleft && dragYdown) {
                    if (screenWidth+mouseX-click.getXOnScreen() > 781) {
                        UI.setLocation(screenX+click.getXOnScreen()-mouseX, screenY);
                        UI.setSize(screenWidth+mouseX-click.getXOnScreen(), screenHeight+click.getYOnScreen() - mouseY);
                    } else {
                        UI.setSize(screenWidth, screenHeight+click.getYOnScreen() - mouseY);
                    }
                } else if (dragXright) {
                    UI.setSize(screenWidth+click.getXOnScreen()-mouseX, screenHeight);
                } else if (dragYdown) {
                    UI.setSize(screenWidth, screenHeight+click.getYOnScreen() - mouseY);
                } else if (dragXleft) {
                    if (screenWidth+mouseX-click.getXOnScreen() > 781) {
                        UI.setLocation(screenX+click.getXOnScreen()-mouseX, screenY);
                        UI.setSize(screenWidth+mouseX-click.getXOnScreen(), screenHeight);
                    }
                }
                resizing = true;
                resizeTimer.start();
            }
        };

        MouseListener windowResizeClick = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent click) {
                mouseX = click.getXOnScreen();// - UI.getX() ;
                mouseY = click.getYOnScreen();// - UI.getY() ;
                screenWidth = UI.getWidth();
                screenHeight = UI.getHeight();
                screenX =UI.getX();
                screenY =UI.getY();
            }

            @Override
            public void mouseReleased(MouseEvent click) {
                java.awt.Cursor ResizeCursor = new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR);
                setCursor(ResizeCursor);
            }
        };
        this.addMouseListener(windowResizeClick);
        this.addMouseMotionListener(windowResize);

        //top right corner buttons
        MouseListener minClick = new MouseAdapter() {
            @Override
            @SuppressWarnings("static-access")
            public void mouseClicked(MouseEvent click) {
                UI.setExtendedState(ArmadilloUI.ICONIFIED);
            }
        };
        minMenuItem.addMouseListener(minClick);
        MouseListener maxClick = new MouseAdapter() {
            @Override
            @SuppressWarnings({"static-access", "static-access", "static-access"})
            public void mouseClicked(MouseEvent click) {
                if (UI.getExtendedState()==ArmadilloUI.MAXIMIZED_BOTH) {
                    UI.setExtendedState(ArmadilloUI.NORMAL);
                } else {
                    UI.setExtendedState(ArmadilloUI.MAXIMIZED_BOTH);
                }
            }
        };
        maxMenuItem.addMouseListener(maxClick);
        MouseListener closeClick = new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent click) {
                UI.exitMenuItemActionPerformed(null);
            }
        };
        closeMenuItem.addMouseListener(closeClick);
    }

    //adds all key listeners need for this class
    private void addKeyListeners() {

    }

    
    /**
     * name is misleading and deprecated, actual repaints the main UI
     */
    public void changesize() {
	    //this.setSize(this.getWidth()+constantsize,this.getHeight()+1);
	    //constantsize = -(constantsize);
	    //this.setSize(this.getWidth()-1,this.getHeight()-1);
            UI.setExtendedState(UI.getExtendedState());
            UI.repaint();
    }


    /**
     *starts playing the current song from the beginning
     */
    public void playCurrentSong() {
        TrackObject track = playingList.getCurrentTrack();
        if (track != null) {
            playTimer.stop();
            Play(track);
            playTimer.start();
        }        
    }

    /**
     * checks whether the file type of a file is of an acceptable format
     * @param file
     * @return
     */
    public boolean CheckFileType(String file) {
        if (file.toString().endsWith(".mp3")){
            return true;
        }
        if (file.toString().endsWith(".wav")){
            return true;
        }
        if (file.toString().endsWith(".ogg")){
            return true;
        }
        if (file.toString().endsWith(".m4a")){
            return true;
        }
        return false;
    }

    /**
     * updates the text next to the progress bar with the passed text
     * @param text
     */
    public void updateProgressBarText(String text) {
        if (text.length() > 30) {
            text= text.substring(0, 30);
        }
        progressBarLabel.setText(text);
    }

    /**
     * update the current value of the progress bar (0-100)
     * @param percentage
     */
    public void updateProgressBar(int percentage) {
        progressBar.setValue(percentage);
        if (percentage == 0) {
            progressBar.setValue(0);
            this.updateProgressBarText("Idle");
        }
    }

    /**
     * Calls the function in playingList to remove a track
     * @param trackID
     */
    public void removeTrackFromPlayingList(int trackID) {
        playingList.removeTrack(trackID);
    }

    /**
     *clear the graph
     */
    public void ClearGraph() {
        drawGraph1.clear();
    }

    /**
     * reloads all the information about a track above the seek bar
     */
    public void refreshCurrentTrackInfo() {
        long time = musicPlayer.GetTimeElapsed();        
        time /= 1000;        
        charlength = (trackPanel.getWidth())/10;
        endsub = startsub + charlength;
        if (endsub>TrackText.length()) {
            endsub = TrackText.length();
        }
            if (TrackText.length() > charlength) {
            if (updown) {
                if (endsub < TrackText.length()) {
                    startsub++;
                    endsub++;
                } else {
                    updown=false;
                }
            } else {
                if (startsub > 0) {
                    startsub--;
                    endsub--;
                } else {
                    updown = true;
                }
            }

            trackLabel.setText(UIUtilities.FormatTime(time)+"/"+UIUtilities.FormatTime(durationSeconds)+"   " +TrackText.substring(startsub,endsub));
        } else {
            trackLabel.setText(UIUtilities.FormatTime(time)+"/"+UIUtilities.FormatTime(durationSeconds)+"   " +TrackText);
        }
    }

    /**
     *resets the shuffle parameter in the playinglist class
     */
    public void clearShuffle() {
        randomMenuItem.setState(false);
        shuffle = false;
        playingList.setShuffle(false);
    }

    /**
     * sets the text of stats of the selected playlist table
     * @param text
     */
    public void setTableLabelText(String text) {
        selectedPlaylistStatsLabel.setText(text);
    }

    /**
     * sets the text of the title above the selected playlist table
     * @param text
     */
    public void setTableTitleText(String text) {
        if (text.equals("")) {
            this.selectedPlaylistLabel.setText("Selected Playlist: None");
        } else if (text.equals("Loading tracks...")) {
            selectedPlaylistLabel.setText(text);
        } else {
            selectedPlaylistLabel.setText("Selected Playlist: "+text);
        }
    }

    /**
     * sets the text of the now playing stats label
     * @param text
     */
    public void setListLabelText(String text) {
        nowPlayingStatsLabel.setText(text);
    }

    /**
     * returns the scroll bar object for the playing list scrollpane
     * @return
     */
    public JScrollBar getPlayingListScrollBar() {
        return nowPlayingScrollPane.getVerticalScrollBar();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonPanel = new javax.swing.JPanel();
        rewindButton = new javax.swing.JButton();
        playPauseButton = new javax.swing.JButton();
        fastForwardButton = new javax.swing.JButton();
        volPanel = new javax.swing.JPanel();
        volSlider = new javax.swing.JSlider();
        volLabel = new javax.swing.JLabel();
        trackPanel = new javax.swing.JPanel();
        seekBarSlider = new javax.swing.JSlider();
        trackLabel = new javax.swing.JLabel();
        fileNavPanel = new javax.swing.JPanel();
        fileNavScrollPane = new javax.swing.JScrollPane();
        navTree = new armadilloUI.NavTree();
        selectPlaylistPanel = new javax.swing.JPanel();
        selectedPlaylistScrollPane = new javax.swing.JScrollPane();
        selectedPlaylistTable = new armadilloUI.PlaylistTable();
        selectedPlaylistLabel = new javax.swing.JLabel();
        nowPlayingPanel = new javax.swing.JPanel();
        nowPlayingScrollPane = new javax.swing.JScrollPane();
        playingList = new armadilloUI.PlayingList();
        nowPlayingLabel = new javax.swing.JLabel();
        selectedPlaylistStatsPanel = new javax.swing.JPanel();
        selectedPlaylistStatsLabel = new javax.swing.JLabel();
        nowPlayingStatsPanel = new javax.swing.JPanel();
        nowPlayingStatsLabel = new javax.swing.JLabel();
        graphPanel = new javax.swing.JPanel();
        drawGraph1 = new armadilloUI.DrawGraph();
        searchTextField = new javax.swing.JTextField();
        progressBar = new javax.swing.JProgressBar();
        progressBarLabel = new javax.swing.JLabel();
        shuffleCheckBox = new javax.swing.JCheckBox();
        loopAllCheckBox = new javax.swing.JCheckBox();
        loopOneCheckBox = new javax.swing.JCheckBox();
        logoLabel = new javax.swing.JLabel();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        importSongMenuItem = new javax.swing.JMenuItem();
        importFolderMenuItem = new javax.swing.JMenuItem();
        openExitSeparator = new javax.swing.JSeparator();
        exitMenuItem = new javax.swing.JMenuItem();
        EditMenu = new javax.swing.JMenu();
        loopCurrentMenuItem = new javax.swing.JRadioButtonMenuItem();
        loopAllMenuItem = new javax.swing.JRadioButtonMenuItem();
        loopRandomSeparator = new javax.swing.JSeparator();
        randomMenuItem = new javax.swing.JCheckBoxMenuItem();
        randomPreferencesSeparator = new javax.swing.JSeparator();
        skinsMenu = new javax.swing.JMenu();
        skinWater = new javax.swing.JRadioButtonMenuItem();
        skinWind = new javax.swing.JRadioButtonMenuItem();
        skinFire = new javax.swing.JRadioButtonMenuItem();
        skinEarth = new javax.swing.JRadioButtonMenuItem();
        skinCustom = new javax.swing.JRadioButtonMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        preferencesMenuItem = new javax.swing.JMenuItem();
        moodsMenu = new javax.swing.JMenu();
        createBeatPlaylistMenuItem = new javax.swing.JMenuItem();
        beatMoodSeparator = new javax.swing.JSeparator();
        createMoodPlaylistMenuItem = new javax.swing.JMenuItem();
        playlistsMenu = new javax.swing.JMenu();
        createPlaylistMenuItem = new javax.swing.JMenuItem();
        addSongToPlaylistMenu = new javax.swing.JMenu();
        createDeleteSeparator1 = new javax.swing.JPopupMenu.Separator();
        editPlaylistMenuItem = new javax.swing.JMenuItem();
        createDeleteSeparator = new javax.swing.JSeparator();
        deletePlaylistMenuItem = new javax.swing.JMenuItem();
        removeSongFromPlaylistMenu = new javax.swing.JMenu();
        trackMenu = new javax.swing.JMenu();
        editTrackMenuItem = new javax.swing.JMenuItem();
        manualBPMMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        helpMenuItem = new javax.swing.JMenuItem();
        aboutMenuItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Armadillo Music Player");
        setBackground(java.awt.Color.white);
        setMinimumSize(new java.awt.Dimension(780, 460));

        buttonPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(64, 64, 64)));

        rewindButton.setText("|<");
        rewindButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                rewindButtonActionPerformed(evt);
            }
        });

        playPauseButton.setText(">");
        playPauseButton.setFocusCycleRoot(true);
        playPauseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playPauseButtonActionPerformed(evt);
            }
        });

        fastForwardButton.setText(">|");
        fastForwardButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fastForwardButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout buttonPanelLayout = new javax.swing.GroupLayout(buttonPanel);
        buttonPanel.setLayout(buttonPanelLayout);
        buttonPanelLayout.setHorizontalGroup(
            buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(buttonPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(rewindButton, javax.swing.GroupLayout.PREFERRED_SIZE, 52, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(playPauseButton, javax.swing.GroupLayout.PREFERRED_SIZE, 46, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(fastForwardButton, javax.swing.GroupLayout.DEFAULT_SIZE, 52, Short.MAX_VALUE)
                .addContainerGap())
        );
        buttonPanelLayout.setVerticalGroup(
            buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(buttonPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(playPauseButton, javax.swing.GroupLayout.PREFERRED_SIZE, 43, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(fastForwardButton, javax.swing.GroupLayout.PREFERRED_SIZE, 43, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(rewindButton, javax.swing.GroupLayout.PREFERRED_SIZE, 43, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        volPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(64, 64, 64)));
        volPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        volSlider.setMaximum(90);
        volSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                volSliderStateChanged(evt);
            }
        });
        volPanel.add(volSlider, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 20, 90, -1));

        volLabel.setText("Volume");
        volPanel.add(volLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 10, -1, -1));

        trackPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(64, 64, 64)));

        seekBarSlider.setMaximum(1000);
        seekBarSlider.setValue(0);
        seekBarSlider.setAutoscrolls(true);
        seekBarSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                seekBarSliderStateChanged(evt);
            }
        });

        trackLabel.setText("No Song Currently Playing");

        javax.swing.GroupLayout trackPanelLayout = new javax.swing.GroupLayout(trackPanel);
        trackPanel.setLayout(trackPanelLayout);
        trackPanelLayout.setHorizontalGroup(
            trackPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(trackPanelLayout.createSequentialGroup()
                .addGap(20, 20, 20)
                .addComponent(trackLabel))
            .addGroup(trackPanelLayout.createSequentialGroup()
                .addComponent(seekBarSlider, javax.swing.GroupLayout.DEFAULT_SIZE, 528, Short.MAX_VALUE)
                .addGap(2, 2, 2))
        );
        trackPanelLayout.setVerticalGroup(
            trackPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(trackPanelLayout.createSequentialGroup()
                .addGroup(trackPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(trackPanelLayout.createSequentialGroup()
                        .addGap(9, 9, 9)
                        .addComponent(trackLabel))
                    .addGroup(trackPanelLayout.createSequentialGroup()
                        .addGap(19, 19, 19)
                        .addComponent(seekBarSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        fileNavPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        fileNavScrollPane.setBorder(null);
        fileNavScrollPane.setViewportView(navTree);

        javax.swing.GroupLayout fileNavPanelLayout = new javax.swing.GroupLayout(fileNavPanel);
        fileNavPanel.setLayout(fileNavPanelLayout);
        fileNavPanelLayout.setHorizontalGroup(
            fileNavPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(fileNavScrollPane, javax.swing.GroupLayout.PREFERRED_SIZE, 202, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        fileNavPanelLayout.setVerticalGroup(
            fileNavPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(fileNavScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 285, Short.MAX_VALUE)
        );

        selectPlaylistPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        selectedPlaylistScrollPane.setBorder(null);

        selectedPlaylistTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        selectedPlaylistScrollPane.setViewportView(selectedPlaylistTable);

        selectedPlaylistLabel.setText("Selected Playlist: None");

        javax.swing.GroupLayout selectPlaylistPanelLayout = new javax.swing.GroupLayout(selectPlaylistPanel);
        selectPlaylistPanel.setLayout(selectPlaylistPanelLayout);
        selectPlaylistPanelLayout.setHorizontalGroup(
            selectPlaylistPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(selectedPlaylistScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 332, Short.MAX_VALUE)
            .addGroup(selectPlaylistPanelLayout.createSequentialGroup()
                .addContainerGap(88, Short.MAX_VALUE)
                .addComponent(selectedPlaylistLabel)
                .addContainerGap(96, Short.MAX_VALUE))
        );
        selectPlaylistPanelLayout.setVerticalGroup(
            selectPlaylistPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(selectPlaylistPanelLayout.createSequentialGroup()
                .addComponent(selectedPlaylistLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(selectedPlaylistScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 270, Short.MAX_VALUE))
        );

        nowPlayingPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        nowPlayingScrollPane.setBorder(null);

        playingList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        nowPlayingScrollPane.setViewportView(playingList);

        nowPlayingLabel.setText("Now Playing");

        javax.swing.GroupLayout nowPlayingPanelLayout = new javax.swing.GroupLayout(nowPlayingPanel);
        nowPlayingPanel.setLayout(nowPlayingPanelLayout);
        nowPlayingPanelLayout.setHorizontalGroup(
            nowPlayingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(nowPlayingScrollPane, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 270, Short.MAX_VALUE)
            .addGroup(nowPlayingPanelLayout.createSequentialGroup()
                .addGap(86, 86, 86)
                .addComponent(nowPlayingLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 89, Short.MAX_VALUE)
                .addGap(95, 95, 95))
        );
        nowPlayingPanelLayout.setVerticalGroup(
            nowPlayingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(nowPlayingPanelLayout.createSequentialGroup()
                .addComponent(nowPlayingLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(nowPlayingScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 270, Short.MAX_VALUE))
        );

        selectedPlaylistStatsPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(64, 64, 64)));

        selectedPlaylistStatsLabel.setText("        0 Tracks");

        javax.swing.GroupLayout selectedPlaylistStatsPanelLayout = new javax.swing.GroupLayout(selectedPlaylistStatsPanel);
        selectedPlaylistStatsPanel.setLayout(selectedPlaylistStatsPanelLayout);
        selectedPlaylistStatsPanelLayout.setHorizontalGroup(
            selectedPlaylistStatsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, selectedPlaylistStatsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(selectedPlaylistStatsLabel)
                .addContainerGap(232, Short.MAX_VALUE))
        );
        selectedPlaylistStatsPanelLayout.setVerticalGroup(
            selectedPlaylistStatsPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(selectedPlaylistStatsLabel)
        );

        nowPlayingStatsPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(64, 64, 64)));
        nowPlayingStatsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        nowPlayingStatsLabel.setText("0 Tracks");
        nowPlayingStatsPanel.add(nowPlayingStatsLabel, new org.netbeans.lib.awtextra.AbsoluteConstraints(41, 1, -1, -1));

        graphPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        drawGraph1.setBorder(null);

        javax.swing.GroupLayout drawGraph1Layout = new javax.swing.GroupLayout(drawGraph1);
        drawGraph1.setLayout(drawGraph1Layout);
        drawGraph1Layout.setHorizontalGroup(
            drawGraph1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 820, Short.MAX_VALUE)
        );
        drawGraph1Layout.setVerticalGroup(
            drawGraph1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 80, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout graphPanelLayout = new javax.swing.GroupLayout(graphPanel);
        graphPanel.setLayout(graphPanelLayout);
        graphPanelLayout.setHorizontalGroup(
            graphPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(drawGraph1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        graphPanelLayout.setVerticalGroup(
            graphPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(drawGraph1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        drawGraph1.init();

        searchTextField.setText("Enter Search Terms");
        searchTextField.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                searchTextFieldMouseClicked(evt);
            }
        });
        searchTextField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                searchTextFieldKeyPressed(evt);
            }
            public void keyReleased(java.awt.event.KeyEvent evt) {
                searchTextFieldKeyReleased(evt);
            }
        });

        progressBar.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                progressBarMousePressed(evt);
            }
        });

        progressBarLabel.setText("Idle");
        progressBarLabel.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                progressBarLabelMousePressed(evt);
            }
        });

        shuffleCheckBox.setText("Shuffle");
        shuffleCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                shuffleCheckBoxActionPerformed(evt);
            }
        });

        loopAllCheckBox.setText("Loop All");
        loopAllCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loopAllCheckBoxActionPerformed(evt);
            }
        });

        loopOneCheckBox.setText("Loop One");
        loopOneCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loopOneCheckBoxActionPerformed(evt);
            }
        });

        fileMenu.setForeground(new java.awt.Color(64, 64, 64));
        //new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/babyIcon2.gif"))
        //fileMenu.setIcon(new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/babyIcon2.gif")));
        ImageIcon myFileIcon = new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/babyIcon.gif"));
        Image FileImg = myFileIcon.getImage();
        Image NewFileImg = FileImg.getScaledInstance(20, 20,  java.awt.Image.SCALE_SMOOTH);
        //newIcon = new ImageIcon(newimg);
        ImageIcon myFileIconNew = new ImageIcon(NewFileImg);
        fileMenu.setIcon(myFileIconNew);
        fileMenu.setText("File");

        importSongMenuItem.setText("Import file(s) to library");
        importSongMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                importSongMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(importSongMenuItem);

        importFolderMenuItem.setText("Import folder(s) to library");
        importFolderMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                importFolderMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(importFolderMenuItem);
        fileMenu.add(openExitSeparator);

        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        EditMenu.setText("Options");

        loopCurrentMenuItem.setText("Loop current song");
        loopCurrentMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loopCurrentMenuItemActionPerformed(evt);
            }
        });
        EditMenu.add(loopCurrentMenuItem);

        loopAllMenuItem.setText("Loop all");
        loopAllMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loopAllMenuItemActionPerformed(evt);
            }
        });
        EditMenu.add(loopAllMenuItem);
        EditMenu.add(loopRandomSeparator);

        randomMenuItem.setText("Random play order");
        randomMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                randomMenuItemActionPerformed(evt);
            }
        });
        EditMenu.add(randomMenuItem);
        EditMenu.add(randomPreferencesSeparator);

        skinsMenu.setText("Skins");

        if (theme.equals("Water"))
        skinWater.setSelected(true);
        else if (theme.equals("Wind"))
        skinWind.setSelected(true);
        else if (theme.equals("Fire"))
        skinFire.setSelected(true);
        else if (theme.equals("Earth"))
        skinEarth.setSelected(true);
        else if (theme.equals("Custom"))
        skinCustom.setSelected(true);
        skinWater.setText("Water");
        skinWater.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                skinsMenuItemActionPerformed(evt, "Water");
            }
        });
        skinsMenu.add(skinWater);
        skinsButtonGroup.add(skinWater);

        skinWind.setText("Wind");
        skinWind.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                skinsMenuItemActionPerformed(evt, "Wind");
            }
        });
        skinsMenu.add(skinWind);
        skinsButtonGroup.add(skinWind);

        skinFire.setText("Fire");
        skinFire.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                skinsMenuItemActionPerformed(evt, "Fire");
            }
        });
        skinsMenu.add(skinFire);
        skinsButtonGroup.add(skinFire);

        skinEarth.setText("Earth");
        skinEarth.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                skinsMenuItemActionPerformed(evt, "Earth");
            }
        });
        skinsMenu.add(skinEarth);
        skinsButtonGroup.add(skinEarth);

        skinCustom.setText("Custom");
        skinCustom.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                skinsMenuItemActionPerformed(evt, "Custom");
            }
        });
        skinsMenu.add(skinCustom);
        skinsButtonGroup.add(skinCustom);

        skinsMenuItemActionPerformed(null, theme);

        EditMenu.add(skinsMenu);
        EditMenu.add(jSeparator1);

        preferencesMenuItem.setText("Preferences");
        preferencesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                preferencesMenuItemActionPerformed(evt);
            }
        });
        EditMenu.add(preferencesMenuItem);

        menuBar.add(EditMenu);

        moodsMenu.setText("Moods");

        createBeatPlaylistMenuItem.setText("Create Beat playlist");
        createBeatPlaylistMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                createBeatPlaylistMenuItemActionPerformed(evt);
            }
        });
        moodsMenu.add(createBeatPlaylistMenuItem);
        moodsMenu.add(beatMoodSeparator);

        createMoodPlaylistMenuItem.setText("Create Mood playlist");
        createMoodPlaylistMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                createMoodPlaylistMenuItemActionPerformed(evt);
            }
        });
        moodsMenu.add(createMoodPlaylistMenuItem);

        menuBar.add(moodsMenu);

        playlistsMenu.setText("Playlists");

        createPlaylistMenuItem.setText("Create playlist");
        createPlaylistMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                createPlaylistMenuItemActionPerformed(evt);
            }
        });
        playlistsMenu.add(createPlaylistMenuItem);

        addSongToPlaylistMenu.setText("Add selected song to playlist");

        try {
            ArrayList<String> playlistsNames = db.getPlaylistsNames();
            for (int i=0; i<playlistsNames.size(); i++)
            {
                JMenuItem playlistItem = new JMenuItem(playlistsNames.get(i));
                AddRemovePlaylistListener playlistListener = new AddRemovePlaylistListener("add",db.getPlayListID(playlistsNames.get(i)),null,selectedPlaylistTable, this);
                playlistItem.addActionListener(playlistListener);
                addSongToPlaylistMenu.add(playlistItem);
            }
        } catch (Exception ex) {
            Logger.getLogger(PlaylistTable.class.getName()).log(Level.SEVERE, null, ex);
        }

        playlistsMenu.add(addSongToPlaylistMenu);
        playlistsMenu.add(createDeleteSeparator1);

        editPlaylistMenuItem.setText("Edit selected playlist");
        editPlaylistMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editPlaylistMenuItemActionPerformed(evt);
            }
        });
        playlistsMenu.add(editPlaylistMenuItem);
        playlistsMenu.add(createDeleteSeparator);

        deletePlaylistMenuItem.setText("Delete selected playlist");
        deletePlaylistMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                deletePlaylistMenuItemActionPerformed(evt);
            }
        });
        playlistsMenu.add(deletePlaylistMenuItem);

        removeSongFromPlaylistMenu.setText("Remove selected song from playlist");

        try {
            ArrayList<String> playlistsNames = db.getPlaylistsNames();
            for (int i=0; i<playlistsNames.size(); i++)
            {
                JMenuItem playlistItem = new JMenuItem(playlistsNames.get(i));
                AddRemovePlaylistListener playlistListener = new AddRemovePlaylistListener("remove", db.getPlayListID(playlistsNames.get(i)),selectedPlaylistTable.getCurrentSelection(),selectedPlaylistTable, this);
                playlistItem.addActionListener(playlistListener);
                removeSongFromPlaylistMenu.add(playlistItem);
            }
        } catch (Exception ex) {
            Logger.getLogger(PlaylistTable.class.getName()).log(Level.SEVERE, null, ex);
        }

        playlistsMenu.add(removeSongFromPlaylistMenu);

        menuBar.add(playlistsMenu);

        trackMenu.setText("Track");

        editTrackMenuItem.setText("Edit Track");
        editTrackMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editTrackMenuItemActionPerformed(evt);
            }
        });
        trackMenu.add(editTrackMenuItem);

        manualBPMMenuItem.setText("Manually Calculate BPM");
        manualBPMMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                manualBPMMenuItemActionPerformed(evt);
            }
        });
        trackMenu.add(manualBPMMenuItem);

        menuBar.add(trackMenu);

        helpMenu.setText("Help");

        helpMenuItem.setText("Help");
        helpMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(helpMenuItem);

        aboutMenuItem.setText("About");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);
        menuBar.add(Box.createHorizontalGlue());
        menuBar.add(minMenuItem);
        menuBar.add(maxMenuItem);
        menuBar.add(closeMenuItem);

        setJMenuBar(menuBar);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(buttonPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(trackPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(volPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(fileNavPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(searchTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 204, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(selectPlaylistPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(selectedPlaylistStatsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(nowPlayingPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(nowPlayingStatsPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .addComponent(graphPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, 114, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(progressBarLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 126, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 99, Short.MAX_VALUE)
                        .addComponent(logoLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 88, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 136, Short.MAX_VALUE)
                        .addComponent(loopOneCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(loopAllCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(shuffleCheckBox)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(volPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(trackPanel, 0, 60, Short.MAX_VALUE)
                    .addComponent(buttonPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 60, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(nowPlayingPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(selectPlaylistPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(selectedPlaylistStatsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(nowPlayingStatsPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(fileNavPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(searchTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(graphPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(shuffleCheckBox)
                            .addComponent(loopAllCheckBox)
                            .addComponent(loopOneCheckBox))
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(progressBarLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(logoLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 22, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        // SAVE NOW PLAYING STATE, CLOSE CONNECTIONS WITH DB 
        playingList.saveState();
        selectedPlaylistTable.closeTableDbObject();
        playingList.closePlayingListDbObject();
        db.closeConnection();
        System.exit(0);
    }//GEN-LAST:event_exitMenuItemActionPerformed

    private void seekBarSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_seekBarSliderStateChanged

    }//GEN-LAST:event_seekBarSliderStateChanged

    private void playPauseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playPauseButtonActionPerformed
        if (empty==false) {
            Pause();
        }
    }//GEN-LAST:event_playPauseButtonActionPerformed

    private void volSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_volSliderStateChanged
        int vol = volSlider.getValue();
        musicPlayer.SetVolume(vol);
        try {
            util.config("volume",Integer.toString(vol));
        } catch (Exception e) {
            ArmadilloLog.LogError(("Error changing volume - "+e.toString()));
        }
    }//GEN-LAST:event_volSliderStateChanged

    /********************* TODO: USE PLAYLIST INSTEAD OF ARRAYLIST*********/
    //SKIPS TO PREVIOUS SONG ON nowPlayingList
    private void rewindButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_rewindButtonActionPerformed
       //if song is over 5 seconds into playing skips back to the start
        
        //if ((player.GetTimeElapsed() > 5000) || (timeTimer.isRunning() == false)) {
        if ((musicPlayer.GetTimeElapsed() > 5000)) {
            playTimer.stop();
            Play(playingList.getCurrentTrack());
            //MusicFile thissong = (MusicFile) nowPlayingList.getSelectedValue();
            //filename = thissong.Filename;
            //Play();
            playTimer.start();
        //otherwise it skips to the previous song
        } else {
            playTimer.stop();
            Play(playingList.getPreviousTrack());
            playTimer.start();
        }
        
        
    }//GEN-LAST:event_rewindButtonActionPerformed

    //SKIPS TO NEXT SONG ON nowPlayingList
    private void fastForwardButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fastForwardButtonActionPerformed
        TrackObject track = playingList.getNextTrack();
        if (track != null) {
            playTimer.stop();
            Play(track);
            playTimer.start();
        }
        //playNextSong();
    }//GEN-LAST:event_fastForwardButtonActionPerformed

    private void createPlaylistMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_createPlaylistMenuItemActionPerformed
             // if a track is selected in the table, create new playlist with this track added automatically
            new CreatePlaylistUI("custom", this).setVisible(true);
    }//GEN-LAST:event_createPlaylistMenuItemActionPerformed

    private void helpMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_helpMenuItemActionPerformed
        new HelpUI().setVisible(true);
    }//GEN-LAST:event_helpMenuItemActionPerformed

    private void loopCurrentMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loopCurrentMenuItemActionPerformed
        if (loopOne == false) {
            loopOne = true;
            loopAll = false;
            loopAllMenuItem.setSelected(false);
        } else {
            loopOne = false;
        }
        loopOneCheckBox.setSelected(loopOne);
        loopAllCheckBox.setSelected(loopAll);
        playingList.setLoopSingle(loopOne);
    }//GEN-LAST:event_loopCurrentMenuItemActionPerformed

    private void loopAllMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loopAllMenuItemActionPerformed
        if (loopAll == false) {
            loopAll = true;
            loopOne = false;
            loopCurrentMenuItem.setSelected(false);
        } else {
            loopAll = false;
        }
        loopAllCheckBox.setSelected(loopAll);
        loopOneCheckBox.setSelected(loopOne);
        playingList.setLoopAll(loopAll);
    }//GEN-LAST:event_loopAllMenuItemActionPerformed

    private void randomMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_randomMenuItemActionPerformed
        if (shuffle == true) {
            shuffle = false;
        } else {
            shuffle = true;
        }
        shuffleCheckBox.setSelected(shuffle);
        playingList.setShuffle(shuffle);
    }//GEN-LAST:event_randomMenuItemActionPerformed

    private void importSongMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_importSongMenuItemActionPerformed
        if ( is != null && is.isAlive())
            new DialogUI(24).setVisible(true);
        else if ( bpmthread != null && bpmthread.isAlive())
            new DialogUI(25).setVisible(true);
        else
            importSongs("file");
    }

    private void setBG( Component[] jc, Color  bg, int depth )
      {
            for( int i = 0; i < jc.length; i++ ) {
                  Component c = jc[i];
                  if( c instanceof Container )// {
                        setBG( ((Container)c).getComponents(), bg, depth );
                  c.setBackground( bg );
            }
      }

    private void importSongs(String what) {
        if (filesToImport.size() == 0)
        {
            JFileChooser importFile = new JFileChooser();
            Color bg = new java.awt.Color(32,32,32);
            try {
                String FilepanelColor = util.config("customPanelBackgroundColour");
                if (util.config("skin").equals("Custom")) {
                    bg = new java.awt.Color(Integer.parseInt(FilepanelColor));
                }
            } catch (Exception e) {e.printStackTrace();}
            
            setBG( importFile.getComponents(), bg, 0 );
            importFile.repaint();
            importFile.setFileFilter(filter);
            importFile.setMultiSelectionEnabled(true);
            if (what.equals("folder"))
            {
                importFile.setDialogTitle("Import Folder(s)");
                importFile.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            }
            else if (what.equals("file"))
            {
                importFile.setDialogTitle("Import File(s)");
                importFile.setFileSelectionMode(JFileChooser.FILES_ONLY);
            }
            int returnVal = importFile.showOpenDialog(this);
            if (returnVal == JFileChooser.APPROVE_OPTION)
                filesSelected = importFile.getSelectedFiles();
        }
        for (int i=0; i<filesSelected.length; i++)
        {
            if (filesSelected[i].exists())
            {
                if (!(filesSelected[i].isDirectory()))
                    filesToImport.add(filesSelected[i]);
                else
                    this.getDirContents(filesSelected[i]);
        
            }
        }

        /* NOT NEEDED AS CHECKED BY THE IMPORT THREAD
         boolean importMood = true;
        try {
            String importMoodValue = util.config("moodOnImport");

            if (importMoodValue == null) {
                util.config("moodOnImport","true");
            } else {
                importMood = Boolean.parseBoolean(importMoodValue);
            }
        } catch (Exception e) {
            logError(("Error reading config file - "+e.toString()));
        }*/

        // START NEW THREAD TO ADD TRACKS TO LIBRARY
        is = new ImportSong("is", filesToImport, this);

        File[] none = {};
        filesSelected = none;
        filesToImport.clear();
    }//GEN-LAST:event_importSongMenuItemActionPerformed

    /**
     *  imports the given files
     * @param newFiles
     */
    public void importThis(File[] newFiles)
    {
        for (int i=0; i<newFiles.length; i++)
        filesToImport.add(newFiles[i]);
        importSongs("file");
    }

    /**
     * refreshes the menu bar, used when new playlist are made, or a playlist is deleted.
     */
    public void refreshMenuBar() {
        addSongToPlaylistMenu.removeAll();
        try {
            ArrayList<String> playlistsNames = db.getPlaylistsNames();
            for (int i=0; i<playlistsNames.size(); i++)
            {
                JMenuItem playlistItem = new JMenuItem(playlistsNames.get(i));
                AddRemovePlaylistListener playlistListener = new AddRemovePlaylistListener("add",db.getPlayListID(playlistsNames.get(i)),null,selectedPlaylistTable, this);
                playlistItem.addActionListener(playlistListener);
                addSongToPlaylistMenu.add(playlistItem);
            }
        } catch (Exception ex) {
            Logger.getLogger(PlaylistTable.class.getName()).log(Level.SEVERE, null, ex);
        }
        removeSongFromPlaylistMenu.removeAll();
        try {
            ArrayList<String> playlistsNames = db.getPlaylistsNames();
            for (int i=0; i<playlistsNames.size(); i++)
            {
                JMenuItem playlistItem = new JMenuItem(playlistsNames.get(i));
                AddRemovePlaylistListener playlistListener = new AddRemovePlaylistListener("remove", db.getPlayListID(playlistsNames.get(i)),selectedPlaylistTable.getCurrentSelection(),selectedPlaylistTable, this);
                playlistItem.addActionListener(playlistListener);
                removeSongFromPlaylistMenu.add(playlistItem);
            }
        } catch (Exception ex) {
            Logger.getLogger(PlaylistTable.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void createBeatPlaylistMenuItemActionPerformed(java.awt.event.ActionEvent evt) {                                                           
        new CreatePlaylistUI("beat", this).setVisible(true);
    }

    private void createMoodPlaylistMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_createMoodPlaylistMenuItemActionPerformed
        new CreatePlaylistUI("mood", this).setVisible(true);
        //reload nav tree
    }//GEN-LAST:event_createMoodPlaylistMenuItemActionPerformed

    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        new AboutUI().setVisible(true);
    }

    private void deletePlaylistMenuItemActionPerformed(java.awt.event.ActionEvent evt) {                                                       
        if (navTree.getSelectedList() > 2)
        {
            /*boolean othersAlive = false;
            if (bpmthread != null)// || UI.getImportThread() != null)
                othersAlive = true;
            if (othersAlive)
            {
                if (bpmthread.isAlive())// || UI.getImportThread().isAlive())
                {
                    System.out.println("BPM alive");
                    bpmthread.suspendBPMAnalysis();
                    //UI.getImportThread().suspendImporting();
                }
            }*/
            try {
                new DeleteUI("playlist", navTree.getSelectedList(),this, navTree, selectedPlaylistTable, playingList).setVisible(true);
            }
            catch (java.lang.NullPointerException e) {
                new DialogUI(1).setVisible(true);
            }
        }
        else
            new DialogUI(2).setVisible(true);
    }
    
    private void editPlaylistMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editPlaylistMenuItemActionPerformed
            try {
                if (navTree.getSelectedList() > 0)
                    new CreatePlaylistUI(navTree.getSelectedList(), this).setVisible(true);
                else
                    new DialogUI(1).setVisible(true);
            }
            catch (java.lang.NullPointerException nullex) {
                new DialogUI(1).setVisible(true);
            }
    }//GEN-LAST:event_editPlaylistMenuItemActionPerformed

    private void searchTextFieldMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_searchTextFieldMouseClicked
        if (searchTextField.getText().equals("Enter Search Terms")) {
            searchTextField.setText("");
        }
    }//GEN-LAST:event_searchTextFieldMouseClicked

    @SuppressWarnings("static-access")
    private void searchTextFieldKeyReleased(java.awt.event.KeyEvent evt) {
        if (evt.getKeyChar() == KeyEvent.VK_ENTER) {
            navTree.selectLastSearched();
            String[] keywords = searchTextField.getText().split(" ");
            int playlistID = db.search(keywords);
            selectedPlaylistTable.viewPlaylist("playlist", playlistID,"Last searched");
        }
        if (searchTextField.getText().equals("")) {
            searchTextField.setText("Enter Search Terms");
        }
        //Add code to start search thread (and kill previous search thread) for search terms
    }                                           

    private void preferencesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_preferencesMenuItemActionPerformed
        //code which opens the preferences window
        PreferencesUI preferencesUI = new PreferencesUI(util, this);
        preferencesUI.setVisible(true);
    }//GEN-LAST:event_preferencesMenuItemActionPerformed

    private void loopOneCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loopOneCheckBoxActionPerformed
        if (loopOneCheckBox.isSelected()) {
            loopOne = true;
         
            loopAllCheckBox.setSelected(false);

            loopAllMenuItem.setSelected(false);
        } else {
            loopOne = false;
         
        }
        playingList.setLoopSingle(loopOne);
        loopCurrentMenuItem.setSelected(loopOne);
    }//GEN-LAST:event_loopOneCheckBoxActionPerformed

    private void loopAllCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loopAllCheckBoxActionPerformed
        if (loopAllCheckBox.isSelected()) {
            loopAll = true;
         
            loopOneCheckBox.setSelected(false);
            loopCurrentMenuItem.setSelected(false);
        } else {
            loopAll = false;
        }
        loopAllMenuItem.setSelected(loopAll);
        playingList.setLoopAll(loopAll);
    }//GEN-LAST:event_loopAllCheckBoxActionPerformed

    private void shuffleCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_shuffleCheckBoxActionPerformed
        if (shuffle == true) {
            shuffle = false;
        } else {
            shuffle = true;
        }
        randomMenuItem.setSelected(shuffle);
        playingList.setShuffle(shuffle);
    }//GEN-LAST:event_shuffleCheckBoxActionPerformed

    private void editTrackMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editTrackMenuItemActionPerformed
        int[] trackIDs = selectedPlaylistTable.getCurrentSelection();
        if (trackIDs.length > 0) {
            for (int i=0; i<trackIDs.length; i++) {
                new EditTrackUI(trackIDs[i], selectedPlaylistTable, navTree,playingList, this).setVisible(true);
            }
        } else {
            //Error no track selected
            new DialogUI(8).setVisible(true);
        }
    }//GEN-LAST:event_editTrackMenuItemActionPerformed

    private void manualBPMMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_manualBPMMenuItemActionPerformed
        //code to do TapBPM
        int[] trackIDs = selectedPlaylistTable.getCurrentSelection();
        for (int i=0; i<trackIDs.length; i++) {
            new TapBPMUI(trackIDs[i],false,this).setVisible(true);
        }
    }//GEN-LAST:event_manualBPMMenuItemActionPerformed

    private void progressBarLabelMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_progressBarLabelMousePressed
        importShowPopup(evt);

    }//GEN-LAST:event_progressBarLabelMousePressed

    private void progressBarMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_progressBarMousePressed
        importShowPopup(evt);
    }//GEN-LAST:event_progressBarMousePressed

    private void importFolderMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_importFolderMenuItemActionPerformed
        if ( is != null && is.isAlive())
            new DialogUI(24).setVisible(true);
        else if ( bpmthread != null && bpmthread.isAlive())
            new DialogUI(25).setVisible(true);
        else
            importSongs("folder");
    }//GEN-LAST:event_importFolderMenuItemActionPerformed

    private void searchTextFieldKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_searchTextFieldKeyPressed
        if (searchTextField.getText().equals("Enter Search Terms")) {
            searchTextField.setText("");
        }
    }//GEN-LAST:event_searchTextFieldKeyPressed

    /**
     * changes the current skin
     * @param evt
     * @param skinID
     */
    public void skinsMenuItemActionPerformed(java.awt.event.ActionEvent evt, String skinID) {
        ArmadilloLog.LogOperation("Changing skin");
        try {
            // write changes to config file
            util.config("skin", skinID);
        } catch (Exception ex) {
           ArmadilloLog.LogError(("Error changing skins - "+ex.toString()));
        }
        java.awt.Color bgcolor = new java.awt.Color(0, 0, 0);
        java.awt.Color blackcolor = new java.awt.Color(0, 0, 0);
        java.awt.Color fontcolor = new java.awt.Color(255, 255, 255);
	java.awt.Color scrollcolor = new java.awt.Color(32, 32, 32);
	java.awt.Color scrollcolor2 = new java.awt.Color(64, 64, 64);
        Font font = new Font("URW Gothic L",Font.PLAIN,12);
        //ImageIcon logo1 = new ImageIcon("graphics/logo.gif");
        //ImageIcon logo2 = new ImageIcon("graphics/logo2.gif");
        //ImageIcon logo3 = new ImageIcon("graphics/logo3.gif");

        if (skinID.equals("Water")) {
		bgcolor = new java.awt.Color(0, 0, 0);
		fontcolor = new java.awt.Color(0, 255, 255);
		UIManager.put("text", new java.awt.Color(0,255,255));
                drawGraph1.skin(fontcolor, java.awt.Color.white, java.awt.Color.red);
                scrollcolor = new java.awt.Color(32,32,32);
                UIManager.put("nimbusBase", scrollcolor);
                this.repaint();
                                try {
                    UIManager.setLookAndFeel(UIManager.getLookAndFeel());
                }
                catch (Exception e){
                    ArmadilloLog.LogError(("Error altering UI look and feel - "+e.toString()));
                }
                //logoLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/logo3copy.gif")));
               // fileMenu.setIcon(new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/logo3copy.gif")));
               // logoLabel.setIcon(logo3);
	} else if (skinID.equals("Wind")) {
		bgcolor = new java.awt.Color(0, 0, 0);
		fontcolor = new java.awt.Color(255, 255, 255);
		UIManager.put("text", new java.awt.Color(255,255,255));
                drawGraph1.skin(fontcolor, java.awt.Color.white, java.awt.Color.red);
                scrollcolor = new java.awt.Color(32,32,32);
                UIManager.put("nimbusBase", scrollcolor);
                this.repaint();
                                try {
                    UIManager.setLookAndFeel(UIManager.getLookAndFeel());
                }
                catch (Exception e){
                    ArmadilloLog.LogError(("Error altering UI look and feel - "+e.toString()));
                }
	} else if (skinID.equals("Fire")) {
		bgcolor = new java.awt.Color(0, 0, 0);
		fontcolor = new java.awt.Color(255, 0, 0);
		UIManager.put("text", new java.awt.Color(255,0,0));
                drawGraph1.skin(fontcolor, java.awt.Color.white, java.awt.Color.red);
                scrollcolor = new java.awt.Color(32,32,32);
                UIManager.put("nimbusBase", scrollcolor);
                this.repaint();
                                try {
                    UIManager.setLookAndFeel(UIManager.getLookAndFeel());
                }
                catch (Exception e){
                    ArmadilloLog.LogError(("Error altering UI look and feel - "+e.toString()));
                }
                //logoLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/logo2copy.gif")));
             //   fileMenu.setIcon(new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/logo2copy.gif")));
              //  logoLabel.setIcon(logo2);
	} else if (skinID.equals("Earth")) {
		bgcolor = new java.awt.Color(0, 0, 0);
		fontcolor = new java.awt.Color(0, 255, 0);
		UIManager.put("text", new java.awt.Color(0,255,0));
                drawGraph1.skin(fontcolor, java.awt.Color.white, java.awt.Color.red);
                scrollcolor = new java.awt.Color(32,32,32);
                UIManager.put("nimbusBase", scrollcolor);
                this.repaint();
                try {
                    UIManager.setLookAndFeel(UIManager.getLookAndFeel());
                }
                catch (Exception e){
                    ArmadilloLog.LogError(("Error altering UI look and feel - "+e.toString()));
                }
                //logoLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/logocopy.gif")));
            //    fileMenu.setIcon(new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/logocopy.gif")));
              //l  logoLabel.setIcon(logo1);
	} else if (skinID.equals("Custom")) {                
                try {
                    String textfont = util.config("customFontColour");
                    if (textfont != null) {
                        fontcolor =new java.awt.Color(Integer.parseInt(textfont));
                        UIManager.put("text", new java.awt.Color(Integer.parseInt(textfont)));
                        changesize();
                    }

                    String textbgcolor = util.config("customBackgroundColour");
                    if (textbgcolor != null) {
                        bgcolor =new java.awt.Color(Integer.parseInt(textbgcolor));
                        UIManager.put("control", bgcolor);
                        changesize();
                    }
                    String textscrollcolor = util.config("customPanelBackgroundColour");
                    if (textscrollcolor != null) {
                        scrollcolor =new java.awt.Color(Integer.parseInt(textscrollcolor));
                try {
                    UIManager.setLookAndFeel(UIManager.getLookAndFeel());
                }
                catch (Exception e){
                    ArmadilloLog.LogError(("Error altering UI look and feel - "+e.toString()));
                }
                        UIManager.put("control", bgcolor);
                        UIManager.put("nimbusBase", scrollcolor);
                       // UIManager.put("nimbusBase", bgcolor);
                        changesize();

                    }
                    String textBorderColour = util.config("customBorderColour");
                    if (textBorderColour != null) {
                        scrollcolor2 =new java.awt.Color(Integer.parseInt(textBorderColour));
                        changesize();
                    }
                    
                    bgcolor =new java.awt.Color(Integer.parseInt(util.config("customBackgroundColour")));
                    scrollcolor =new java.awt.Color(Integer.parseInt(util.config("customPanelBackgroundColour")));
                    scrollcolor2 =new java.awt.Color(Integer.parseInt(util.config("customBorderColour")));
                    drawGraph1.skin(fontcolor, java.awt.Color.white, java.awt.Color.red);
                } catch (Exception ex) {
                    ArmadilloLog.LogError(("Error altering UI look and feel - "+ex.toString()));
                }
        }

	//Sets theme + the basics and then updates
        UIManager.put("nimbusBase", scrollcolor);
        UIManager.put("control", bgcolor);
        this.repaint();
        playPauseButton.setBackground(fontcolor);
        playPauseButton.setForeground(blackcolor);
        fastForwardButton.setBackground(fontcolor);
        fastForwardButton.setForeground(blackcolor);
        rewindButton.setBackground(fontcolor);
        rewindButton.setForeground(blackcolor);
	fileNavPanel.setBackground(scrollcolor2);
        fileNavPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
	navTree.setBackground(scrollcolor);
	playingList.setBackground(scrollcolor);
	selectedPlaylistTable.setBackground(scrollcolor);
	selectPlaylistPanel.setBackground(scrollcolor2);
	nowPlayingPanel.setBackground(scrollcolor2);
	trackPanel.setBackground(scrollcolor);
        trackPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
	volPanel.setBackground(scrollcolor);
        volPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
	buttonPanel.setBackground(scrollcolor);
        buttonPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
	graphPanel.setBackground(scrollcolor);
        graphPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
        fileMenu.setForeground(fontcolor);
        menuBar.setBackground(scrollcolor);
        menuBar.setForeground(scrollcolor);
	EditMenu.setForeground(fontcolor);
	helpMenu.setForeground(fontcolor);
	moodsMenu.setForeground(fontcolor);
	playlistsMenu.setForeground(fontcolor);
        trackMenu.setForeground(fontcolor);
	volLabel.setFont(font);
	trackLabel.setFont(font);
	selectedPlaylistStatsLabel.setFont(font);
        selectedPlaylistStatsPanel.setBackground(scrollcolor);
        selectedPlaylistStatsPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
	selectedPlaylistLabel.setFont(font);
	nowPlayingStatsLabel.setFont(font);
        nowPlayingStatsPanel.setBackground(scrollcolor);
        nowPlayingStatsPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
        nowPlayingLabel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
	nowPlayingLabel.setFont(font);
        nowPlayingPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));

	playingList.setFont(font);
        navTree.setFont(font);
        searchTextField.setFont(font);
        searchTextField.setBackground(scrollcolor);
        searchTextField.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
	selectedPlaylistTable.setFont(font);
	selectPlaylistPanel.setFont(font);
        selectPlaylistPanel.setBorder(javax.swing.BorderFactory.createLineBorder(scrollcolor2));
        searchTextField.setFont(font);
        progressBarLabel.setFont(font);
        minMenuItem.setForeground(scrollcolor);
        maxMenuItem.setForeground(scrollcolor);
        closeMenuItem.setForeground(scrollcolor);
        UIManager.put("nimbusBase", scrollcolor);
        UIManager.put("control", bgcolor);
        this.repaint();
   //     logoLabel.setIcon(null);
        
        changesize();
    }

    /**
     * clears the current track
     */
    public void ClearTrack() {
        musicPlayer.playbin.setState(State.NULL);
        this.trackLabel.setText("No Song Currently Playing");
        this.playPauseButton.setText(">");
        musicPlayer.Stop();
        empty=true;
    }
    
    /**
     * plays the given track object
     * @param track
     */
    public void Play(TrackObject track) {
        ArmadilloLog.LogOperation("Playing Track");
        //stop previous song playing etc.
        playTimer.stop();
        musicPlayer.Stop();
        this.currentTrack = track;

        if (track != null) {
            //Equalizer
            boolean EQ = false;
            try {
                String volEq = util.config("volEq");
                if (volEq == null) {
                    util.config("volEq","false");
                } else {
                    EQ = Boolean.parseBoolean(volEq);
                }
            } catch (Exception e) {
                ArmadilloLog.LogError(("Error reading config file - "+e.toString()));
            }
            if (EQ) {

                float newDec = new Float(track.getDec());
                if ((oldDec >1) && (newDec >1)) {
                    float percentageChange = (oldDec/newDec);
                    float newVol = volSlider.getValue()*percentageChange;
                    volSlider.setValue(Math.round(newVol));
                    musicPlayer.SetVolume(Math.round(newVol));
                }
                if ((newDec > 1)) {
                    oldDec=newDec;
                }
            }


            empty=false;
            musicPlayer.playbin.setState(State.READY);
            try {
                drawGraph1.clear();

                final File filename = new File(track.getFileName());

                // throw an error if file not found
                if (!(filename.exists()))
                {
                    new FileNotFoundUI(track.getTrackID(), this).setVisible(true);
                    return;
                }
                musicPlayer.Play(filename);
                musicPlayer.Seek(0);

                playPauseButton.setText("||");
                double mins = track.getLength();
                duration = (long) mins*1000;
                durationSeconds =(long) mins;
                TrackText = track.getPlayingData();
                this.refreshCurrentTrackInfo();
                startsub=0;
                endsub = (trackPanel.getWidth()-150)/10;
                if (endsub>TrackText.length()) {
                    endsub = TrackText.length();
                }
                updown = true;

                String configGraphDetail;
                configGraphDetail = util.config("graphDetail");
                if (configGraphDetail != null) {
                    this.graphDetail = Integer.parseInt(configGraphDetail);
                } else {
                    //default graph detail is lowest, 640 (can be 640,1024,1920)
                    this.graphDetail = 640;
                    util.config("graphDetail", "640");
                }
                final int idx = ++gidx;
                gsync.set(idx);

                if (!startingUP) {
                    gt = new Thread(new Runnable()
                    {
                        @Override
                        public void run()
                        {
                            GraphData gd = new GraphData(filename.toString(), idx, gsync);

                            if (gd.readData(graphDetail) == 1)
                            {
                                drawGraph1.update(gd.getData());
                            }

                        }
                    });
                    gt.start();

                } else {
                        GraphData gd;
                        gd = new GraphData(filename.toString(), idx, gsync);

                        if (gd.readData(640) == 1)
                        {
                            drawGraph1.update(gd.getData());
                        }
                        startingUP = false;
                }
            } catch (Exception error) {
               ArmadilloLog.LogError(("Error loading track from database - "+error.toString()));
            }

            playTimer.start();
        } else {
            musicPlayer.playbin.setState(State.NULL);
            this.ClearTrack();
        }

    }

    /**
     * pauses the media player, or resumes depending on current state
     */
    public void Pause() {
        if (currentTrack != null) {
            if (seekBarSlider.getValue()==1000){
                TrackObject next = playingList.getNextTrack();
                if (next != null) {
                    Play(next);
                }
            }
            if (playTimer.isRunning()) {
                playTimer.stop();
                musicPlayer.playbin.pause();
                playPauseButton.setText(">");
            } else {
                playTimer.start();
                musicPlayer.playbin.play();
                //musicPlayer.playbin.play();
                playPauseButton.setText("||");
            }
        }
    }

    /**
     *stop the media player
     */
    public void Stop() {
        musicPlayer.Stop();
        playTimer.stop();
    }

    /**
     *calls the function to reload songs with passed ID inside the selected playlist table
     * @param changedTrackID
     */
    public void reloadTable(int changedTrackID) {
        selectedPlaylistTable.reload(changedTrackID);
    }


    /**
     * calls function to reload entire selected playlist contents
     */
    public void reloadTable() {
        selectedPlaylistTable.reload();
    }

    /**
     * calls function to reload entire navigation tree
     */
    public void reloadNavTree() {
        navTree.reload();
    }

    /**
     *calls function which only reload ths bpm section of the navigation tree
     */
    public void reloadNavTreeBPM() {
        navTree.reloadBPM();
    }

    /**
     * calls function of the playing list to reload it
     */
    public void reloadPlayingList() {
        playingList.reload();
    }

    /**
     * returns the table object used in the main ui
     * @return
     */
    public PlaylistTable getTable()
    {
        return this.selectedPlaylistTable;
    }

    /**
     * return reference to the now playing list
     * @return
     */
    public PlayingList getList()
    {
        return this.playingList;
    }

    /**
     *return reference to the nav tree
     * @return
     */
    public NavTree getTree()
    {
        return this.navTree;
    }

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenu EditMenu;
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JMenu addSongToPlaylistMenu;
    private javax.swing.JSeparator beatMoodSeparator;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JMenuItem createBeatPlaylistMenuItem;
    private javax.swing.JSeparator createDeleteSeparator;
    private javax.swing.JPopupMenu.Separator createDeleteSeparator1;
    private javax.swing.JMenuItem createMoodPlaylistMenuItem;
    private javax.swing.JMenuItem createPlaylistMenuItem;
    private javax.swing.JMenuItem deletePlaylistMenuItem;
    public armadilloUI.DrawGraph drawGraph1;
    private javax.swing.JMenuItem editPlaylistMenuItem;
    private javax.swing.JMenuItem editTrackMenuItem;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JButton fastForwardButton;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JPanel fileNavPanel;
    private javax.swing.JScrollPane fileNavScrollPane;
    private javax.swing.JPanel graphPanel;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JMenuItem helpMenuItem;
    private javax.swing.JMenuItem importFolderMenuItem;
    private javax.swing.JMenuItem importSongMenuItem;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JLabel logoLabel;
    private javax.swing.JCheckBox loopAllCheckBox;
    private javax.swing.JRadioButtonMenuItem loopAllMenuItem;
    private javax.swing.JRadioButtonMenuItem loopCurrentMenuItem;
    private javax.swing.JCheckBox loopOneCheckBox;
    private javax.swing.JSeparator loopRandomSeparator;
    private javax.swing.JMenuItem manualBPMMenuItem;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenu moodsMenu;
    private armadilloUI.NavTree navTree;
    private javax.swing.JLabel nowPlayingLabel;
    private javax.swing.JPanel nowPlayingPanel;
    private javax.swing.JScrollPane nowPlayingScrollPane;
    private javax.swing.JLabel nowPlayingStatsLabel;
    private javax.swing.JPanel nowPlayingStatsPanel;
    private javax.swing.JSeparator openExitSeparator;
    private javax.swing.JButton playPauseButton;
    private armadilloUI.PlayingList playingList;
    private javax.swing.JMenu playlistsMenu;
    private javax.swing.JMenuItem preferencesMenuItem;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel progressBarLabel;
    private javax.swing.JCheckBoxMenuItem randomMenuItem;
    private javax.swing.JSeparator randomPreferencesSeparator;
    private javax.swing.JMenu removeSongFromPlaylistMenu;
    private javax.swing.JButton rewindButton;
    private javax.swing.JTextField searchTextField;
    private javax.swing.JSlider seekBarSlider;
    private javax.swing.JPanel selectPlaylistPanel;
    private javax.swing.JLabel selectedPlaylistLabel;
    private javax.swing.JScrollPane selectedPlaylistScrollPane;
    private javax.swing.JLabel selectedPlaylistStatsLabel;
    private javax.swing.JPanel selectedPlaylistStatsPanel;
    private armadilloUI.PlaylistTable selectedPlaylistTable;
    private javax.swing.JCheckBox shuffleCheckBox;
    private javax.swing.JRadioButtonMenuItem skinCustom;
    private javax.swing.JRadioButtonMenuItem skinEarth;
    private javax.swing.JRadioButtonMenuItem skinFire;
    private javax.swing.JRadioButtonMenuItem skinWater;
    private javax.swing.JRadioButtonMenuItem skinWind;
    private javax.swing.JMenu skinsMenu;
    private javax.swing.JLabel trackLabel;
    private javax.swing.JMenu trackMenu;
    private javax.swing.JPanel trackPanel;
    private javax.swing.JLabel volLabel;
    private javax.swing.JPanel volPanel;
    private javax.swing.JSlider volSlider;
    // End of variables declaration//GEN-END:variables

    private void importShowPopup(MouseEvent evt) {
        //if (evt.isPopupTrigger() && !(progressBarLabel.getText().equals("Idle"))) {
        if ((evt.getButton()==3) && !(progressBarLabel.getText().equals("Idle"))) {
            popupImport.show(evt.getComponent(), evt.getX(), evt.getY());
        }
        //else if (evt.isPopupTrigger() && progressBarLabel.getText().equals("Idle")) {
        else if ((evt.getButton()==3) && progressBarLabel.getText().equals("Idle")) {
            if (db.anyTracksToAnalyse())
                popupStartImport.show(evt.getComponent(), evt.getX(), evt.getY());
        }

    }

    // used when importing on startup
    void setbpmThread(ImportSongBPMThread t) {
        this.bpmthread = t;
    }

    /**
     * resume the analyse of tracks
     */
    public void resumeAnalysing()
    {
        ArmadilloLog.LogOperation("Resuming analysis");
        this.bpmthread = new ImportSongBPMThread("bpmthread", this);
    }

    private void getDirContents(File file) {
        File files[] = file.listFiles();
        for (int i = 0, n = files.length; i < n; i++)
        {
            if (files[i].getName().endsWith("mp3") ||
                files[i].getName().endsWith("wav") ||
                files[i].getName().endsWith("ogg") ||
                files[i].getName().endsWith("m4a"))
            {
                filesToImport.add(files[i]);
            }
            if (files[i].isDirectory())
                getDirContents(files[i]);
        }
    }

    ImportSongBPMThread getbpmThread() {
        return this.bpmthread;
    }

    ImportSong getImportThread() {
        return this.is;
    }
}
