/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/
package armadilloUI;

class Bin
{
    private int hz_count[] = new int[50];
    private double hz_dec[] = new double[50];
    private double max_dec[] = new double[50];
    private double deci = 0;
    private int count = 0;
    private double max = -1;
    private boolean Imax = false;
    private boolean Imax_dec[] = new boolean[50];
    private double Adec  = 0;
    private int countD = 0;

    public int[] ghz()
    {
        return hz_count;
    }

    public double[] gdec()
    {
        return max_dec;
    }

    public Bin()
    {
        for (int i = 0; i <= 49; i++)
        {
            hz_count[i] = 1;
            hz_dec[i]   = 1;
            max_dec[i]  = -99999;
            Imax_dec[i] = false;
        }
    }
    
    public double getAverageDec()
    {
        return Adec/countD;
    }
    
    public void inc(int hz_bin, double d)
    {
        hz_count[hz_bin]++;
        hz_dec[hz_bin] += d;
        if (d > 0)
        {
            countD++;
            Adec += d;    
        }
        if (Imax_dec[hz_bin] == false) 
        {
            max_dec[hz_bin] = d;
            Imax_dec[hz_bin] = true;
        }else
        {
        if (d > max_dec[hz_bin]) max_dec[hz_bin] = d;
        }
   }
    
    public int get(int location)
    {
        return hz_count[location];
    }
    
    public double get_d(int location)
    {
        return hz_dec[location]/(double)hz_count[location];
    }
    
    public void store_dec(double Decibels)
    {
        if (Imax == false) 
        {
            max = Decibels;
            Imax = true;
        } else
        {
            if (Decibels > max) max = Decibels;
            deci += Decibels;
            count++;
        }
    }
    
    public double max_d()
    {
        return max;
    }
    
    public double get_d()
    {
        return deci/(double)count;
    }

    public double get_maxd(int location)
    {
        return max_dec[location];
    }
    
    public void normalize(int speed)
    {
        switch (speed)
        {
            case 1 :
            {
                for (int i = 0; i <= 49; i++)
                {
                    hz_count[i] = hz_count[i] * 4;
                }
                break;
            }
            case 2 :
            {
                for (int i = 0; i <= 49; i++)
                {
                    hz_count[i] = hz_count[i] * 2;
                }
                break;
            }
        }
    }

    public void normalize()
    {
        count = 0;
        int loopCount = 0;
        for (int i = 0; i <=49 ; i++)
        {
            count += hz_count[i];
        }
        //System.out.println(count);
        while ((count < 45000) || (count > 55000))
        {
            if (count < 45000) 
            {
                double temp = 45000 / (double)count;
                increase(temp);
            }
            if (count > 55000)
            {
                double temp = 55000 / (double)count;
                increase(temp);
            }
            double factor = 1.02;
            double factor2 = 0.98;
            loopCount++;
            if (loopCount == 9) break;
            if (count < 45000)
            {
                increase(factor);
                factor = factor - 0.002;
                factor2 = factor2 + 0.002;
            }
            if (count > 55000)
            {
                decrease(factor2);
                factor = factor - 0.002;
                factor2 = factor2 + 0.002;
            }
            count = 0;
            for (int i = 0; i <=49 ; i++)
            {
                count += hz_count[i];
            }
        }
    }

    public void increase(double factor)
    {
        for (int i = 0; i <=49 ; i++)
        {
            double temp = (hz_count[i] * factor);
            hz_count[i] = (int)temp;
        }
    }

    public void decrease(double factor)
    {
        for (int i = 0; i <=49 ; i++)
        {
            double temp = (hz_count[i] * factor);
            hz_count[i] = (int)temp;
        }
    }
}
