/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/
package armadilloUI;

/**
 * Stores a complex number
 * @author steve/lev
 */
public class Complex
{
    /* 
    private fields holding the complex number in two fields: real and imaginary
    */
    private final double re;
    private final double im;
    
    /**
     *     Constructor to set the field values
     * @param real
     * @param imag
     */
    public Complex(double real, double imag)
    {
        re = real;
        im = imag;
    }
        
    /**
     *     Getters which return the values of the complex number, maintains the private
     *nature of the fields because another class cannot change the values without
     * using the method in this class
     * @return
     */
    public double get_re() {return re;}
    /**
     *     Getters which return the values of the complex number, maintains the private
    nature of the fields because another class cannot change the values without
    using the method in this class
     * @return
     */
    public double get_im() {return im;}
     

    /**
     *     Minus one complex number from another
     * @param b
     * @return
     */
    public Complex minus(Complex b)
    {
        Complex a = this ;
        double real = a.re - b.re;
        double imag = a.im - b.im;
        return new Complex (real , imag);
    }
    
    /**
     *     Add two complex numbers together
     * @param b
     * @return
     */
    public Complex plus(Complex b)
    {
        Complex a = this ;
        double real = a.re + b.re;
        double imag = a.im + b.im;
        return new Complex (real , imag);
     }

    /**
     *     Times two complex numbers together
     * @param b
     * @return
     */
    public Complex times(Complex b)
    {
        Complex a = this ;
        double real = a.re * b.re - a.im * b.im;
        double imag = a.re * b.im + a.im * b.re;
        return new Complex (real,imag) ;
     }
}
