/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * TODO - what happens if bpm is not calculated for a song??
 */

package armadilloUI;

import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.table.DefaultTableModel.*;
import javax.swing.*;
/**
 *  Creating Playlists UI and functionality
 * @author Kate
 */
public class CreatePlaylistUI extends javax.swing.JFrame {

    private UIUtilities util = new UIUtilities();
    private ArrayList<Integer> newPlaylistIDs = new ArrayList<Integer>();
    private ArrayList<String[]> libraryData = new ArrayList<String[]>();
    private ArmadilloDB db;
    private final boolean finished = true;
    private String type;
    private int playlistID;
    private DefaultListModel listModel;
    private ArmadilloUI UI;
    private String oldName;
    private java.awt.Color bgcolor = new java.awt.Color(0, 0, 0);
    private java.awt.Color fontcolor = new java.awt.Color(255, 255, 255);
    private java.awt.Color scrollcolor = new java.awt.Color(32, 32, 32);
    private java.awt.Color scrollcolor2 = new java.awt.Color(64, 64, 64);
    // Mood variables
    private double[] Dtemp = new double[50];
    private int[] Htemp = new int[50];
    private int score = 0;
    private int scored = 0;
    private String Panelcolor;
    private int addedSongs = 0;
    private int limit;
    private boolean allowed;
    private boolean isAnalysing;

    /** Used when creating new playlist
     * @param whatKind 
     * @param ui
     */
    public CreatePlaylistUI(String whatKind, ArmadilloUI ui) {
        ArmadilloLog.LogOperation("Constructing Create-Playlist UI");
        try {
            db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            ArmadilloLog.LogError("Create Playlist UI cannot access DB - " + ex.toString());
            Logger.getLogger(CreatePlaylistUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        this.UI = ui;
        type = whatKind;
        //System.out.println(type);
        initComponents();
        this.setLocationRelativeTo(null);
        allSongsTable.setUp();
        allSongsTable.viewPlaylist("Last played", 1,"playlist");
        navTree1.setSelectedTable(allSongsTable);
        navTree1.setPlayingList(null);
        navTree1.setUIandFilter(null, null);
        //navTree.setEditable(true);
        navTree1.setUp();

        try{
            Panelcolor = util.config("customPanelBackgroundColour");
            if (util.config("skin").equals("Custom")){
                java.awt.Color treecol = new java.awt.Color(Integer.parseInt(Panelcolor));
                navTree1.setBackground(treecol);
                jTextFieldPlaylistName.setBackground(treecol);
                jTextFieldBPM.setBackground(treecol);
                jTextFieldMaxTracks.setBackground(treecol);
                jListSelectedItems.setBackground(treecol);
                allSongsTable.setBackground(treecol);
            }
            else {
                java.awt.Color panelColl = new java.awt.Color(32,32,32);
                navTree1.setBackground(panelColl);
                jTextFieldPlaylistName.setBackground(panelColl);
                jTextFieldBPM.setBackground(panelColl);
                jTextFieldMaxTracks.setBackground(panelColl);
                jListSelectedItems.setBackground(panelColl);
                allSongsTable.setBackground(panelColl);
            }
        }
        catch(Exception e){
            ArmadilloLog.LogError("Create Playlist UI cannot load config file - " + e.toString());
            e.printStackTrace();
        }

    }

    /** Used when editing a playlist
     * @param whichPlaylist
     * @param ui
     */
    public CreatePlaylistUI(int whichPlaylist, ArmadilloUI ui) {
        ArmadilloLog.LogOperation("Constructing create playlist UI");
        try {
            db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            Logger.getLogger(CreatePlaylistUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        type = "edit";
        playlistID = whichPlaylist;
        initComponents();
        this.setLocationRelativeTo(null);
        allSongsTable.setUp();
        allSongsTable.viewPlaylist("all songs", 0,"all songs");
        ArrayList<String[]> contents = db.getPlaylistContents(playlistID);
        for (int i=0; i<contents.size(); i++)
        {
            // ADD ID OF TRACKS TO THE LIST OF TRACKS TO BE ADDED
            newPlaylistIDs.add(Integer.parseInt(contents.get(i)[6]));
            // ADD SELECTED TRACK TO THE LIST ON THE RIGHT
            Object track = contents.get(i)[0];
            listModel.addElement(track);
        }
        this.UI = ui;
        oldName = db.getPlayListName(playlistID);
        jTextFieldPlaylistName.setText(oldName);
        this.setTitle("Edit Playlist");
        RecalculateBPM();
        navTree1.setSelectedTable(allSongsTable);
        navTree1.setPlayingList(null);
        navTree1.setUIandFilter(null, null);
        //navTree.setEditable(true);
        navTree1.setUp();
    }

    /** Used when double clicked on track to create new playlist with this track
     * @param trackID
     */
    public CreatePlaylistUI(int trackID) {
        ArmadilloLog.LogOperation("Constructing create playlist UI");
        try {
            db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            Logger.getLogger(CreatePlaylistUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        type = "custom";
        initComponents();
        this.setLocationRelativeTo(null);
        allSongsTable.setUp();
        allSongsTable.viewPlaylist("all songs", 0,"all songs");
        String[] trackData = db.getTrack(trackID);
        // ADD ID OF TRACK TO THE LIST OF TRACKS TO BE ADDED
        newPlaylistIDs.add(Integer.parseInt(trackData[6]));
        // ADD TRACK TITLE TO THE LIST ON THE RIGHT
        Object track = trackData[0];
        listModel.addElement(track);
        this.setTitle("Edit Playlist");
        RecalculateBPM();
    }

    /**
     * recalculated the average BPM text field
     */
    public void RecalculateBPM() {
        ArmadilloLog.LogOperation("Recalculating BPM");
        int runningTotal = 0;
        if (newPlaylistIDs.size() > 0) {
            int n = 0;
            for (int i =0; i <newPlaylistIDs.size(); i++) {
                int BPM=0;
                BPM = db.getBPM(newPlaylistIDs.get(i));
                if (BPM != -1) {
                    runningTotal = runningTotal + BPM;
                    n=n+1;
                }
            }
            int average;
            if (n == 0) {
                average = 0;
            } else {
                average = Math.round(runningTotal/n);
            }
            jTextFieldBPM.setText(Integer.toString(average));
        } else {
            jTextFieldBPM.setText("0");
        }
    }
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPaneLibrary = new javax.swing.JScrollPane();
        allSongsTable = new armadilloUI.PlaylistTable();
        jScrollPaneSelectedItems = new javax.swing.JScrollPane();
        jListSelectedItems = new javax.swing.JList();
        jButtonAdd = new javax.swing.JButton();
        jButtonRemove = new javax.swing.JButton();
        jButtonClear = new javax.swing.JButton();
        jButtonAccept = new javax.swing.JButton();
        jTextFieldPlaylistName = new javax.swing.JTextField();
        jTextFieldBPM = new javax.swing.JTextField();
        jLabelAvBPM = new javax.swing.JLabel();
        generateButton = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        navTree1 = new armadilloUI.NavTree();
        jButtonMoveUp = new javax.swing.JButton();
        jButtonMoveDown = new javax.swing.JButton();
        playlistNameLabel = new javax.swing.JLabel();
        moodButton = new javax.swing.JButton();
        jTextFieldMaxTracks = new javax.swing.JTextField();
        jLabelNumberOfTracksToAdd = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Create New Playlist");
        setIconImage(util.getWindowIcon());
        setMinimumSize(new java.awt.Dimension(772, 373));

        jScrollPaneLibrary.setBackground(new java.awt.Color(32, 32, 32));
        jScrollPaneLibrary.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(64, 64, 64)));
        jScrollPaneLibrary.setAutoscrolls(true);
        jScrollPaneLibrary.setName("jScrollPaneLibrary"); // NOI18N

        allSongsTable.setBackground(new java.awt.Color(32, 32, 32));
        allSongsTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        allSongsTable.setName("allSongsTable"); // NOI18N
        jScrollPaneLibrary.setViewportView(allSongsTable);

        jScrollPaneSelectedItems.setMaximumSize(new java.awt.Dimension(258, 130));
        jScrollPaneSelectedItems.setName("jScrollPaneSelectedItems"); // NOI18N

        jListSelectedItems.setBackground(new java.awt.Color(32, 32, 32));
        jListSelectedItems.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(64, 64, 64)));
        jListSelectedItems.setName("jListSelectedItems"); // NOI18N
        jScrollPaneSelectedItems.setViewportView(jListSelectedItems);
        listModel = new DefaultListModel();
        jListSelectedItems.setModel(listModel);

        jButtonAdd.setText("-->");
        jButtonAdd.setName("jButtonAdd"); // NOI18N
        jButtonAdd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddActionPerformed(evt);
            }
        });

        jButtonRemove.setText("<--");
        jButtonRemove.setName("jButtonRemove"); // NOI18N
        jButtonRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRemoveActionPerformed(evt);
            }
        });

        jButtonClear.setText("Clear");
        jButtonClear.setName("jButtonClear"); // NOI18N
        jButtonClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonClearActionPerformed(evt);
            }
        });

        jButtonAccept.setText("Accept");
        jButtonAccept.setName("jButtonAccept"); // NOI18N
        jButtonAccept.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAcceptActionPerformed(evt);
            }
        });

        jTextFieldPlaylistName.setForeground(new java.awt.Color(120, 115, 115));
        jTextFieldPlaylistName.setText("Enter Playlist Name");
        jTextFieldPlaylistName.setName("jTextFieldPlaylistName"); // NOI18N

        jTextFieldBPM.setForeground(new java.awt.Color(120, 115, 115));
        jTextFieldBPM.setName("jTextFieldBPM"); // NOI18N
        if ((type.equals("custom")) || (type.equals("mood")))
        jTextFieldBPM.setVisible(false);

        jLabelAvBPM.setText("Average BPM:");
        jLabelAvBPM.setName("jLabelAvBPM"); // NOI18N

        generateButton.setText("Add BPM Tracks");
        generateButton.setName("generateButton"); // NOI18N
        if (type.equals("custom"))
        generateButton.setVisible(false); // NOI18N
        if (type.equals("beat")) {
            //System.out.println("test");
            generateButton.setVisible(true);
        } else {
            if (type.equals("mood"))
            generateButton.setVisible(false);
        }
        generateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                generateButtonActionPerformed(evt);
            }
        });

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        navTree1.setName("navTree1"); // NOI18N
        jScrollPane1.setViewportView(navTree1);

        jButtonMoveUp.setText("/\\");
            jButtonMoveUp.setName("jButtonMoveUp"); // NOI18N
            jButtonMoveUp.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jButtonMoveUpActionPerformed(evt);
                }
            });

            jButtonMoveDown.setText("\\/");
            jButtonMoveDown.setName("jButtonMoveDown"); // NOI18N
            jButtonMoveDown.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jButtonMoveDownActionPerformed(evt);
                }
            });

            playlistNameLabel.setText("Playlist Name:");
            playlistNameLabel.setName("playlistNameLabel"); // NOI18N

            moodButton.setText("Add Similar Mood Tracks");
            if (type.equals("custom"))
            moodButton.setVisible(false); // NOI18N
            if (type.equals("beat"))
            moodButton.setVisible(false);
            if (type.equals("mood"))
            moodButton.setVisible(true);
            moodButton.setName("moodButton"); // NOI18N
            moodButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    moodButtonActionPerformed(evt);
                }
            });

            jTextFieldMaxTracks.setForeground(new java.awt.Color(120, 115, 115));
            jTextFieldMaxTracks.setText("20");
            jTextFieldMaxTracks.setName("jTextFieldMaxTracks"); // NOI18N
            if (type.equals("custom"))
            jTextFieldMaxTracks.setVisible(false);

            jLabelNumberOfTracksToAdd.setText("Maximum song to add:");
            jLabelNumberOfTracksToAdd.setName("jLabelNumberOfTracksToAdd"); // NOI18N

            if (type.equals("custom")) {
                jLabelAvBPM.setVisible(false); // NOI18N
            }
            if (type.equals("beat")) {
                jLabelAvBPM.setText("Average BPM");
                jLabelAvBPM.setVisible(true);
            }
            if (type.equals("mood")) {
                jLabelAvBPM.setVisible(false);
            }
            if (type.equals("custom")) {
                jLabelNumberOfTracksToAdd.setVisible(false); // NOI18N
            }
            if (type.equals("beat")) {
                jLabelNumberOfTracksToAdd.setVisible(true);
            }
            if (type.equals("mood")) {
                jLabelNumberOfTracksToAdd.setVisible(true);
            }

            javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
            getContentPane().setLayout(layout);
            layout.setHorizontalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(layout.createSequentialGroup()
                    .addContainerGap()
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createSequentialGroup()
                            .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 193, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(jScrollPaneLibrary, javax.swing.GroupLayout.DEFAULT_SIZE, 443, Short.MAX_VALUE)
                            .addGap(8, 8, 8)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(jButtonRemove, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 68, Short.MAX_VALUE)
                                .addComponent(jButtonAdd, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 68, Short.MAX_VALUE)
                                .addComponent(jButtonMoveDown, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jButtonMoveUp, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(playlistNameLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(jTextFieldPlaylistName, javax.swing.GroupLayout.PREFERRED_SIZE, 155, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addComponent(jScrollPaneSelectedItems, javax.swing.GroupLayout.PREFERRED_SIZE, 264, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGroup(layout.createSequentialGroup()
                            .addComponent(jLabelNumberOfTracksToAdd)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(jTextFieldMaxTracks, javax.swing.GroupLayout.PREFERRED_SIZE, 44, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(jLabelAvBPM)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(jTextFieldBPM, javax.swing.GroupLayout.PREFERRED_SIZE, 46, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(generateButton)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(moodButton)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 135, Short.MAX_VALUE)
                            .addComponent(jButtonAccept, javax.swing.GroupLayout.PREFERRED_SIZE, 81, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(jButtonClear, javax.swing.GroupLayout.PREFERRED_SIZE, 89, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addContainerGap())
            );

            layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jButtonAccept, jButtonClear});

            layout.setVerticalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(layout.createSequentialGroup()
                    .addContainerGap()
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 328, Short.MAX_VALUE)
                        .addComponent(jScrollPaneLibrary, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 328, Short.MAX_VALUE)
                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jTextFieldPlaylistName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(playlistNameLabel))
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(layout.createSequentialGroup()
                                    .addComponent(jButtonAdd)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(jButtonRemove)
                                    .addGap(52, 52, 52)
                                    .addComponent(jButtonMoveUp)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                    .addComponent(jButtonMoveDown))
                                .addComponent(jScrollPaneSelectedItems, javax.swing.GroupLayout.DEFAULT_SIZE, 289, Short.MAX_VALUE))))
                    .addGap(18, 18, 18)
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelNumberOfTracksToAdd)
                            .addComponent(jTextFieldMaxTracks, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabelAvBPM)
                            .addComponent(jTextFieldBPM, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(generateButton)
                            .addComponent(moodButton))
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jButtonClear)
                            .addComponent(jButtonAccept)))
                    .addContainerGap())
            );

            pack();
        }// </editor-fold>//GEN-END:initComponents

    private void jButtonClearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonClearActionPerformed
        listModel.clear();
        jTextFieldPlaylistName.setText("Enter Playlist Name");
        jTextFieldBPM.setText("enter value");
        newPlaylistIDs.clear();
    }//GEN-LAST:event_jButtonClearActionPerformed

    private void jButtonAcceptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAcceptActionPerformed
        //Makes a new playlist with all the songs in the jlist
        this.pauseOtherThreads();
        String playlistName = jTextFieldPlaylistName.getText();
        if (!(playlistName.equals("")) && (playlistName.equals("Enter Playlist Name") == false)) {
            if (newPlaylistIDs.size() > 0) {
                //If editing a playlist, delete the old one                
                if (type.equals("edit")) {
                    if (playlistName.equals(oldName)) {
                        db.removePlaylist(db.getPlayListID(oldName));
                        db.addPlaylist(playlistName, newPlaylistIDs);
                        UI.getTree().setUp();
                        db.closeConnection();
                        if (isAnalysing)
                            UI.resumeAnalysing();
                        isAnalysing = false;
                        setVisible(false);
                    } else {
                        if (db.getPlayListID(playlistName) > -1) {
                            new DialogUI(22).setVisible(true);
                        } else {
                            db.removePlaylist(db.getPlayListID(oldName));
                            db.addPlaylist(playlistName, newPlaylistIDs);
                            UI.getTree().setUp();
                            db.closeConnection();
                            if (isAnalysing)
                                UI.resumeAnalysing();
                            isAnalysing = false;
                            setVisible(false);
                        }
                    }
                } else if (db.getPlayListID(playlistName) > -1) {
                    new DialogUI(22).setVisible(true);
                } else {
                    db.addPlaylist(playlistName, newPlaylistIDs);
                    UI.getTree().setUp();
                    UI.refreshMenuBar();
                    db.closeConnection();
                    if (isAnalysing)
                        UI.resumeAnalysing();
                    isAnalysing = false;
                    setVisible(false);
                }

            } else {
                new DialogUI(3).setVisible(true);
            }


        }
        else {
            new DialogUI(4).setVisible(true);
        }
    }//GEN-LAST:event_jButtonAcceptActionPerformed

    private void jButtonAddActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddActionPerformed
        int[] tracks = allSongsTable.getSelectedRows();
        int[] actualrows = tracks;
        for (int i=0; i<tracks.length; i++) {
            actualrows[i] =allSongsTable.convertRowIndexToModel(tracks[i]);            
        }
        int[] trackIDs = allSongsTable.getTrackIDs();
        for (int i=0; i<tracks.length; i++)
        {
            // ADD SELECTED TRACK TO THE LIST ON THE RIGHT
            Object track = allSongsTable.getModel().getValueAt(tracks[i], 0);
            //Object track = allSongsTable.getValueAt(tracks[i], 0);
            listModel.addElement(track);
        }
        for (int i =0; i<actualrows.length; i++) {
            // ADD ID OF SELECTED TRACK TO THE LIST OF TRACKS TO BE ADDED
            newPlaylistIDs.add(trackIDs[actualrows[i]]);
        }
        RecalculateBPM();
    }//GEN-LAST:event_jButtonAddActionPerformed

    private void jButtonRemoveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRemoveActionPerformed
        // ALLOW MULTIPLE SELECTION
        int lessBy = 0;
        int[] tracks = jListSelectedItems.getSelectedIndices();
        for (int i=0; i<tracks.length; i++)
        {
            // REMOVE TRACK ID FROM THE LIST OF TRACKS TO BE ADDED
            newPlaylistIDs.remove(tracks[i]-lessBy);
            listModel.remove(tracks[i]-lessBy);
            lessBy++;
        }
        RecalculateBPM();
    }//GEN-LAST:event_jButtonRemoveActionPerformed

    private void generateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_generateButtonActionPerformed
        // ADD CODE TO SEARCH FOR SONGS WITH SIMILAR BPM
        try {
            int maxTracks = Integer.parseInt(jTextFieldMaxTracks.getText())+1;
            try {
                int AverageBPM = Integer.parseInt(jTextFieldBPM.getText());
                ArrayList<Integer> contents = db.getTracksWithMatchingBPM(AverageBPM);
                int j = 1;
                do {
                    ArrayList<Integer> contents2 = db.getTracksWithMatchingBPM(AverageBPM+j);
                    if (contents2.get(0) != -1) {
                        contents.addAll(contents2);
                    }
                    ArrayList<Integer> contents3 = db.getTracksWithMatchingBPM(AverageBPM-j);
                    if (contents3.get(0) != -1) {
                        contents.addAll(contents3);
                    }
                    j++;
                } while((contents.size() < (maxTracks+newPlaylistIDs.size())) && (j<10));
                //ArrayList<Integer> contents = db.getTracks(AverageBPM);
                int added = 0;
                for (int i=0; i<contents.size(); i++) {
                    int trackID = contents.get(i);
                    if (newPlaylistIDs.contains(contents.get(i)) == false) {
                        String[] list = db.getTrack(trackID);
                        if (list != null) {
                            Object track = list[0];
                            //Object track = allSongsTable.getValueAt(tracks[i], 0);
                            listModel.addElement(track);
                            newPlaylistIDs.add(contents.get(i));
                            added++;
                        }
                    }
                    if (added == maxTracks) {
                        i=contents.size()+2;
                    }
                }
                if (added < maxTracks) {
                    new DialogUI(19,added).setVisible(true);
                }
            } catch (NumberFormatException e) {
                new DialogUI(6).setVisible(true);
            }
        } catch (NumberFormatException e) {
            new DialogUI(18).setVisible(true);
        }
        RecalculateBPM();
    }//GEN-LAST:event_generateButtonActionPerformed

    private void jButtonMoveUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonMoveUpActionPerformed
        int[] tracks = jListSelectedItems.getSelectedIndices();
        for (int i = 0 ; i< tracks.length; i++) {
            if (tracks[i] != 0) {
                if (i==0) {
                    //first item to be moved up
                    int temp = newPlaylistIDs.get(tracks[i]);
                    newPlaylistIDs.set(tracks[i], newPlaylistIDs.get(tracks[i]-1));
                    newPlaylistIDs.set(tracks[i]-1, temp);
                    Object temp2 = listModel.get(tracks[i]);
                    listModel.set(tracks[i], listModel.get(tracks[i]-1));
                    listModel.set(tracks[i]-1, temp2);
                    tracks[i] = tracks[i] -1;
                }  else if (tracks[i-1] != tracks[i]-1) {
                    int temp = newPlaylistIDs.get(tracks[i]);
                    newPlaylistIDs.set(tracks[i], newPlaylistIDs.get(tracks[i]-1));
                    newPlaylistIDs.set(tracks[i]-1, temp);
                    Object temp2 = listModel.get(tracks[i]);
                    listModel.set(tracks[i], listModel.get(tracks[i]-1));
                    listModel.set(tracks[i]-1, temp2);
                    if (tracks[i-1] != tracks[i]-1) {
                        tracks[i] = tracks[i] - 1;
                    }
                }

            }
            //newPlaylistIDs
            //listModel
        }
        jListSelectedItems.setSelectedIndices(tracks);
    }//GEN-LAST:event_jButtonMoveUpActionPerformed

    private void jButtonMoveDownActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonMoveDownActionPerformed
        int[] tracks = jListSelectedItems.getSelectedIndices();
        for (int i = tracks.length-1 ; i>-1; i--) {
            if (tracks[i] != newPlaylistIDs.size()-1) {
                if (i==tracks.length-1) {
                    //first item to be moved down
                    int temp = newPlaylistIDs.get(tracks[i]+1);
                    newPlaylistIDs.set(tracks[i]+1, newPlaylistIDs.get(tracks[i]));
                    newPlaylistIDs.set(tracks[i], temp);
                    Object temp2 = listModel.get(tracks[i]+1);
                    listModel.set(tracks[i]+1, listModel.get(tracks[i]));
                    listModel.set(tracks[i], temp2);
                    tracks[i] = tracks[i] + 1;
                }  else if (tracks[i+1] != tracks[i]+1) {
                    int temp = newPlaylistIDs.get(tracks[i]+1);
                    newPlaylistIDs.set(tracks[i+1], newPlaylistIDs.get(tracks[i]));
                    newPlaylistIDs.set(tracks[i], temp);
                    Object temp2 = listModel.get(tracks[i]+1);
                    listModel.set(tracks[i]+1, listModel.get(tracks[i]));
                    listModel.set(tracks[i], temp2);
                    if (tracks[i+1] != tracks[i]+1) {
                        tracks[i] = tracks[i] + 1;
                    }
                }
                
            }
            //newPlaylistIDs
            //listModel
        }
        jListSelectedItems.setSelectedIndices(tracks);
    }//GEN-LAST:event_jButtonMoveDownActionPerformed

    private void moodButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moodButtonActionPerformed
        // scoring etc here
        boolean valid = true;
        String maximumTracksToAdd = jTextFieldMaxTracks.getText();
        int maxSongs = 0;
        try
        {
            maxSongs = Integer.parseInt(maximumTracksToAdd);
        } catch(NumberFormatException e) {
            valid = false;
            new DialogUI(18).setVisible(true);}
        if ((!(maxSongs > 0)) && (valid == true) )
        {
            valid = false;
            new DialogUI(18).setVisible(true);
        }
        if (valid == true)
        {
            score = 0;
            Model model = new Model("temp");
            libraryData = db.getLibrary();
            for (int i=0; i<newPlaylistIDs.size(); i++)
            {
                String[] trackContents = db.getTrack(newPlaylistIDs.get(i));
                Dtemp = db.getFPDoubles(newPlaylistIDs.get(i));
                Htemp = db.getFPInts(newPlaylistIDs.get(i));
                Object artist = trackContents[1];
                Object genre = trackContents[3];
                Object album = trackContents[2];
                int BPM = db.getBPM(newPlaylistIDs.get(i));
                model.addsong(Dtemp, Htemp,artist,genre,album,BPM);
            }
            findaverage(model);
            findaveraged(model);
            score(model,maxSongs,0.1,1);
            if (addedSongs != maxSongs) score(model,maxSongs,0.3,2);
            new DialogUI(19,addedSongs).setVisible(true);
        }
    }//GEN-LAST:event_moodButtonActionPerformed

    // frequency score
    private void findaverage(Model model)
    {
        int songs = 0;
        int averageT = 0;
        for (int i=0; i<newPlaylistIDs.size(); i++)
        {
            songs++;
            Htemp = db.getFPInts(newPlaylistIDs.get(i));
            for (int t = 0; t <= 49 ; t++)
            {
                int c  = model.get(t);
                int c2 = Htemp[t];

                double Tpen  = c * 0.1;
                double Twpen = c * 0.2;
                double Thpen = c * 0.3;

                if (t < 10)
                {
                    if ((c2 - 2.5 < c) && (c2 + 2.5 < c))
                    {
                        averageT += 6;
                    }
                }else
                {
                    if ((c2 > c-Thpen) && (c2 < c+Thpen)) averageT += 2;
                    if ((c2 > c-Twpen) && (c2 < c+Twpen)) averageT += 4;
                    if ((c2 > c-Tpen)  && (c2 < c+Tpen))  averageT += 6;
                }
            }
        score = averageT / songs;
        }
    }

    private void findaveraged(Model model)
    {
        int songs = 0;
        int averageT = 0;
        for (int i=0; i<newPlaylistIDs.size(); i++)
        {
            songs++;
            Htemp = db.getFPInts(newPlaylistIDs.get(i));
            for (int t = 0; t <= 49 ; t++)
            {
                if (Dtemp[t] != -99999)
                {
                    double cd  = model.get_d(t);
                    double cd2 = Dtemp[t];

                    double dTpen  =  cd * 0.1;
                    double dTwpen = cd * 0.2;
                    double dThpen = cd * 0.3;

                    if ((cd2 > cd-dThpen) && (cd2 < cd+dThpen)) averageT += 1;
                    if ((cd2 > cd-dTwpen) && (cd2 < cd+dTwpen)) averageT += 2;
                    if ((cd2 > cd-dTpen)  && (cd2 < cd+dTpen))  averageT += 3;
                }
            }
            scored = averageT / songs;
        }
    }

    private int scoreSong(int i, Model model)
    {
        int localScore = 0;
        Htemp = db.getFPInts(Integer.parseInt(libraryData.get(i)[6]));
        for (int t = 0; t <= 49 ; t++)
        {
            int c  = model.get(t);
            int c2 = Htemp[t];

            double Tpen  = c * 0.1;
            double Twpen = c * 0.2;
            double Thpen = c * 0.3;

            if (t < 10)
            {
                if ((c2 - 2.5 < c) && (c2 + 2.5 < c))
                {
                    localScore += 6;
                }
            } else
            {
                if ((c2 > c-Thpen) && (c2 < c+Thpen)) localScore += 2;
                if ((c2 > c-Twpen) && (c2 < c+Twpen)) localScore += 4;
                if ((c2 > c-Tpen)  && (c2 < c+Tpen))  localScore += 6;
            }
        }
        //System.out.println(localScore);
        return localScore;
    }

    private int scoreSongd(int i, Model model)
    {
        int localScore = 0;
        Dtemp = db.getFPDoubles(Integer.parseInt(libraryData.get(i)[6]));
        for (int t = 0; t <= 49 ; t++)
        {
            if (Dtemp[t] != -99999)
            {
                double cd  = model.get_d(t);
                double cd2 = Dtemp[t];

                double dTpen  =  cd * 0.1;
                double dTwpen = cd * 0.2;
                double dThpen = cd * 0.3;

                if ((cd2 > cd-dThpen) && (cd2 < cd+dThpen)) localScore += 1;
                if ((cd2 > cd-dTwpen) && (cd2 < cd+dTwpen)) localScore += 2;
                if ((cd2 > cd-dTpen)  && (cd2 < cd+dTpen))  localScore += 3;
            }
        }
        return localScore;
    }

    private void score(Model model, int max, double thres, int loopNo)
    {
        limit = max;
        LinkedList Artist = model.getArt();
        LinkedList Album = model.getAlm();
        LinkedList Genre = model.getGen();
        int localScore = 0;
        int localScored = 0;
        addedSongs = 0;
        double average = (score + scored) /2;
        Random rand = new Random();
        int index = rand.nextInt(libraryData.size()-1);
        for (int i=0; i< libraryData.size(); i++)
        {
            if ((loopNo == 1) && (i % 50 == 0)) 
            {
                index = rand.nextInt(libraryData.size()-1);
            }
            allowed = true;
            index++;
            if (index >= libraryData.size()) index = 0;
            //System.out.println(libraryData.get(index)[1]);
            double ArtS = 0;
            double AlbS = 0;
            double GenS = 0;
            double BpmS = 0;
            localScore  = scoreSong(index, model);
            localScored = scoreSongd(index, model);

            int BPM = db.getBPM(Integer.parseInt(libraryData.get(index)[6]));

            for (int t = 0; t < Artist.size(); t++)
            {
                //System.out.println(Artist.size());
                Object artist = libraryData.get(index)[1];
                if (artist != null)
                {
                    if (artist.equals(Artist.get(t)))
                    {
                        ArtS = 100;
                    }
                }
            }

            for (int t = 0; t < Album.size(); t++)
            {
               Object album = libraryData.get(index)[2];
               if (album != null)
               {
                   if (album.equals(Album.get(t)))
                   {
                       AlbS = 100;
                   }
              }
            }

            for (int t = 0; t < Genre.size(); t++)
            {
               Object genre = libraryData.get(index)[2];
               if (genre != null)
               {
                   if (genre.equals(Genre.get(t)))
                   {
                       GenS = 100;
                   }
               }
            }

            double bpmA = model.getBPM() * 0.15;
            double Bpm_beat = model.getBPM();
            if ((BPM > bpmA - Bpm_beat) && (BPM < bpmA + Bpm_beat))
            {
               BpmS = average;
            }
            double bpmSlider = getConfig("BPMSlider");
            double moodSlide = getConfig("moodSlider");
            double volumeSlide = getConfig("volumeSlider");
            double genreSlider = getConfig("genreSlider");
            double artistSlide = getConfig("artistSlider");
            double albumSlider = getConfig("albumSlider");

            double PefScore = (score*moodSlide)+(scored*volumeSlide);
                   PefScore += (100*artistSlide)+(100*albumSlider)+(100*genreSlider)+(100*bpmSlider);
            double LibScore = (localScore*moodSlide)+(localScored*volumeSlide);
                   LibScore += (ArtS*artistSlide)+(AlbS*albumSlider)+(GenS*genreSlider)+(BpmS*bpmSlider);
            
            double TeLS = PefScore * thres;
            double low  = (PefScore - TeLS);
            //System.out.println("scores: "+ low + " " + LibScore + " " + PefScore);
            allowed = true;
            if (LibScore > low)
            {
                Object ID = Integer.parseInt(libraryData.get(index)[6]);
                for (int r=0; r<newPlaylistIDs.size(); r++)
                {
                    String[] trackContents = db.getTrack(newPlaylistIDs.get(r));
                    Object id = trackContents[6];
                    int Pindex = Integer.parseInt(id.toString());
                    int temp = Integer.parseInt(ID.toString());
                    if (temp == Pindex) allowed = false;
                }
                if (allowed == true)
                {
                    newPlaylistIDs.add(Integer.parseInt(libraryData.get(index)[6]));
                    Object track = libraryData.get(index)[0];
                    listModel.addElement(track);
                    addedSongs++;
                    allowed = false;
                    if (addedSongs == limit)
                    {
                        break;
                    }
                }
            }
        }
    }

    /**
     * gets the values of the passed fields from the config file
     * @param value
     * @return
     */
    public double getConfig(String value)
{
    double toReturn = 0;
    try
     {
        String temp = util.config(value);
        if (temp != null) {
            toReturn = Double.parseDouble(temp);
            toReturn = toReturn / 100;
        } else  {
            return 0.75;
        }
     } catch (Exception e) {
        ArmadilloLog.LogError("Create playlist UI cannot access config file - "+e.toString());
     }
    return toReturn;

    }

    private void pauseOtherThreads()
    {
        // first pause the analysing thread (if its active)
        this.isAnalysing = false;
        if (UI.getbpmThread() != null)
        {
            if (UI.getbpmThread().isAlive())
                this.isAnalysing = UI.getbpmThread().pauseAnalysing(false);
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private armadilloUI.PlaylistTable allSongsTable;
    private javax.swing.JButton generateButton;
    private javax.swing.JButton jButtonAccept;
    private javax.swing.JButton jButtonAdd;
    private javax.swing.JButton jButtonClear;
    private javax.swing.JButton jButtonMoveDown;
    private javax.swing.JButton jButtonMoveUp;
    private javax.swing.JButton jButtonRemove;
    private javax.swing.JLabel jLabelAvBPM;
    private javax.swing.JLabel jLabelNumberOfTracksToAdd;
    private javax.swing.JList jListSelectedItems;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPaneLibrary;
    private javax.swing.JScrollPane jScrollPaneSelectedItems;
    private javax.swing.JTextField jTextFieldBPM;
    private javax.swing.JTextField jTextFieldMaxTracks;
    private javax.swing.JTextField jTextFieldPlaylistName;
    private javax.swing.JButton moodButton;
    private armadilloUI.NavTree navTree1;
    private javax.swing.JLabel playlistNameLabel;
    // End of variables declaration//GEN-END:variables

}
