/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * SQLite connection & utilities
 * @author Kasia
 */

package armadilloUI;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;

/**
 * Manages the connection between the SQLite database and our java interface
 * @author Kasia
 */
public class DbConnection {

   
    private String sDriver = "";
    private String sUrl = null;
    private int iTimeout = 30;

    /**
     * the current Connection of the DB
     */
    public Connection conn = null;
    private Statement statement = null;
    private String colon = ":";
    private String semicolon = ";";
    private String ipsep = "//";

/**
 * initialises the DBConnection class
 * @param sDriverVar
 * @param sUrlVar
 */
public void init(String sDriverVar, String sUrlVar)
{
    setDriver(sDriverVar);
    setUrl(sUrlVar);
    setConnection();
    setStatement();
}

private void setDriver(String sDriverVar)
{
    sDriver = sDriverVar;
}

private void setUrl(String sUrlVar)
{
    sUrl = sUrlVar;
}

/**
 * sets up the connection to the DB
 */
public void setConnection()
{
    try
    {
        Class.forName(sDriver);
    }
    catch (Exception e) {
        ArmadilloLog.LogError("DBConnection cannot connect - " +e.toString());
    }

    try
    {
        // connect to a database
        conn = DriverManager.getConnection(sUrl);
    }
    catch (SQLException e)
    {
        // connection failed
        ArmadilloLog.LogError("DBConnection SQL Error - " +e.toString());
    }
}

/**
 * sets up a SQL query
 */
public void setStatement() {
    if (conn == null)
    {
        setConnection();
    }

    try
    {
        statement = conn.createStatement();
        statement.setQueryTimeout(iTimeout); // set timeout to 30 sec
    }
    catch (Exception e)
    {
        System.err.println(e);
        ArmadilloLog.LogError("DBConnection cannot set Statement - " +e.toString());
    }
}

/**
 * Executes a SQL statement on the DB
 * @param instruction
 * @throws SQLException
 */
public void executeStmt(String instruction) throws SQLException
{
    try
    {
        statement.executeUpdate(instruction);
    }
    catch (SQLException e)
    {
       System.err.println(e);
       ArmadilloLog.LogError("DBConnection cannot execute statement  - " +e.toString());
    }
}

/**
 * Executes a SQL query on the DB
 * @param instruction
 * @return
 */
public ResultSet executeQry(String instruction)
{
    ResultSet rs = null;
    try
    {
        rs = statement.executeQuery(instruction);
        return rs;
    }
    catch (SQLException e)
    {
        System.err.println(e);
        ArmadilloLog.LogError("DBConnection cannot execute query - " +e.toString());
        return rs; // ie. null
    }
}

/**
 * closes the connection to the SQL DB
 */
public void closeConnection()
{
    try
    {
        conn.close();
    }
    catch (Exception e) {
        ArmadilloLog.LogError("DBConnection cannot close connection - " +e.toString());
        System.out.println(e);
    }
}

}
