
/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
 * Message code:
 * 1 - "Please select a playlist."
 * 2 - "This item cannot be deleted."
 * 3 - "Playlist cannot be empty."
 * 4 - "Enter playlist name."
 * 5 - "Playlist is empty."
 * 6 - "BPM must be an integer value."
 * 7 - "No matching songs found."
 * 8 - "Please select a song."
 * 9 - "Duplicate files detected and not imported"
 * 10 -"Gstreamer not found, install and restart"
 * 11 - "This item cannot be edited."
 * 12 - "Enter a name"
 * 13 - "Field cannot be empty"
 * 14 - "Cannot add Track(s) to playlist"
 * 15 - "Cannot load settings from Config file"
 * 16 - "Track cannot be reloaded"
 * 17 - "Cannot add Track to now playing"
 * 18 - "Number of songs to add must be an Integer Larger than 0"
 * 19 - "Only found x matching songs"
 * 22 - "Playlist name taken, please choose a unique name"
 * 23 - "c";
 * 24 -
 */

/*
 * DialogUI.java
 *
 * Created on 04-Mar-2010, 22:08:10
 */

package armadilloUI;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Dynamic Warning message display class
 * @author Kate
 */
public class DialogUI extends javax.swing.JFrame {
    
    private int messageID;
    private File duplicateFile;
    private int limit = 0;
    private String level = new String();

    /** Creates new standard form of DialogUI
     * @param whichCase
     */
    public DialogUI(int whichCase) {
        ArmadilloLog.LogOperation("Constructing DialogUI");
        this.messageID = whichCase;
        initComponents();
        this.setLocationRelativeTo(null);
    }

    /**
     * Constructer used to also display limit of songs
     * @param whichCase
     * @param Songsfound
     */
    public DialogUI(int whichCase, int Songsfound) {
        ArmadilloLog.LogOperation("Constructing DialogUI");
        this.messageID = whichCase;
        limit = Songsfound;
        initComponents();
        this.setLocationRelativeTo(null);
    }

 
    /**
     * for the limit of songs
     * @param whichCase
     * @param nLevel
     */
    public DialogUI(int whichCase, String nLevel) {
        ArmadilloLog.LogOperation("Constructing DialogUI");
        this.messageID = whichCase;
        level = nLevel;
        initComponents();
        this.setLocationRelativeTo(null);
    }

    /** Creates DialogUI for when a duplicate is selected to import.
     * @param duplicate
     * @param whichCase
     */
    public DialogUI(File duplicate, int whichCase) {
        ArmadilloLog.LogOperation("Constructing DialogUI");
        this.messageID = whichCase;
        this.duplicateFile = duplicate;
        initComponents();
        this.setLocationRelativeTo(null);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.JButton jButtonOK = new javax.swing.JButton();
        javax.swing.JLabel jLabelMessage = new javax.swing.JLabel();
        javax.swing.JLabel jLabelFilename = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Warning");
        setAlwaysOnTop(true);
        setResizable(false);

        jButtonOK.setText("OK");
        jButtonOK.setName("jButtonOK"); // NOI18N
        jButtonOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonOKActionPerformed(evt);
            }
        });

        jLabelMessage.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        if (messageID == 1)
        jLabelMessage.setText("Please select a playlist."); // NOI18N
        else if (messageID == 2)
        jLabelMessage.setText("This item cannot be deleted.");
        else if (messageID == 3)
        jLabelMessage.setText("Playlist cannot be empty.");
        else if (messageID == 4)
        jLabelMessage.setText("Enter playlist name.");
        else if (messageID == 5)
        jLabelMessage.setText("Playlist is empty.");
        else if (messageID == 6)
        jLabelMessage.setText("BPM must be an integer value.");
        else if (messageID == 7)
        jLabelMessage.setText("No matching songs found.");
        else if (messageID == 8)
        jLabelMessage.setText("Please select a song in the Playlist Table.");
        else if (messageID == 9)
        {
            jLabelMessage.setText("Duplicate files detected and have not been imported");
        }
        else if (messageID == 10)
        jLabelMessage.setText("Error - Gstreamer not found, install and restart");
        else if (messageID == 11)
        jLabelMessage.setText("This item cannot be Edited");
        else if (messageID == 12)
        jLabelMessage.setText("Enter a name");
        else if (messageID == 13)
        jLabelMessage.setText("Field cannot be empty");
        else if (messageID == 14)
        jLabelMessage.setText("Cannot add Track(s) to playlist");
        else if (messageID == 15)
        jLabelMessage.setText("Cannot load settings from Config file");
        else if (messageID == 16)
        jLabelMessage.setText("Track cannot be reloaded");
        else if (messageID == 17)
        jLabelMessage.setText("Cannot add Track to now playing");
        else if (messageID == 18)
        jLabelMessage.setText("Number of songs to add must be an Integer Larger than 0");
        else if (messageID == 19)
        jLabelMessage.setText("Only found "+ limit +" matching songs");
        else if (messageID == 20)
        jLabelMessage.setText("Song length to short to require analysis");
        else if (messageID == 21)
        jLabelMessage.setText("Due to Song length the analysis level is set to"+level);
        else if (messageID == 22)
        jLabelMessage.setText("Playlist name taken, please choose a unique name");
        else if (messageID == 23)
        jLabelMessage.setText("Armadillo is already Running");
        else if (messageID == 24)
        jLabelMessage.setText("Importing in progress, please wait until it finishes.");
        else if (messageID == 25)
        jLabelMessage.setText("Analysing tracks is in progress, please stop it to continue.");
        else
        jLabelMessage.setText("Unknown error");
        jLabelMessage.setName("jLabelMessage"); // NOI18N

        jLabelFilename.setName("jLabelFilename"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(178, 178, 178)
                        .addComponent(jButtonOK, javax.swing.GroupLayout.PREFERRED_SIZE, 72, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(43, 43, 43)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jLabelFilename, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jLabelMessage, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 350, Short.MAX_VALUE))))
                .addContainerGap(51, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(25, 25, 25)
                .addComponent(jLabelFilename)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabelMessage)
                .addGap(18, 18, 18)
                .addComponent(jButtonOK)
                .addContainerGap(39, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonOKActionPerformed
        setVisible(false);
        if ((messageID ==10) || (messageID == 23)) {
            System.exit(0);
        }
    }//GEN-LAST:event_jButtonOKActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

}
