/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * EditNameUI.java
 *
 * Created on 17-Apr-2010, 10:25:53
 */

package armadilloUI;

/**
 *
 * @author jonathan
 */
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * UI and functionality for editing a tags name
 * @author jonathan
 */
public class EditNameUI extends javax.swing.JFrame {

    private String type;
    private String artist;
    private String currentName;
    private int id;
    private ArmadilloDB db;
    private ArmadilloUI UI;
    private String panelColor;
    private UIUtilities util;
    private java.awt.Color panelCol;
    private java.awt.Color panelColl;
    private boolean isAnalysing;

    /** Creates new form EditNameUI
     * @param ui
     * @param passedtype
     * @param artist
     * @param currentName
     * @param id
     */
    public EditNameUI(ArmadilloUI ui, String passedtype, String currentName,String artist, int id) {
        ArmadilloLog.LogOperation("Constructing EditNameUI");
        try {
            db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            Logger.getLogger(EditTrackUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        this.type = passedtype;
        this.artist = artist;
        this.currentName = currentName;
        this.id = id;
        this.UI =ui;
        util = new UIUtilities();
        initComponents();
        this.setLocationRelativeTo(null);
                try{
            panelColor =  util.config("customPanelBackgroundColour");
            if (util.config("skin").equals("Custom")){
                panelCol = new java.awt.Color(Integer.parseInt(panelColor));
                nameTextField.setBackground(panelCol);
              }
            else {
                panelCol = new java.awt.Color(32,32,32);
                nameTextField.setBackground(panelCol);
            }
            }
        catch(Exception e){
            ArmadilloLog.LogError("Edit Name UI cannot read config file - " +e.toString());
            e.printStackTrace();
        }
        if (type.equals("artist root")) {
            this.setTitle("Edit Artist Name");
            this.typeLabel.setText("Edit Artist Name:");
            nameTextField.setBackground(panelCol);

        } else if (type.equals("album")) {
            this.setTitle("Edit Album Name");
            this.typeLabel.setText("Edit Album Name:");
            nameTextField.setBackground(panelCol);
        }else if (type.equals("genre")) {
            this.setTitle("Edit Genre Name");
            this.typeLabel.setText("Edit Genre Name:");
            nameTextField.setBackground(panelCol);
        }
        nameTextField.setText(currentName);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        typeLabel = new javax.swing.JLabel();
        nameTextField = new javax.swing.JTextField();
        acceptButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        typeLabel.setText("Edit Artist Name:");
        typeLabel.setName("typeLabel"); // NOI18N

        nameTextField.setText("Default");
        nameTextField.setName("nameTextField"); // NOI18N

        acceptButton.setText("Accept");
        acceptButton.setName("acceptButton"); // NOI18N
        acceptButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                acceptButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(typeLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(acceptButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cancelButton))
                    .addComponent(nameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 137, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(typeLabel)
                    .addComponent(nameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cancelButton)
                    .addComponent(acceptButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        this.dispose();
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void acceptButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_acceptButtonActionPerformed
        //UPDATE DB WITH NEW NAME, RELOAD NAVTREE, TABLE and NOW PLAYING
        if (!nameTextField.getText().equals("")) {
            this.pauseOtherThreads();
            if (this.type.equals("artist root")) {
                int tag = db.getTag(this.currentName, 2);
                db.editTag(tag, 2, this.nameTextField.getText());
            } else if (this.type.equals("album")) {            
                int tag = db.getTag(this.currentName, 3);
                db.editTag(tag, 3, this.nameTextField.getText());
            } else if (this.type.equals("genre")) {           
                int tag = db.getTag(this.currentName, 4);
                db.editTag(tag, 4, this.nameTextField.getText());
            }

            UI.reloadNavTree();
            UI.reloadTable();
            UI.reloadPlayingList();
            if (isAnalysing)
                UI.resumeAnalysing();
            isAnalysing = false;
            this.dispose();
        } else {
            new DialogUI(6).setVisible(true);
        }
    }//GEN-LAST:event_acceptButtonActionPerformed

    private void pauseOtherThreads()
    {
        // first pause the analysing thread (if its active)
        this.isAnalysing = false;
        if (UI.getbpmThread() != null)
        {
            if (UI.getbpmThread().isAlive())
                this.isAnalysing = UI.getbpmThread().pauseAnalysing(false);
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton acceptButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JTextField nameTextField;
    private javax.swing.JLabel typeLabel;
    // End of variables declaration//GEN-END:variables

}
