/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * EditTrackUI.java
 *
 * Created on 05-Mar-2010, 22:33:40
 */

package armadilloUI;

import java.awt.Color;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Editing a tracks information UI and functionality
 * @author Kate
 */
public class EditTrackUI extends javax.swing.JFrame {

    private ArmadilloDB dbEdit;
    private ArmadilloUI ui;
    private UIUtilities util;
    private int trackID;
    private String[] oldProperties;
    private int oldBPM;
    private PlaylistTable table;
    private NavTree tree;
    private PlayingList list;
    private String panelColor;
    private Color foreground;
    private boolean isAnalysing;

    /** Creates new form EditTrackUI
     * @param trackID
     * @param table
     * @param ui
     * @param navtree
     * @param list
     */
    public EditTrackUI(int trackID, PlaylistTable table, NavTree navtree, PlayingList list, ArmadilloUI ui) {
        ArmadilloLog.LogOperation("Constructing Edit Track UI");
        try {
            dbEdit = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            ArmadilloLog.LogError("Edit Track UI cannot access database - "+ex.toString());
            Logger.getLogger(EditTrackUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        util = new UIUtilities();
        this.ui = ui;
        this.trackID = trackID;
        this.table = table;
        this.tree = navtree;
        this.list = list;

        // first pause the analysing thread (if its active)
        isAnalysing = false;
        if (ui.getbpmThread() != null)
        {
            if (ui.getbpmThread().isAlive())
                isAnalysing = ui.getbpmThread().pauseAnalysing(false);
        }
        
        oldProperties = dbEdit.getTrack(trackID);
        oldBPM = dbEdit.getBPM(trackID);
        initComponents();
        this.setLocationRelativeTo(null);
        try{
            panelColor =  util.config("customPanelBackgroundColour");
            if (util.config("skin").equals("Custom")){
                java.awt.Color panelCol = new java.awt.Color(Integer.parseInt(panelColor));
                jTextFieldEditAlbum.setBackground(panelCol);
                jTextFieldEditAuthor.setBackground(panelCol);
                jTextFieldEditBPM.setBackground(panelCol);
                jTextFieldEditGenre.setBackground(panelCol);
                jTextFieldEditTitle.setBackground(panelCol);
            }
            else {
                java.awt.Color panelColl = new java.awt.Color(32,32,32);
                jTextFieldEditAlbum.setBackground(panelColl);
                jTextFieldEditAuthor.setBackground(panelColl);
                jTextFieldEditBPM.setBackground(panelColl);
                jTextFieldEditGenre.setBackground(panelColl);
                jTextFieldEditTitle.setBackground(panelColl);
            }
        }
        catch(Exception e){
            ArmadilloLog.LogError("Edit Track UI cannot access config file - "+e.toString());
            e.printStackTrace();
        }
    }
    /**
     * sets the BPM field with value passed (used by recalculate and manual bpm
     * @param BPM
     */
    public void setBPMField(int BPM) {
        jTextFieldEditBPM.setText(Integer.toString(BPM));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTextFieldEditTitle = new javax.swing.JTextField();
        jTextFieldEditAuthor = new javax.swing.JTextField();
        jTextFieldEditAlbum = new javax.swing.JTextField();
        jTextFieldEditGenre = new javax.swing.JTextField();
        jTextFieldEditBPM = new javax.swing.JTextField();
        javax.swing.JLabel jLabel1 = new javax.swing.JLabel();
        javax.swing.JLabel jLabel2 = new javax.swing.JLabel();
        javax.swing.JLabel jLabel3 = new javax.swing.JLabel();
        javax.swing.JLabel jLabel4 = new javax.swing.JLabel();
        javax.swing.JLabel jLabel5 = new javax.swing.JLabel();
        javax.swing.JToggleButton jToggleButtonCancel = new javax.swing.JToggleButton();
        javax.swing.JToggleButton jToggleButtonSave = new javax.swing.JToggleButton();
        javax.swing.JToggleButton jButtonCalculateBPM = new javax.swing.JToggleButton();
        jLabelBusy = new javax.swing.JLabel();
        javax.swing.JButton jButton1 = new javax.swing.JButton();
        javax.swing.JLabel jLabelInfo = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Edit track");
        setIconImage(util.getWindowIcon());
        setResizable(false);

        jTextFieldEditTitle.setName("jTextFieldEditTitle"); // NOI18N

        jTextFieldEditAuthor.setName("jTextFieldEditAuthor"); // NOI18N

        jTextFieldEditAlbum.setName("jTextFieldEditAlbum"); // NOI18N

        jTextFieldEditGenre.setName("jTextFieldEditGenre"); // NOI18N

        jTextFieldEditBPM.setName("jTextFieldEditBPM"); // NOI18N

        jLabel1.setText("Title:");
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setText("Author:");
        jLabel2.setName("jLabel2"); // NOI18N

        jLabel3.setText("Album:");
        jLabel3.setName("jLabel3"); // NOI18N

        jLabel4.setText("Genre:");
        jLabel4.setName("jLabel4"); // NOI18N

        jLabel5.setText("BPM:");
        jLabel5.setName("jLabel5"); // NOI18N

        jToggleButtonCancel.setText("Cancel");
        jToggleButtonCancel.setName("jToggleButtonCancel"); // NOI18N
        jToggleButtonCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonCancelActionPerformed(evt);
            }
        });

        jToggleButtonSave.setText("Save");
        jToggleButtonSave.setName("jToggleButtonSave"); // NOI18N
        jToggleButtonSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButtonSaveActionPerformed(evt);
            }
        });

        jButtonCalculateBPM.setText("Recalculate");
        jButtonCalculateBPM.setName("jButtonCalculateBPM"); // NOI18N
        jButtonCalculateBPM.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jButtonCalculateBPMMousePressed(evt);
            }
        });
        jButtonCalculateBPM.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonCalculateBPMActionPerformed(evt);
            }
        });

        jLabelBusy.setName("jLabelBusy"); // NOI18N

        jButton1.setText("Manual Estimate");
        jButton1.setName("jButton1"); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jLabelInfo.setInheritsPopupMenu(false);
        jLabelInfo.setName("jLabelInfo"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(65, 65, 65)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel1)
                    .addComponent(jLabel2)
                    .addComponent(jLabel3)
                    .addComponent(jLabel4)
                    .addComponent(jLabel5))
                .addGap(18, 18, 18)
                .addComponent(jLabelBusy)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jLabelInfo, javax.swing.GroupLayout.PREFERRED_SIZE, 143, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 67, Short.MAX_VALUE)
                        .addComponent(jToggleButtonSave)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jToggleButtonCancel))
                    .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(jTextFieldEditGenre, javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jTextFieldEditAlbum, javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jTextFieldEditAuthor, javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jTextFieldEditBPM, javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jTextFieldEditTitle, javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createSequentialGroup()
                            .addComponent(jButtonCalculateBPM, javax.swing.GroupLayout.PREFERRED_SIZE, 93, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                            .addComponent(jButton1))))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jToggleButtonCancel, jToggleButtonSave});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(53, 53, 53)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jTextFieldEditTitle, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel1))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jTextFieldEditAuthor, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jTextFieldEditAlbum, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jTextFieldEditGenre, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel4))
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGap(19, 19, 19)
                        .addComponent(jTextFieldEditBPM, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jButtonCalculateBPM)
                            .addComponent(jButton1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(jLabelBusy)
                                .addGap(25, 25, 25))
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                    .addComponent(jToggleButtonCancel)
                                    .addComponent(jToggleButtonSave)
                                    .addComponent(jLabelInfo))
                                .addContainerGap())))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(40, 40, 40)
                        .addComponent(jLabel5)
                        .addContainerGap())))
        );

        jTextFieldEditTitle.setText(oldProperties[0]);
        jTextFieldEditAuthor.setText(oldProperties[1]);
        jTextFieldEditAlbum.setText(oldProperties[2]);
        jTextFieldEditGenre.setText(oldProperties[3]);
        if (oldBPM > -1)
        jTextFieldEditBPM.setText(oldBPM+"");

        pack();
    }// </editor-fold>//GEN-END:initComponents


    private void jToggleButtonCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonCancelActionPerformed
       setVisible(false);
    }//GEN-LAST:event_jToggleButtonCancelActionPerformed

    private void jToggleButtonSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButtonSaveActionPerformed
        
        String newValue;
        if (jTextFieldEditTitle.getText().equals("")) {
            new DialogUI(13).setVisible(true);
        } else if (jTextFieldEditAuthor.getText().equals("")) {
            new DialogUI(13).setVisible(true);
        } else if (jTextFieldEditAlbum.getText().equals("")) {
            new DialogUI(13).setVisible(true);
        } else if (jTextFieldEditGenre.getText().equals("")) {
            new DialogUI(13).setVisible(true);
        } else {
            newValue = jTextFieldEditTitle.getText();
            if ( !(newValue.equals(oldProperties[0])))
                dbEdit.editTrackTag(trackID, 1, newValue);
            newValue = jTextFieldEditAuthor.getText();
            if ( !(newValue.equals(oldProperties[1])))
                dbEdit.editTrackTag(trackID, 2, newValue);
            newValue = jTextFieldEditAlbum.getText();
            if ( !(newValue.equals(oldProperties[2])))
                dbEdit.editTrackTag(trackID, 3, newValue);
            newValue = jTextFieldEditGenre.getText();
            if ( !(newValue.equals(oldProperties[3])))
                dbEdit.editTrackTag(trackID, 4, newValue);
            newValue = jTextFieldEditBPM.getText();
            if ( !(newValue.equals(oldProperties[4])))
            {
                if (newValue.equals("") || newValue.equals("NaN"))
                {
                    newValue = "-1";
                }
                try {
                    int beats = Integer.parseInt(newValue);
                    dbEdit.editTrackTag(trackID, 5, Integer.toString(beats));
                } catch (Exception e) {
                    new DialogUI(6).setVisible(true);
                }
            }
            dbEdit.closeConnection();            
            table.reload(trackID);
            tree.reload();
            list.reload();
            if (isAnalysing)
                ui.resumeAnalysing();
            isAnalysing = false;
            setVisible(false);
        }
    }//GEN-LAST:event_jToggleButtonSaveActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        TapBPMUI tap =new TapBPMUI(trackID,true,table.getArmadilloUI());
        tap.setEditTrackUI(this);
        tap.setVisible(true);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jButtonCalculateBPMMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jButtonCalculateBPMMousePressed
        this.jTextFieldEditBPM.setText("Recalculating....");
        foreground = this.jTextFieldEditBPM.getForeground();
        this.jTextFieldEditBPM.setForeground(Color.red);
    }//GEN-LAST:event_jButtonCalculateBPMMousePressed

    private void jButtonCalculateBPMActionPerformed(java.awt.event.ActionEvent evt) {
        String path = dbEdit.getTrack(trackID)[5];
        String[] args = {path};
        BPM bpm = new BPM();
        Boolean success = bpm.run(args, ui.moodDetail, false);
        //bpm.run(args, 3, true);
        //Bin temp = bpm.get_bin();
        //temp.normalize();
        // STORE BPM IN DB
        int id = dbEdit.getTag(bpm.getBPM()+"", 5);
        if ( id == -1)
            id = dbEdit.insertTag(bpm.getBPM()+"", 5);
        dbEdit.tagTrack (id, trackID);
        // STORE FINGERPRINTING DATA
        //db.storeFPDoubles(trackID, temp.gdec());
        //db.storeFPInts(trackID, temp.ghz());
        //db.storeAverageDec(trackID, temp.getAverageDec());
        int idTag = dbEdit.getTag(bpm.getBPM()+"", 5);
        if ( idTag == -1)
            idTag = dbEdit.insertTag(bpm.getBPM()+"", 5);
        jTextFieldEditBPM.setText(java.lang.Math.round(bpm.getBPM())+"");
        jTextFieldEditBPM.setForeground(foreground);
        dbEdit.setAnalysed("true", trackID);

    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    javax.swing.JLabel jLabelBusy;
    javax.swing.JTextField jTextFieldEditAlbum;
    javax.swing.JTextField jTextFieldEditAuthor;
    javax.swing.JTextField jTextFieldEditBPM;
    javax.swing.JTextField jTextFieldEditGenre;
    javax.swing.JTextField jTextFieldEditTitle;
    // End of variables declaration//GEN-END:variables

}
