/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

package armadilloUI;

import static java.lang.Math.*;
import java.lang.NumberFormatException.*; 

   /* 
     Class implements the FFT, INPUT: array of complex numbers
     OUTPUT: array of complex numbers
   */
class FFT
{
    public Complex[] FFT(Complex[] fft, int N)
    {
        int pA = 0;
        if (N == 1) return new Complex[] {fft[0]};
        double kth = PI / (N/2);
        Complex wk = new Complex(cos(kth), sin(kth));
        Complex w  = new Complex(1,0);
        Complex[] even = new Complex[N/2];
        Complex[] odd  = new Complex[N/2];
        for (int i = 0; i < N; i = i + 2)
        {
            even[pA] = fft[i];
            pA ++;
        }
        pA = 0;
        for (int i = 1; i < N; i = i + 2)
        {
            odd[pA] = fft[i];
            pA ++;
        }   
        Complex[] e = FFT(even, N/2);
        Complex[] o = FFT(odd,  N/2);
        Complex[] y = new Complex[N];
        for (int k = 0; k < (N/2); k++)
        {  
            y[k]       = e[k].plus(w.times(o[k]));
            y[k + N/2] = e[k].minus(w.times(o[k]));
            w = w.times(wk);
        }
        return y;
    }
}
