/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

package armadilloUI;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.*;
import java.awt.*;


/**
 * UI and functionality for not locating a file on the computer
 * @author Kate
 */
public class FileNotFoundUI extends javax.swing.JFrame {

    private File updatedFile;
    private int trackID;
    private ArmadilloDB db;
    private ArmadilloUI ui;
    private UIUtilities util;
    private boolean isAnalysing;

    /** Creates new FileNotFoundUI
     * @param track_id
     * @param ui
     */
    public FileNotFoundUI(int track_id, ArmadilloUI ui) {
        ArmadilloLog.LogOperation("Constructing File not Found UI");
        try {
            db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            Logger.getLogger(FileNotFoundUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        trackID = track_id;
        this.ui = ui;
        initComponents();
        this.setLocationRelativeTo(null);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.JLabel jLabelFileNotFound = new javax.swing.JLabel();
        javax.swing.JButton jButtonUpdate = new javax.swing.JButton();
        javax.swing.JButton jButtonRemove = new javax.swing.JButton();
        javax.swing.JLabel jLabel2 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Confirmation");

        jLabelFileNotFound.setText("File not found! Do you want to update its location or remove it from the library?");
        jLabelFileNotFound.setName("jLabelFileNotFound"); // NOI18N

        jButtonUpdate.setText("Update");
        jButtonUpdate.setName("jButtonUpdate"); // NOI18N
        jButtonUpdate.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonUpdateActionPerformed(evt);
            }
        });

        jButtonRemove.setText("Remove");
        jButtonRemove.setName("jButtonRemove"); // NOI18N
        jButtonRemove.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRemoveActionPerformed(evt);
            }
        });

        jLabel2.setName("jLabel2"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabelFileNotFound, javax.swing.GroupLayout.PREFERRED_SIZE, 511, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(25, Short.MAX_VALUE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(201, Short.MAX_VALUE)
                .addComponent(jButtonUpdate, javax.swing.GroupLayout.PREFERRED_SIZE, 114, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(68, 68, 68)
                .addComponent(jButtonRemove, javax.swing.GroupLayout.PREFERRED_SIZE, 81, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(63, 63, 63))
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jButtonRemove, jButtonUpdate});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 63, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabelFileNotFound, javax.swing.GroupLayout.DEFAULT_SIZE, 75, Short.MAX_VALUE))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButtonRemove)
                    .addComponent(jButtonUpdate))
                .addGap(24, 24, 24))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonUpdateActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonUpdateActionPerformed
         FileNameExtensionFilter filter = new FileNameExtensionFilter("Music Files ( .mp3 .wav .ogg .m4a )", "mp3", "wav","ogg","m4a");

        JFileChooser locateFile = new JFileChooser();
        locateFile.setDialogTitle("Choose new location");
        Color bg = new java.awt.Color(32,32,32);
        try {
            String FilepanelColor = util.config("customPanelBackgroundColour");
            if (util.config("skin").equals("Custom")) {
                bg = new java.awt.Color(Integer.parseInt(FilepanelColor));
            }
        } catch (Exception e) {e.printStackTrace();}
        
            setBG( locateFile.getComponents(), bg, 0 );
            locateFile.repaint();

        locateFile.setFileFilter(filter);
        locateFile.setMultiSelectionEnabled(false);
        int returnVal = locateFile.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION)
        {
            updatedFile = locateFile.getSelectedFile();
            try {
                this.pauseOtherThreads();
                db.updateLocation(trackID, updatedFile);
                if (isAnalysing)
                    ui.resumeAnalysing();
                isAnalysing = false;
                setVisible(false);
            }
            catch (Exception ex) {
                ArmadilloLog.LogError("File not found UI cannot access DB - "+ex.toString());
                Logger.getLogger(DeleteUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        ui.reloadTable(trackID);
    }//GEN-LAST:event_jButtonUpdateActionPerformed


    private void setBG( Component[] jc, Color  bg, int depth )
      {
            for( int i = 0; i < jc.length; i++ ) {
                  Component c = jc[i];
                  if( c instanceof Container )// {
                        setBG( ((Container)c).getComponents(), bg, depth );
                  c.setBackground( bg );
            }
      }

    private void jButtonRemoveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRemoveActionPerformed
        db.removeTrack(trackID);
        ui.reloadTable(trackID);
        ui.reloadPlayingList();
        ui.removeTrackFromPlayingList(trackID);// CHANGE IF NECESSARY
        setVisible(false);
        ui.playCurrentSong();
    }//GEN-LAST:event_jButtonRemoveActionPerformed

    private void pauseOtherThreads()
    {
        // first pause the analysing thread (if its active)
        this.isAnalysing = false;
        if (ui.getbpmThread() != null)
        {
            if (ui.getbpmThread().isAlive())
                this.isAnalysing = ui.getbpmThread().pauseAnalysing(false);
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

}
