/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

package armadilloUI;

// http://www.phys.unsw.edu.au/jw/dB.html - allowed neg dec.

import static java.lang.Math.*;
import java.lang.NumberFormatException.*; 

class FingerPrint
{  
    private Bin moods = new Bin();
    private double temp;
    private double temp1;

    public void run(float[] find, int rate, int freq_N)
    {
        int N = find.length;
        int n = N;
        n = N;
        Complex[] fft = new Complex[n];
        for (int i = 0; i < n; i++)
        {
            double d_fft = find[i];
            fft[i] = new Complex(d_fft,0);
        }
        FFT instance = new FFT();
        Complex[] result = instance.FFT(fft, n);
        instance = null;
        sort(result,n, rate, freq_N);
    }
     
    public Bin return_bin()
    {
        return moods;
    }
     
     /*
        this method sorts out the hz values, it splits the values and takes a count
        of each range of values.  The average of the decibels is also taken.
     */
    public void sort(Complex[] fft, int N, int rate, int freq_N)
    { 
        for (int t = 0; t < N; t++)
        {
            int temp_rate = rate / 2;
            temp = 0;
            temp1 = 0;
            double re = fft[t].get_re();
            double im = fft[t].get_im();
            double Decibels = 0;
            double MagSqrd = ((re*re) + (im * im));
            if ((re == 0) && (im == 0)) Decibels = 0;
            else Decibels = 10.0 * log10(MagSqrd);
            if (re > 0)
            {           	
            	double FreqIntensity = sqrt((re*re) + (im*im));          	
                double hz = FreqIntensity * (rate / freq_N);
            	moods.store_dec(Decibels);
                temp = temp_rate / 2;
                temp1 = temp;
                if ((hz >= temp)  && (hz < (rate/2)))  moods.inc(49,Decibels);
                for (int i = 48 ; i > 0; i--) write(hz,i,Decibels,1.25);
                if  (hz <= temp) moods.inc(0,Decibels);
                //System.exit(1);         
            } 
        }   
    }

    public void write(double hz, int binN, double Decibels, double div)
    {
        temp1 = temp;
        temp = temp / div;
        temp = roundDouble(temp,2);
        temp1 = roundDouble(temp1,2);
        //System.out.println(temp + "      " + temp1);
        if ((hz >= temp)  && (hz < temp1))  moods.inc(binN,Decibels);
    }

    // Round a number to 2dp
    public final double roundDouble(double d, int places) 
    {
        return round(d * pow(10, (double) places)) / pow(10,(double) places);
    } 
    
    public void normalize(int speed)
    {
        moods.normalize(speed);
    }  
}
