/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

package armadilloUI;

import java.io.File;
import java.util.concurrent.TimeUnit;
import org.gstreamer.elements.PlayBin;
import org.gstreamer.*;

/**
 * Returns track's metadata
 *
 * @author Kate
 */
public class GetMetadata {

    private String[] metadata = new String[4];
    private String[] empty = {};
    private int metaID;
    private long duration = -1;

    /**
     * gets the meta data from a file
     * @param fileToProcess
     * @return
     */
    public String[] getMeta(final File fileToProcess) {
        clearMetadata();
        empty = Gst.init("GetMetadata", empty);
        PlayBin playbin = new PlayBin("GetMetadataPlayer");
        playbin.setVideoSink(ElementFactory.make("fakesink", "videosink"));
        playbin.setInputFile(fileToProcess);
        playbin.getBus().connect(new Bus.TAG()
        {
               public void tagsFound(GstObject source, TagList tagList) 
                   {
                   for (String tagName : tagList.getTagNames()) {
                    for (Object tagData : tagList.getValues(tagName)) {
                        if (tagName.equals("title"))
                            metaID = 0;
                        else if (tagName.equals("artist"))
                            metaID = 1;
                        else if (tagName.equals("album"))
                            metaID = 2;
                        else if (tagName.equals("genre"))
                            metaID = 3;
                        else
                            continue;
                        metadata[metaID] = tagData.toString();
                        if (metadata[metaID].equals("Unknown"))
                            metadata[metaID] = "<Unknown>";
                        // if no title, use filename
                        if (metadata[0].equals("<Unknown>") || metadata[0].equals(""))
                            metadata[0] = fileToProcess.getName();
                    }
                }
            }
        });

        playbin.setState(State.PAUSED);
        playbin.getState();

        this.duration = playbin.queryDuration(TimeUnit.SECONDS);

        playbin.setState(State.NULL);

        return metadata;
    }

    /**
     * returns the duration of a track found from a file
     * @return
     */
    public long getDuration()
    {
        return duration;
    }
    private void clearMetadata()
    {
        String[] initial = {"<Unknown>", "<Unknown>", "<Unknown>", "<Unknown>"};
        this.metadata = initial;
    }

}
