/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

package armadilloUI;

import java.io.File;
import java.util.ArrayList;
import java.util.logging.*;

class ImportSong implements Runnable
{
   // private final Object synchObj = new Object();
    private ArmadilloDB db;
    private ArmadilloUI ui;
    private int trackID = -1;
    private String[] trackMeta;
    private Thread t;
    public ArrayList<File> importList = new ArrayList<File>();
    private ArrayList<Integer> newIDs = new ArrayList<Integer>();
    private boolean theresstufftoanalyse = false;
    private int percentage = 0;
    // thread that calculates bpm and other for tracks
    private ImportSongBPMThread bpmThread;
    private boolean isAnalysing;

    public ImportSong(String name, ArrayList<File> toImport, ArmadilloUI ui)
    {
        ArmadilloLog.LogOperation("Constructing Import Song Thread");
        t = new Thread(this, name);
        for (int y=0; y<toImport.size(); y++)
            this.importList.add(toImport.get(y));
        this.ui=ui;
        openImportDbObject();
        t.setPriority(Thread.NORM_PRIORITY-1);
        t.start();
    }

    public void run()
    {
        boolean duplicates = false;
        //this.pauseOtherThreads();
        for (int i=0; i<importList.size(); i++)
        {
            percentage = (((i+1)*100)/importList.size());
            ui.updateProgressBarText("Adding "+(i+1)+" of "+importList.size());
            ui.updateProgressBar(percentage);
            // don't import duplicates!
            if (db.exists(importList.get(i).getPath()))
                duplicates = true;
            else {
                // first add track to library
                newIDs.add(addTrackToLibrary(importList.get(i)));
                //System.out.println("added track to library");
                theresstufftoanalyse = true;
            }
        }
        ui.updateProgressBar(0);
        if (duplicates) {
            new DialogUI(9).setVisible(true);
        }
        ui.reloadTable();
        ui.reloadNavTree();
        UIUtilities util = new UIUtilities();
        boolean importBool = true;
        try {
            String launchMood = util.config("moodOnImport");

            if (launchMood == null) {
                util.config("moodOnImport","true");
            } else {
                importBool = Boolean.parseBoolean(launchMood);
            }
        } catch (Exception e) {
            ArmadilloLog.LogError("Import Song thread cannot access config file - "+e.toString());
            e.printStackTrace();
        }
        // CHECK IF THERE ARE ANY SONGS LEFT TO ANALYSE
        if (importBool) //importMoods)
        {
            int[] toAnalyse = new int[newIDs.size()];
            for (int i=0; i<newIDs.size(); i++)
                toAnalyse[i] = newIDs.get(i);

            if (theresstufftoanalyse)
            {
                bpmThread = ui.getbpmThread();
                if (bpmThread == null || !(bpmThread.isAlive()))
                {
                    // start new thread to calculate bpms of the new tracks
                    bpmThread = new ImportSongBPMThread("bpmThread", toAnalyse, ui);
                    ui.setbpmThread(bpmThread);
                }
                else
                    bpmThread.addToQueue(toAnalyse);
            }
            theresstufftoanalyse = false;
            percentage = 0;
            closeImportDbObject();
            ui.reloadTable();
            ui.reloadNavTree();
            //if (isAnalysing)
                //ui.resumeAnalysing();
            //isAnalysing = false;
        }
    }
    
    public int addTrackToLibrary(final File newFile)
    {
        GetMetadata meta = new GetMetadata();
        trackMeta = meta.getMeta(newFile);
        if (false) {
            // testing only
            System.out.print("TRACK metadata: ");
           // System.out.println(trackMeta[0] + " " + trackMeta[1] + " " + trackMeta[2] + " " + trackMeta[3]);
        }
       // System.out.println("passed the meta");
        // ADD TRACK TO DB, return ID of the track
        trackID = db.insertTrack(newFile.getPath(), meta.getDuration(), trackMeta);
        return trackID;
    }

    public void openImportDbObject()
    {
        try {
            this.db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            ArmadilloLog.LogError("Import song thread cannot access database - "+ex.toString());
            Logger.getLogger(ImportSong.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void closeImportDbObject()
    {
        this.db.closeConnection();
    }

    boolean isAlive() {
        return t.isAlive();
    }

    private void pauseOtherThreads()
    {
        // first pause the analysing thread (if its active)
        this.isAnalysing = false;
        if (ui.getbpmThread() != null)
        {
            if (ui.getbpmThread().isAlive())
                this.isAnalysing = ui.getbpmThread().pauseAnalysing(false);
        }
    }
    /*public void suspendImporting() throws InterruptedException {
        synchronized (synchObj) {
            synchObj.wait();
        }
    }

    public void resumeImporting() throws InterruptedException {
        synchronized (synchObj) {
            synchObj.notify();
        }
    }*/
}

