/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * This class draws the graph of a song
 * It is used in the ArmadilloUI class to insert the graph of a song into the UI
 */

package armadilloUI;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Line2D;
import java.awt.event.*;
import java.lang.Math.*;

class MyPanel extends JPanel
{
    float initialWidth;
    float initialHeight;

    int array_size = 5;
    double start_x = 10;
    double start_y = 230;
    double[] lst_x = new double[array_size];
    double[] lst_y = new double[array_size];
    int plast = 0;

    double scaleX = 1;
    double scaleY = 1;

    // SETS THE BORDER
    public MyPanel()
    {
        setBorder(BorderFactory.createLineBorder(Color.black));
    }

    public void init()
    {
        initialHeight = getHeight();
        initialWidth = getWidth();

        addComponentListener(new ComponentAdapter(){
            @Override
            public void componentResized(ComponentEvent e)
            {
                scaleX = getWidth() / initialWidth;
                scaleY = getHeight() / initialHeight;
            }
        });
    }

    // SETS THE DIMENSION
    @Override
    public Dimension getPreferredSize()
    {
        return new Dimension(500, 500);
    }

    // DRAWS THE LINES ON THE GRAPH, SAVES THE END POINT OF THE LINE, THIS MEANS
    // IT KNOWS WHERE TO START THE NEXT LINE AS ALL LINES MUST LINK
    public void draw_line(Graphics2D g2)
    {
        for (int i = 0; i < plast; i++)
        {
            double x = lst_x[i]*scaleX;
            double y = lst_y[i]*scaleY;

			g2.draw(new Line2D.Double(start_x, start_y, x, y));
            start_x = x;
            start_y = y;
        }
    }

    // RENDER THE LINES FROM AXIS TO JAVA CANVAS
    public double render(double plot, char axis)
    {
        if (axis == 'x') return (plot * 3.5)+ start_x;
        if (axis == 'y') return (plot * -3) + start_y;
        return 0;
    }

    // SAVE LINE DETAILS INTO THE ARRAY AND PAINT ONCE ALL DATA RECIEVED
    public void update(float[] lines)
    {
        double [] buffer = new double[lines.length];
        for (int i = 0; i < lines.length; i++)
        {
            buffer[i] = (double)lines[i];
        }
        lst_x = buffer;
        double [] buffery = new double[lines.length];
        for (int i = 0; i < lines.length; i++)
        {
            buffery[i] = (i+35) ;
        }
        plast = lines.length;
        lst_y = buffery;
        repaint();
    }

    // SAVE LINE DETAILS INTO THE ARRAY AND PAINT ONCE ALL DATA RECIEVED
    public void update(double lx, double ly, boolean draw)
    {
        lst_x[plast] = render(lx,'x');
        lst_y[plast] = render(ly,'y');
        if (plast == array_size-1) increase_array();
        plast++;
        if (draw == true) repaint();
    }

    // MAIN PAINT FUNCTION: DRAWS THE GRAPH - ONLY ONCE ON EXEC AND RESIZE
    @Override
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;
        start_x = 10;
        start_y = getHeight() / 2;
        g2.draw(new Line2D.Double(10,25,10, getHeight() - 25));
        g2.draw(new Line2D.Double(10, start_y, getWidth() - 20, start_y));
        g2.setPaint(Color.gray);
        draw_line(g2);
    }

    // DYNAMIC ARRAY: IT NEEDS RESIZING WHEN FULL
    public void increase_array()
    {
        double[] bufferx = new double[array_size*2];
        double[] buffery = new double[array_size*2];
        for (int i =0; i < array_size; i++) bufferx[i] = lst_x[i];
        for (int i =0; i < array_size; i++) buffery[i] = lst_y[i];
        lst_x = bufferx;
        lst_y = buffery;
        array_size = array_size * 2;
    }

    public void del_array()
    {
        array_size = 5;
        double[] buffer = new double[array_size];
        double[] buffery = new double[array_size];
        lst_x = buffer;
        lst_y = buffery;
        plast = 0;
    }

}
