/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * This class extends the JTree class and deals with adding playlists and
 * directories to the tree, also holds all the mouse listeners for the
 * navigation tree
 *
 * Author Jon
 */

package armadilloUI;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.filechooser.FileSystemView;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.ArrayList;

/**
 * Navigation tree class, display top-level information about the state of the database
 * @author jonathan
 */
public class NavTree extends JTree {


    private DefaultTreeModel navTreeModel;
    private PlaylistTable table;
    private PlayingList list;
    private NavTreeNode currentNode;
    private ArmadilloUI UI;
    private FileNameExtensionFilter filter;
    private ArmadilloDB dbNavTree;
    private Boolean[] expanded = {false,false,false,false,false,false, false};
    private long lastDoubleClick=0;
    private long lastSingleClick=0;
    private long lastEntered = 0;
    private String lastClickName ="";

    private long lasttime=0;

    /**
     * basic constructor
     */
    public NavTree() {
        ArmadilloLog.LogOperation("Constructing Nav Tree");
    }

    
    /**
     * MUST BE CALLED BEFORE 'SETUP', THIS TELLS THE TREE WHERE TO
     * DISPLAY SELECTED PLAYLISTS
     * @param passedTable
     */
    public void setSelectedTable(PlaylistTable passedTable) {
        table = passedTable;
    }

    /**
     * sets the playing list for the nav tree to pass songs to
     * @param passedList
     */
    public void setPlayingList(PlayingList passedList) {
        list = passedList;
    }

    /**
     * sets the UI and filter to use for various public methods
     * @param UI
     * @param filter
     */
    public void setUIandFilter(ArmadilloUI UI, FileNameExtensionFilter filter) {
        this.UI = UI;
        this.filter = filter;
    }


    /**
     * Set up - this must be called to use the nav tree
     * : loads library and playlist from database and also adds top level of computer
     */
    public void setUp() {

        DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode();
        this.setRootVisible(false);
        //Sets up library
        NavTreeNode libraryNode = new NavTreeNode("Library","adir");
        DefaultMutableTreeNode library = new DefaultMutableTreeNode(libraryNode);
        rootNode.add(library);

        NavTreeNode allSongsNode = new NavTreeNode("All Songs","all songs",0);
        DefaultMutableTreeNode allSongs = new DefaultMutableTreeNode(allSongsNode);
        //library.add(allSongs);
        //this node contains all songs from the Library

        NavTreeNode artistsNode = new NavTreeNode("Artists","adir");
        DefaultMutableTreeNode artists = new DefaultMutableTreeNode(artistsNode);
        library.add(artists);
        //add all artists to this node
        
        NavTreeNode albumsNode = new NavTreeNode("Albums","adir");
        DefaultMutableTreeNode albums = new DefaultMutableTreeNode(albumsNode);
        library.add(albums);
        //add all albums to this node


        NavTreeNode genresNode = new NavTreeNode("Genres","adir");
        DefaultMutableTreeNode genres = new DefaultMutableTreeNode(genresNode);
        library.add(genres);
        //add all genres to this node

        NavTreeNode bpmNode = new NavTreeNode("BPM","adir");
        DefaultMutableTreeNode bpm = new DefaultMutableTreeNode(bpmNode);
        library.add(bpm);
        //add Different bpm brackets to this node

        try {
            openNavTreeDbObject();
            //ARTISTS
            ArrayList<String[]> artists_DB = dbNavTree.getTags(2);
            
            // USE THIS TO GET ALBUMS FOR EACH ARTIST
            for (int i=0; i<artists_DB.size(); i++)
            {
                int contentsSize = dbNavTree.getTracksByTag(Integer.parseInt(artists_DB.get(i)[1])).size();
                // Don't display <Unknown> tags if no tracks assigned to it
                if (contentsSize > 0)
                {
                NavTreeNode artistRootNode = new NavTreeNode(artists_DB.get(i)[0],"artist root",Integer.parseInt(artists_DB.get(i)[1]));
                DefaultMutableTreeNode artistRoot = new DefaultMutableTreeNode(artistRootNode);
                artists.add(artistRoot);
                NavTreeNode artistAllSongsNode = new NavTreeNode("All "+artists_DB.get(i)[0] + " Tracks","artist",Integer.parseInt(artists_DB.get(i)[1]));
                DefaultMutableTreeNode artistAllSongs = new DefaultMutableTreeNode(artistAllSongsNode);
                artistRoot.add(artistAllSongs);    
                    NavTreeNode albumRootNode = new NavTreeNode("Albums","adir");
                    DefaultMutableTreeNode albumRoot = new DefaultMutableTreeNode(albumRootNode);
                    artistRoot.add(albumRoot);
                    ArrayList<String[]> artistAlbums = dbNavTree.getAlbums(artists_DB.get(i)[0]);
                        for (int j=0; j<artistAlbums.size(); j++)
                        {
                            NavTreeNode albumNode = new NavTreeNode(artistAlbums.get(j)[0],"album",Integer.parseInt(artistAlbums.get(j)[1]));
                            DefaultMutableTreeNode album = new DefaultMutableTreeNode(albumNode);
                            albumRoot.add(album);
                           // add to Nav tree:
                           // ... test.get(j)[0]) <- this is the name, [1] is id
                        }
            }
//
//            for (int i=0; i<artists_DB.size(); i++)
//            {
//                int contentsSize = dbNavTree.getTracksByTag(Integer.parseInt(artists_DB.get(i)[1])).size();
//                // Don't display <Unknown> tags if no tracks assigned to it
//                if (contentsSize > 0)
//                {
//                    NavTreeNode artistRootNode = new NavTreeNode(artists_DB.get(i)[0],"artist root",Integer.parseInt(artists_DB.get(i)[1]));
//                    DefaultMutableTreeNode artistRoot = new DefaultMutableTreeNode(artistRootNode);
//                    artists.add(artistRoot);
//                    NavTreeNode artistAllSongsNode = new NavTreeNode("All "+artists_DB.get(i)[0] + " Tracks","artist",Integer.parseInt(artists_DB.get(i)[1]));
//                    DefaultMutableTreeNode artistAllSongs = new DefaultMutableTreeNode(artistAllSongsNode);
//                    artistRoot.add(artistAllSongs);
//                    //ALBUMS
//                    ArrayList<String[]> artistTracks_DB =dbNavTree.getTracksByTag(Integer.parseInt(artists_DB.get(i)[1]));
//                    ArrayList<String> artistAlbums = new ArrayList();
//                    NavTreeNode albumRootNode = new NavTreeNode("Albums","adir");
//                    DefaultMutableTreeNode albumRoot = new DefaultMutableTreeNode(albumRootNode);
//                    artistRoot.add(albumRoot);
//                    for (int j=0; j<artistTracks_DB.size(); j++)
//                    {
//                        if (artistAlbums.contains(artistTracks_DB.get(j)[2]) == false) {
//                            artistAlbums.add(artistTracks_DB.get(j)[2]);
//                            int tagID = dbNavTree.getTag(artistTracks_DB.get(j)[2], 3);
//
//                            int albumSize = dbNavTree.getTracksByTag(tagID).size();
//                            // Don't display <Unknown> tags if no tracks assigned to it
//                            if (albumSize > 0)
//                            {
//                                NavTreeNode albumNode = new NavTreeNode(artistTracks_DB.get(j)[2],"album",tagID);
//                                DefaultMutableTreeNode album = new DefaultMutableTreeNode(albumNode);
//                                albumRoot.add(album);
//                            }
//
//                        }
//                    }
//                }
//            }
            }

            //ALBUMS
            ArrayList<String[]> albums_DB = dbNavTree.getTags(3);
            for (int i =0; i<albums_DB.size(); i++) {
                int contentsSize = dbNavTree.getTracksByTag(Integer.parseInt(albums_DB.get(i)[1])).size();
                // Don't display <Unknown> tags if no tracks assigned to it
                if (contentsSize > 0)
                {
                    NavTreeNode albumNode = new NavTreeNode(albums_DB.get(i)[0],"album",Integer.parseInt(albums_DB.get(i)[1]));
                    DefaultMutableTreeNode album = new DefaultMutableTreeNode(albumNode);
                    albums.add(album);
                }
            }
            //GENRES
            ArrayList<String[]> genres_DB = dbNavTree.getTags(4);
            for (int i=0; i<genres_DB.size(); i++)
            {                
                int contentsSize = dbNavTree.getTracksByTag(Integer.parseInt(genres_DB.get(i)[1])).size();
                // Don't display <Unknown> tags if no tracks assigned to it
                if (contentsSize > 0)
                {
                    NavTreeNode genreNode = new NavTreeNode(genres_DB.get(i)[0],"genre",Integer.parseInt(genres_DB.get(i)[1]));
                    DefaultMutableTreeNode genre = new DefaultMutableTreeNode(genreNode);
                    genres.add(genre);
                }
            }        
            //BPMS
            for (int i = 30; i<300; i=i+20) {
                ArrayList<Integer> BPM_DB = dbNavTree.getTracks(i);
                if (BPM_DB.get(0) != -1 ) {
                    NavTreeNode bpmBoundryNode = new NavTreeNode((i-10) + " - " + (i+9),"bpm",i);
                    DefaultMutableTreeNode bpmBoundry = new DefaultMutableTreeNode(bpmBoundryNode);
                    bpm.add(bpmBoundry);
                }
            }
            
            //sets up playlists section of tree
            NavTreeNode playlistsNode = new NavTreeNode("Playlists","adir");
            DefaultMutableTreeNode playlists = new DefaultMutableTreeNode(playlistsNode);
            rootNode.add(playlists);
            //add all playlists
            ArrayList<String[]> allplaylists = dbNavTree.getPlaylists();
            for (int i=0; i<allplaylists.size(); i++) {
                String[] playlistdata = allplaylists.get(i);
                NavTreeNode playlistNode = new NavTreeNode(playlistdata[0],"playlist",Integer.parseInt(playlistdata[1]));
                DefaultMutableTreeNode playlist = new DefaultMutableTreeNode(playlistNode);
                playlists.add(playlist);
            }
            closeNavTreeDbObject();
            /*
            NavTreeNode beatPlaylistsNode = new NavTreeNode("Beat Playlists","adir");
            DefaultMutableTreeNode beatPlaylists = new DefaultMutableTreeNode(beatPlaylistsNode);
            rootNode.add(beatPlaylists);
            NavTreeNode moodPlaylistsNode = new NavTreeNode("Mood Playlists","adir");
            DefaultMutableTreeNode moodPlaylists = new DefaultMutableTreeNode(moodPlaylistsNode);
            rootNode.add(moodPlaylists);
            */
        } catch (Exception ex) {
            ArmadilloLog.LogError("Nav tree cannot add library folders from DB - "+ex.toString());
            Logger.getLogger(CreatePlaylistUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        //Put top level of file system on tree
        if (UI != null) {
            FileSystemView fileSystem = FileSystemView.getFileSystemView();
            String homeDirectoryName = fileSystem.getHomeDirectory().toString();
            File homeDirectoryLocation = fileSystem.getHomeDirectory();
            NavTreeNode homeDirectory = new NavTreeNode(homeDirectoryName, "dir", homeDirectoryLocation);
            DefaultMutableTreeNode homeNode = new DefaultMutableTreeNode(homeDirectory);

            rootNode.add(homeNode);

            //Put second layer of file system on tree (so that layer above non-leaf nodes look like non-leaf nodes
            AddNextLayer(homeNode);
        }
        //Adds expansion listener for dynamic computer traversal
        TreeExpansionListener dynamicNavigation = new TreeExpansionListener() {

            public void treeExpanded(TreeExpansionEvent expansion) {
                TreePath path = expansion.getPath();
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
                NavTreeNode directory = (NavTreeNode) node.getUserObject();
                if (directory.getType().equals("dir")) {
                    expanded[6] = true;
                    Enumeration children = node.children();
                    while (children.hasMoreElements()) {
                        DefaultMutableTreeNode childNode =(DefaultMutableTreeNode) children.nextElement();
                        AddNextLayer(childNode);
                    }
                } else if (directory.getType().equals("adir")) {
                    if (directory.getName().equals("Library")) {
                        expanded[0] = true;
                    } else if (directory.getName().equals("Artists")) {
                        expanded[1] = true;
                    } else if (directory.getName().equals("Albums")) {
                        expanded[2] = true;
                    } else if (directory.getName().equals("Genres")) {
                        expanded[3] = true;
                    } else if (directory.getName().equals("BPM")) {
                        expanded[4] = true;
                    } else if (directory.getName().equals("Playlists")) {
                        expanded[5] = true;
                    }
                }
            }

            public void treeCollapsed(TreeExpansionEvent collapse) {
                TreePath path = collapse.getPath();
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
                NavTreeNode directory = (NavTreeNode) node.getUserObject();
                if (directory.getType().equals("dir")) {
                    Enumeration children = node.children();
                    while (children.hasMoreElements()) {
                        DefaultMutableTreeNode childNode =(DefaultMutableTreeNode) children.nextElement();
                        RemoveNextLayer(childNode);
                    }
                    expanded[6] = false;
                }else if (directory.getType().equals("adir")) {
                    if (directory.getName().equals("library")) {
                        expanded[0] = false;
                    } else if (directory.getName().equals("Artists")) {
                        expanded[1] = false;
                    } else if (directory.getName().equals("Albums")) {
                        expanded[2] = false;
                    } else if (directory.getName().equals("Genres")) {
                        expanded[3] = false;
                    } else if (directory.getName().equals("BPM")) {
                        expanded[4] = false;
                    } else if (directory.getName().equals("Playlists")) {
                        expanded[5] = false;
                    }
                }
            }
        };
        this.addTreeExpansionListener(dynamicNavigation);

        //Mouse listeners, handles clicks, double clicks and right clicks
        MouseListener fileNavTreeListener = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent click) {
                int clickCount = click.getClickCount();
                final NavTree navTree = (NavTree) click.getComponent();
                int row = navTree.getRowForLocation(click.getX(), click.getY());
                if (row != -1) {
                    navTree.setSelectionRow(row);
                    DefaultMutableTreeNode treeNode =(DefaultMutableTreeNode) navTree.getSelectionPath().getLastPathComponent();
                    currentNode = (NavTreeNode) treeNode.getUserObject();
                    //right click                    
                    //if (click.isPopupTrigger()) {
                    if (click.getButton() == 3) {
                        if (currentNode.getName().equals("Playlists")) {
                            //Right click menu
                            JPopupMenu rightClickMenu = new JPopupMenu();
                            JMenuItem view = new JMenuItem("View");
                            ActionListener viewListener = new ActionListener() {
                                public void actionPerformed(ActionEvent playEvent) {
                                    table.viewPlaylist(currentNode.getType(),currentNode.getListID(),currentNode.getName());
                                }
                            };
                            view.addActionListener(viewListener);
                        }
                        if ((currentNode.getType().equals("adir") == false)) {
                            //Right click menu
                            JPopupMenu rightClickMenu = new JPopupMenu();

                            JMenuItem view = new JMenuItem("View");
                            ActionListener viewListener = new ActionListener() {
                                public void actionPerformed(ActionEvent playEvent) {
                                    table.viewPlaylist(currentNode.getType(),currentNode.getListID(),currentNode.getName());
                                }
                            };
                            view.addActionListener(viewListener);

                            JMenuItem play = new JMenuItem("Play");
                            ActionListener playListener = new ActionListener() {
                                public void actionPerformed(ActionEvent playEvent) {
                                    if (currentNode.getType().equals("all songs")) {
                                        list.loadAllSongs();
                                    } else if (currentNode.getType().equals("playlist")) {
                                        list.loadPlaylist(currentNode.getListID());
                                    } else if (currentNode.getType().equals("artist")) {
                                        list.loadArtist(currentNode.getListID());
                                    } else if (currentNode.getType().equals("album")) {
                                        list.loadAlbum(currentNode.getListID());
                                    } else if (currentNode.getType().equals("genre")) {
                                        list.loadGenre(currentNode.getListID());
                                    } else if (currentNode.getType().equals("bpm")) {
                                        list.loadBPM(currentNode.getListID());
                                    } else if (currentNode.getType().equals("file")) {
                                        list.loadFile(currentNode.getLocation());
                                    }
                                }
                            };
                            play.addActionListener(playListener);

                            JMenuItem queue = new JMenuItem("Add to queue");
                            ActionListener addListener = new ActionListener() {
                                public void actionPerformed(ActionEvent playEvent) {
                                    if (currentNode.getType().equals("all songs")) {
                                        list.addAllSongs();
                                    } else if (currentNode.getType().equals("playlist")) {
                                        list.addPlaylist(currentNode.getListID());
                                    } else if (currentNode.getType().equals("artist")) {
                                        list.addArtist(currentNode.getListID());
                                    } else if (currentNode.getType().equals("album")) {
                                        list.addAlbum(currentNode.getListID());
                                    } else if (currentNode.getType().equals("genre")) {
                                        list.addGenre(currentNode.getListID());
                                    } else if (currentNode.getType().equals("bpm")) {
                                        list.addBPM(currentNode.getListID());
                                    } else if (currentNode.getType().equals("file")) {
                                        list.addFile(currentNode.getLocation());
                                    }
                                }
                            };
                            queue.addActionListener(addListener);
                            String editString = "Edit "+currentNode.getType();
                            if (currentNode.getType().equals("playlist")==false) {
                                editString = editString+" name";
                                if (currentNode.getType().equals("artist root")) {
                                    editString = "Edit artist name";
                                }
                            }
                            JMenuItem edit = new JMenuItem(editString);
                            if (currentNode.getType().equals("playlist") == false) {
                                ActionListener editListener = new ActionListener() {
                                    public void actionPerformed(ActionEvent editEvent) {
                                        String artistname;
                                        if (currentNode.getType().equals("album")) {
                                            //code to get album's artist
                                            artistname=null;
                                        } else {
                                            artistname=null;
                                        }
                                        new EditNameUI(UI,currentNode.getType(),currentNode.getName(),artistname,currentNode.getListID()).setVisible(true);
                                    }
                                };
                                edit.addActionListener(editListener);
                            } else {
                                ActionListener editListener = new ActionListener() {
                                    public void actionPerformed(ActionEvent editEvent) {
                                        try {
                                            new CreatePlaylistUI(currentNode.getListID(), UI).setVisible(true);
                                        }
                                        catch (java.lang.NullPointerException nullex) {
                                            new DialogUI(1).setVisible(true);
                                        }
                                    }
                                };
                                edit.addActionListener(editListener);
                            }

                            JMenuItem delete = new JMenuItem("Delete playlist");
                            ActionListener deleteListener = new ActionListener() {
                                public void actionPerformed(ActionEvent playEvent) {
                                    if (navTree.getSelectedList() != 1)
                                    {
                                        try {
                                                new DeleteUI("playlist", navTree.getSelectedList(),UI, navTree, UI.getTable(), UI.getList()).setVisible(true);
                                        }
                                        catch (java.lang.NullPointerException e) {
                                            new DialogUI(1).setVisible(true);
                                        }
                                        catch (Exception ex) {
                                            new DialogUI(2).setVisible(true);
                                        }
                                    }
                                    else
                                        new DialogUI(2).setVisible(true);
                                }
                            };
                            delete.addActionListener(deleteListener);

                            JMenuItem importSong = new JMenuItem("Import to library") ;
                            ActionListener importListener = new ActionListener() {
                                public void actionPerformed(ActionEvent playEvent) {
                                    //code to import song
                                    File[] newFile = {currentNode.getLocation()};
                                    UI.importThis(newFile);
                                }
                            };
                            importSong.addActionListener(importListener);

                            JMenuItem deleteSongs = new JMenuItem("Remove "+currentNode.getType()+" from library");
                            ActionListener deleteSongsListener = new ActionListener() {
                                public void actionPerformed(ActionEvent playEvent) {

                                }
                            };
                            deleteSongs.addActionListener(deleteSongsListener);

                            JSeparator separator = new JSeparator();
                            //Add relevant buttons to menu (so dont add 'edit' to directory
                            if ((currentNode.getType().equals("file")==false)&&(currentNode.getType().equals("artist root")== false)) {
                                if (currentNode.getType().equals("dir") == false) {
                                    rightClickMenu.add(view);

                                    rightClickMenu.add(play);
                                    rightClickMenu.add(queue);
                                }
                            }
                            if (currentNode.getType().equals("file")) {
                                rightClickMenu.add(importSong);
                            } else if ( currentNode.getType().equals("dir")) {
                                rightClickMenu.add(importSong);
                            } else {

                                if (currentNode.getType().equals(("playlist"))) {
                                    rightClickMenu.add(separator);
                                    rightClickMenu.add(edit);
                                    rightClickMenu.add(delete);
                                }
                                if ((!currentNode.getType().equals(("all songs"))) && (!currentNode.getType().equals(("artist")))) {
                                    rightClickMenu.add(edit);
                                }
                            }
                            if (UI != null) {
                                rightClickMenu.show(click.getComponent(), click.getX(), click.getY());
                            }
                        }
                    }
                }
            }

            @Override
            public void mouseClicked(MouseEvent click) {
                final NavTree navTree = (NavTree) click.getComponent();

                int clickCount = click.getClickCount();                
                int row = navTree.getRowForLocation(click.getX(), click.getY());
                if (row != -1) {
                    navTree.setSelectionRow(row);
                    DefaultMutableTreeNode treeNode =(DefaultMutableTreeNode) navTree.getSelectionPath().getLastPathComponent();
                    currentNode = (NavTreeNode) treeNode.getUserObject();
                    if (clickCount == 1) {

                        if ((System.currentTimeMillis()-navTree.lastSingleClick) < 100) {

                            navTree.lastSingleClick = System.currentTimeMillis();
                            return;
                        }
                        lastClickName = currentNode.getName();
                        if (!(currentNode.getType().equals("all songs") && (table.getCurrentPlaylistType().equals("all songs")))) {
                                table.viewPlaylist(currentNode.getType(), currentNode.getListID(),currentNode.getName());
                        }
                        navTree.lastSingleClick=System.currentTimeMillis();
                    } else if (clickCount == 2) {
                        if ((System.currentTimeMillis()-navTree.lastDoubleClick) < 400) {
                            navTree.lastDoubleClick = System.currentTimeMillis();
                            return;
                        }
                        if (!lastClickName.equals(currentNode.getName())) {
                            lastClickName = currentNode.getName();
                            return;
                        }
                        if (table.getCurrentPlaylistID()!=currentNode.getListID()) {
                            return;
                        }
                        if (!table.getCurrentPlaylistType().equals(currentNode.getType())) {
                            return;
                        }
                        lastClickName = currentNode.getName();
                        //Two click events

//                        long currentClickTime = System.currentTimeMillis();
//                        if ((currentClickTime - navTree.lastDoubleClick) < 1000) {
//                            navTree.lastDoubleClick = currentClickTime;
//                            return;
//                        } else {
//                            navTree.lastDoubleClick = currentClickTime;
//                        }
//                        if (currentNode.getType().equals("all songs")) {
//                            list.loadAllSongs();
//                        } else if (currentNode.getType().equals("playlist")) {
//                            list.loadPlaylist(currentNode.getListID());
//                        } else if (currentNode.getType().equals("artist")) {
//                            list.loadArtist(currentNode.getListID());
//                        } else if (currentNode.getType().equals("album")) {
//                            list.loadAlbum(currentNode.getListID());
//                        } else if (currentNode.getType().equals("genre")) {
//                            list.loadGenre(currentNode.getListID());
//                        } else if (currentNode.getType().equals("bpm")) {
//                            list.loadBPM(currentNode.getListID());
//                        } else if (currentNode.getType().equals("file")) {
//                            list.loadFile(currentNode.getLocation());
//                        }
                        if (currentNode.getType().equals("all songs")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("playlist")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("artist")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("album")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("genre")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("bpm")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("file")) {
                            table.playContents();
                        }
                        navTree.lastDoubleClick=System.currentTimeMillis();
                    }
                }                
            }
        };

        this.addMouseListener(fileNavTreeListener);

        navTreeModel = new DefaultTreeModel(rootNode);
        //navTreeModel.addTreeModelListener(nodeChanged);

        this.setModel(navTreeModel);
        //set playlists to be default open
        this.expandRow(1);
        //find 'Last Played' and select it
        int selection=2;
        for (int i=2; i<this.getRowCount()-1; i++) {            
            this.setSelectionRow(i);
            DefaultMutableTreeNode treeNode =(DefaultMutableTreeNode) this.getSelectionPath().getLastPathComponent();
            currentNode = (NavTreeNode) treeNode.getUserObject();
            if (currentNode.getName().equals("Last played")) {                
                selection=i;
            }
        }
        this.setSelectionRow(selection);


        KeyListener navTreeKeyListener = new KeyAdapter() {
            @Override
            @SuppressWarnings("static-access")
            public void keyTyped(KeyEvent e) {
                final NavTree navTree = (NavTree) e.getComponent();
                int row = navTree.getSelectionRows()[0];
                int key = e.getKeyChar();
                navTree.setSelectionRow(row);
                DefaultMutableTreeNode treeNode =(DefaultMutableTreeNode) navTree.getSelectionPath().getLastPathComponent();
                if (navTree.isExpanded(row)) {
                    navTree.collapseRow(row);
                } else {
                    navTree.expandRow(row);
                }
                currentNode = (NavTreeNode) treeNode.getUserObject();
                if (key == KeyEvent.VK_ENTER) {
                    if ((System.currentTimeMillis() - lastEntered) <200) {
                        //play node
                        if (currentNode.getType().equals("all songs")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("playlist")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("artist")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("album")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("genre")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("bpm")) {
                            table.playContents();
                        } else if (currentNode.getType().equals("file")) {
                            table.playContents();
                        }
                    } else {
                        table.viewPlaylist(currentNode.getType(), currentNode.getListID(),currentNode.getName());
                    }
                    lastEntered = System.currentTimeMillis();
                }
            }
        };
        this.addKeyListener(navTreeKeyListener);
    }


    /**
     * Reloads only the bpm section of the nav tree
     */
    public void reloadBPM() {
        for (int i=0; i<this.getRowCount(); i++) {
            DefaultMutableTreeNode treeNode =(DefaultMutableTreeNode) this.getPathForRow(i).getLastPathComponent();
            currentNode = (NavTreeNode) treeNode.getUserObject();

            if (currentNode.getName().equals("BPM")) {
                boolean bpmExpanded = this.isExpanded(i);
                //this.collapseRow(i);
                treeNode.removeAllChildren();
                System.out.print(treeNode.getChildCount());
                //BPMS
                for (int j = 30; j<300; j=j+20) {
                    openNavTreeDbObject();
                    ArrayList<Integer> BPM_DB = dbNavTree.getTracks(j);
                    closeNavTreeDbObject();
                    if (BPM_DB.get(0) != -1 ) {
                        NavTreeNode bpmBoundryNode = new NavTreeNode((j-10) + " - " + (j+9),"bpm",j);
                        DefaultMutableTreeNode bpmBoundry = new DefaultMutableTreeNode(bpmBoundryNode);
                        treeNode.add(bpmBoundry);
                    }
                }
                System.out.print(treeNode.getChildCount());
                if (bpmExpanded) {
                    this.expandRow(i);
                }
                navTreeModel.reload(treeNode);
            }
        }
    }

    /**
     * highlights the last played playlist no matter where it is in the tree
     */
    public void selectLastPlayed() {
        for (int i=0; i<this.getRowCount(); i++) {
            DefaultMutableTreeNode treeNode =(DefaultMutableTreeNode) this.getPathForRow(i).getLastPathComponent();
            currentNode = (NavTreeNode) treeNode.getUserObject();
            if (currentNode.getName().equals("Last played")) {
                this.setSelectionRow(i);
            }
        }
    }
    /**
     * highlights the last searched playlist no matter where it is in the tree
     */
    public void selectLastSearched() {
        for (int i=0; i<this.getRowCount(); i++) {
            DefaultMutableTreeNode treeNode =(DefaultMutableTreeNode) this.getPathForRow(i).getLastPathComponent();
            currentNode = (NavTreeNode) treeNode.getUserObject();
            if (currentNode.getName().equals("Last searched")) {
                this.setSelectionRow(i);
            }
        }
    }

    /**
     * reloads the entire navigation tree
     */
    public void reload() {
        setUp();
                int i = 1;
        try {
            if (expanded[0]) {
                this.expandRow(0);                            
                if (expanded[4]) {
                    this.expandRow(4);
                }
                if (expanded[3]) {
                    this.expandRow(3);
                }
                if (expanded[2]) {
                    this.expandRow(2);
                }
                if (expanded[1]) {
                    this.expandRow(1);
                }
            }
            this.setSelectionRow(-1);
        } catch (Exception ex) {
            ArmadilloLog.LogError("Nav tree cannot expand nav tree - "+ex.toString());
            System.err.println("cannot expand nav tree");
        }
    }


   
    /**
     * returns the list_id of selected playlist, or 0 if a list is not
     * selected
     * @return
     */
    public int getSelectedList() {
        DefaultMutableTreeNode treeNode =(DefaultMutableTreeNode) this.getSelectionPath().getLastPathComponent();
        currentNode = (NavTreeNode) treeNode.getUserObject();
        if (currentNode.getType().equals("playlist")) {
            return currentNode.getListID();
        }
        return 0;
    }

    private void AddNextLayer(DefaultMutableTreeNode root){
        NavTreeNode rootNode = (NavTreeNode) root.getUserObject();
        if (rootNode.getType().equals("dir")) {
            File file = rootNode.getLocation();
            File[] children = file.listFiles();
            DefaultMutableTreeNode child;
            NavTreeNode childNode;
            for (int i=0; i<children.length; i++) {
                if (children[i].isHidden()==false) {
                    if(children[i].isDirectory() == true) {
                        childNode = new NavTreeNode(children[i].getName(),"dir",children[i].getAbsoluteFile());
                        child = new DefaultMutableTreeNode(childNode);
                        root.add(child);
                    } else if (CheckFileType(children[i])){
                        childNode = new NavTreeNode(children[i].getName(),"file",children[i].getAbsoluteFile());
                        child = new DefaultMutableTreeNode(childNode);
                        root.add(child);
                    }
                }
            }
        }
    }

    private void RemoveNextLayer(DefaultMutableTreeNode root){
        NavTreeNode rootNode = (NavTreeNode) root.getUserObject();
        if (rootNode.getType().equals("dir")) {
            root.removeAllChildren();
        }
    }

    private boolean CheckFileType(File passedFile) {
        return filter.accept(passedFile);
        
    }

    private void openNavTreeDbObject()
    {
        try {
            dbNavTree = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            System.err.println("dbNavTree");
            ArmadilloLog.LogError("Nav tree cannot access DB - "+ex.toString());
            Logger.getLogger(NavTree.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void closeNavTreeDbObject()
    {
        dbNavTree.closeConnection();
    }
}