/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * This class is the node objects added to the JTree.
 *
 * Author Jon
 *
 */

package armadilloUI;

import java.io.File;

/**
 * This class is the data structure for the nodes within the navigation tree
 * @author jonathan
 */
public class NavTreeNode {
    private String name;
    private String type;
    private int list_id = -1;
    private File location;

    /**
     * This constructor used if node is a computer directory, armadillo directory, or music file
     * @param passedName
     * @param passedType
     * @param passedLocation
     */
    public NavTreeNode(String passedName, String passedType, File passedLocation) {
        name = passedName;
        location = passedLocation;
        if (passedType.equals("dir")) {
            type = "dir";
        } else if (passedType.equals("adir")) {
            type = "adir";
        } else if (passedType.equals("file")) {
            type = "file";
        } else {
            System.err.println("Error - cannot add node to navigation tree, unknown type");
        }
    }

    /**
     * This constructor used if node is created directory (i.e. 'Library)
     * @param passedName
     * @param passedType
     */
    public NavTreeNode(String passedName, String passedType) {
        name = passedName;
        if (passedType.equals("adir")) {
            type = passedType;
        } else {
            System.err.println("Error - cannot add directory to nav tree, unknown type");
        }
    }

    /**
     * This constructor used if node is playlist (including hardwired playlist)
     * @param passedName
     * @param passedType
     * @param passedID
     */
    public NavTreeNode(String passedName, String passedType, int passedID) {
        name = passedName;
        if (passedType.equals("playlist")) {
            type = "playlist";
            list_id = passedID;
        } else if (passedType.equals("all songs")) {
            type = "all songs";
            list_id = passedID;
        } else if (passedType.equals("artist")) {
            type = "artist";
            list_id = passedID;
        } else if (passedType.equals("album")) {
            type = "album";
            list_id = passedID;
        } else if (passedType.equals("genre")) {
            type = "genre";
            list_id = passedID;
        } else if (passedType.equals("bpm")) {
            type = "bpm";
            list_id = passedID; //the id is the BPM integer used to generate it
        } else if (passedType.equals("artist root")) {
            type = "artist root";
            list_id = passedID;
        } else {
            System.err.println("Error - cannot add node to navigation tree, unknown type");
        }
    }

    /**
     * toString function used to display in nav tree
     * @return
     */
    @Override
    public String toString() {
        return name;
    }

    /**
     * return node name
     * @return
     */
    public String getName()
    {
        return this.name;
    }

    /**
     * return node type
     * @return
     */
    public String getType()
    {
        return this.type;
    }

    /**
     * return list (node) id
     * @return
     */
    public int getListID()
    {
        return this.list_id;
    }

    /**
     * return file location
     * @return
     */
    public File getLocation()
    {
        return this.location;
    }
}

