/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * This class extends the JList class and adds full music playing functionality
 * Author : Jon
 */

package armadilloUI;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.swing.DefaultListModel;
import java.util.ArrayList;
import java.io.File;
import java.util.Random;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.JScrollBar;



/**
 * PLaying list class, displays your currently queued songs in a list format
 * @author jonathan
 */
public class PlayingList extends JList {

    private ArmadilloUI UI;
    private ArmadilloDB db;
    private DefaultListModel model;

    private ArrayList<Integer> originalOrderIDs;
    private ArrayList<Integer> currentOrderIDs;
    private ArrayList<Integer> tracks = new ArrayList<Integer>();
    //this points to real index of the song in tracks
    private ArrayList<Integer> actualTrackIndex = new ArrayList<Integer>();

    private int currentTrackIndex;

    private boolean loopOne;
    private boolean loopAll;
    private boolean shuffle;

    private long totalLength=0;
    private int[] rows;

    private boolean isAnalysing;

    /**
     *Basic constructer
     */
    public PlayingList() {
        ArmadilloLog.LogOperation("Constructing Playing List");

    }

    /**
     * Sets up the entire playing list
     * @param UI
     */
    public void setUp(ArmadilloUI UI) {
        try {
            db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            ArmadilloLog.LogError("Playing list cannot add library folders from DB - "+ex.toString());
            Logger.getLogger(PlayingList.class.getName()).log(Level.SEVERE, null, ex);
        }        
        int position = 0;
        UIUtilities util = new UIUtilities();
        try {

            String textposition = util.config("playingListPosition");
            if (textposition != null) {
                position =Integer.parseInt(textposition);
            } else {
                position = 0;
            }
        } catch (Exception ex) {
            ArmadilloLog.LogError("Nav tree cannot access config file - "+ex.toString());
            Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
        }        
        model = new DefaultListModel();
        this.setModel(model);
        this.UI = UI;        
        this.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        originalOrderIDs = new ArrayList();
        currentOrderIDs = new ArrayList();

        currentTrackIndex = -1;
        loopOne = false;
        loopAll = false;
        shuffle = false;
        //add key listener        
        KeyListener playingListKeyListener = new KeyAdapter() {
            @Override
            @SuppressWarnings({"static-access", "static-access", "static-access"})
            public void keyTyped(KeyEvent e) {                   
                int key = e.getKeyChar();
                final PlayingList list = (PlayingList) e.getComponent();
                final DefaultListModel model = (DefaultListModel) list.getModel();

                if (key==KeyEvent.VK_ENTER) {
                    //code to play selected                    
                    list.UI.playTimer.stop();
                    TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(list.getSelectedIndex())));
                    list.UI.Play(trackToPlay);
                    list.currentTrackIndex = list.getSelectedIndex();
                    list.setSelectedTrack(currentTrackIndex);
                } else if ((key==KeyEvent.VK_DELETE)|| (key==KeyEvent.VK_BACK_SPACE)) {
                    //code to delete from now playing
                    int[] indices = list.getSelectedIndices();
                   
                    boolean selected = false;
                    boolean playing = false;
                    for (int i=0; i<indices.length; i++) {
                        if (indices[i] == currentTrackIndex) {
                            selected = true;
                            if (list.UI.playTimer.isRunning()) {
                                playing = true;
                            }
                            list.UI.playTimer.stop();
                            list.UI.ClearTrack();
                            list.UI.ClearGraph();
                        }
                    }
                    for (int i=0; i<indices.length; i++) {
                        int indexToBeRemoved =actualTrackIndex.get(indices[i]-i);
                        for (int r=0; r<actualTrackIndex.size(); r++) {
                            if (actualTrackIndex.get(r) > indexToBeRemoved) {
                                actualTrackIndex.set(r, actualTrackIndex.get(r)-1);
                            }
                        }
                        tracks.remove(indexToBeRemoved);
                        actualTrackIndex.remove(indices[i]-i);
                        model.remove(indices[i]-i);
                    }
                    if (currentTrackIndex > actualTrackIndex.size()-1){
                        currentTrackIndex = actualTrackIndex.size()-1;
                        list.setSelectedTrack(currentTrackIndex);
                    } else {
                        list.setSelectedTrack(currentTrackIndex);
                    }
                    if (selected) {
                        if (tracks.size() > 0) {
                            if (currentTrackIndex > tracks.size()-1) {
                                currentTrackIndex = tracks.size()-1;
                            }
                            TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(list.getSelectedIndex())));
                            list.UI.Play(trackToPlay);
                            if (!playing) {
                                list.UI.Pause();
                            }
                        } else {
                            currentTrackIndex = -1;
                            list.UI.Stop();
                            list.UI.ClearTrack();
                            list.UI.ClearGraph();
                        }
                    }
                    list.setLabel();                    
                    list.saveState();
                }
            }
        };
        this.addKeyListener(playingListKeyListener);

        //add mouse listener
        MouseListener playingListMouseListener = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent click) {
                int clickCount = click.getClickCount();
                final PlayingList list =(PlayingList) click.getComponent();
                final DefaultListModel model = (DefaultListModel) list.getModel();
                Point mousePoint = new Point(click.getX(),click.getY());
                final int row = list.locationToIndex(mousePoint);
                rows = list.getSelectedIndices();
                boolean selectionClicked=false;
                for (int i=0; i<rows.length; i++) {
                    if (row==rows[i]) {
                        selectionClicked=true;                        
                    }
                }
                if (selectionClicked==false) {                    
                    rows = new int[1];
                    rows[0] = row;
                    list.clearSelection();
                    list.setSelectedIndex(row);                    
                }
                ArmadilloUI currentUI = list.UI;           
                //currentTrackID = trackIDs.get(row);
                if (rows[0] > -1) {
                    //if (click.isPopupTrigger()) {
                    if (click.getButton()==3) {
                        JPopupMenu menu = new JPopupMenu();
                        JMenuItem play = new JMenuItem("Play");
                        ActionListener playListener = new ActionListener() {
                            public void actionPerformed(ActionEvent playEvent) {
                                list.UI.playTimer.stop();
                                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(rows[0])));
                                list.UI.Play(trackToPlay);
                                //list.UI.Play(tracks.get(actualTrackIndex.get(rows[0])));
                                list.setSelectedTrack(rows[0]);
                                list.currentTrackIndex = rows[0];
                            }
                        };
                        play.addActionListener(playListener);
                        menu.add(play);

                        JMenuItem moveUp = new JMenuItem("Move up");
                        ActionListener moveUpListener = new ActionListener() {
                            public void actionPerformed(ActionEvent upEvent) {
                                //list.UI.timeTimer.stop();
                                //list.UI.Play(currentOrderIDs.get(row));
                                //list.setSelectedTrack(row);
                                //list.currentSongIndex = row;
                                rows=list.getSelectedIndices();                                
                                for (int i=0; i<rows.length; i++) {
                                    if (rows[i] != 0) {
                                        if (i==0) {
                                                int tempActualTrackIndex = actualTrackIndex.get(rows[i]);
                                                actualTrackIndex.set(rows[i], actualTrackIndex.get(rows[i]-1));
                                                actualTrackIndex.set(rows[i]-1, tempActualTrackIndex);
                                                Object temp2 = list.model.get(rows[i]);
                                                list.model.set(rows[i], list.model.get(rows[i]-1));
                                                list.model.set(rows[i]-1, temp2);                                               
                                                rows[i]=rows[i]-1;
                                        } else if (rows[i-1]!=rows[i]-1)  {
                                            int tempActualTrackIndex = actualTrackIndex.get(rows[i]);
                                            actualTrackIndex.set(rows[i], actualTrackIndex.get(rows[i]-1));
                                            actualTrackIndex.set(rows[i]-1, tempActualTrackIndex);
//                                            int temp = list.currentOrderIDs.get(rows[i]);
//                                            list.currentOrderIDs.set(rows[i], list.currentOrderIDs.get(rows[i]-1));
////                                            list.currentOrderIDs.set(rows[i]-1, temp);
//                                            if (!list.shuffle) {
//                                                list.originalOrderIDs.set(rows[i], list.originalOrderIDs.get(rows[i]-1));
//                                                list.originalOrderIDs.set(rows[i]-1, temp);
//                                            }
                                            Object temp2 = list.model.get(rows[i]);
                                            list.model.set(rows[i], list.model.get(rows[i]-1));
                                            list.model.set(rows[i]-1, temp2);
                                            rows[i]=rows[i]-1;
                                        }
                                    }
                                }
                                list.currentTrackIndex=rows[0];
                                list.setSelectedTrack(rows[0]);
                                list.setSelectedIndices(rows);
                            }
                        };
                        moveUp.addActionListener(moveUpListener);
                        menu.add(moveUp);

                        JMenuItem moveDown = new JMenuItem("Move down");
                        ActionListener moveDownListener = new ActionListener() {
                            public void actionPerformed(ActionEvent downEvent) {
                                //list.UI.timeTimer.stop();
                                //list.UI.Play(currentOrderIDs.get(row));
                                //list.setSelectedTrack(row);
                                //list.currentSongIndex = row;
                                rows=list.getSelectedIndices();
                                list.currentTrackIndex = rows[0];
                                for (int i=rows.length-1; i>-1; i--) {
                                    if (rows[i] != list.currentOrderIDs.size()-1) {
                                        if (i == rows.length-1) {
                                            int tempActualTrackIndex = actualTrackIndex.get(rows[i]);
                                            actualTrackIndex.set(rows[i], actualTrackIndex.get(rows[i]+1));
                                            actualTrackIndex.set(rows[i]+1, tempActualTrackIndex);
//                                            int temp = list.currentOrderIDs.get(rows[i]);
//                                            list.currentOrderIDs.set(rows[i], list.currentOrderIDs.get(rows[i]+1));
//                                            list.currentOrderIDs.set(rows[i]+1, temp);
//                                            if (!list.shuffle) {
//                                                list.originalOrderIDs.set(rows[i], list.originalOrderIDs.get(rows[i]+1));
//                                                list.originalOrderIDs.set(rows[i]+1, temp);
//                                            }
                                            Object temp2 = list.model.get(rows[i]);
                                            list.model.set(rows[i], list.model.get(rows[i]+1));
                                            list.model.set(rows[i]+1, temp2);
                                            list.currentTrackIndex= rows[i]+1;
                                            rows[i]=rows[i]+1;
                                        }  else if (rows[i+1] != rows[i]+1) {
                                            int tempActualTrackIndex = actualTrackIndex.get(rows[i]);
                                            actualTrackIndex.set(rows[i], actualTrackIndex.get(rows[i]+1));
                                            actualTrackIndex.set(rows[i]+1, tempActualTrackIndex);
//                                            int temp = list.currentOrderIDs.get(rows[i]);
//                                            list.currentOrderIDs.set(rows[i], list.currentOrderIDs.get(rows[i]+1));
//                                            list.currentOrderIDs.set(rows[i]+1, temp);
//                                            if (!list.shuffle) {
//                                                list.originalOrderIDs.set(rows[i], list.originalOrderIDs.get(rows[i]+1));
//                                                list.originalOrderIDs.set(rows[i]+1, temp);
//                                            }
                                            Object temp2 = list.model.get(rows[i]);
                                            list.model.set(rows[i], list.model.get(rows[i]+1));
                                            list.model.set(rows[i]+1, temp2);
                                            list.currentTrackIndex= rows[i]+1;
                                            rows[i]=rows[i]+1;
                                        }
                                    }
                                }
                                list.setSelectedTrack(rows[0]);
                                list.setSelectedIndices(rows);
                            }
                        };
                        moveDown.addActionListener(moveDownListener);
                        menu.add(moveDown);

                        JMenuItem remove = new JMenuItem("Remove");
                        ActionListener removeListener = new ActionListener() {
                            public void actionPerformed(ActionEvent removeEvent) {
                                int[] indices = list.getSelectedIndices();
                                for (int i=0; i<indices.length; i++) {
                                    System.out.println(indices[i]);
                                }

                                boolean selected = false;
                                boolean playing = false;
                                for (int i=0; i<indices.length; i++) {
                                    if (indices[i] == currentTrackIndex) {
                                        selected = true;
                                        if (list.UI.playTimer.isRunning()) {
                                            playing = true;
                                        }
                                        list.UI.playTimer.stop();
                                        list.UI.ClearTrack();
                                        list.UI.ClearGraph();
                                    }
                                }
                                for (int i=0; i<indices.length; i++) {
                                    int indexToBeRemoved =actualTrackIndex.get(indices[i]-i);
                                    for (int r=0; r<actualTrackIndex.size(); r++) {
                                        if (actualTrackIndex.get(r) > indexToBeRemoved) {
                                            actualTrackIndex.set(r, actualTrackIndex.get(r)-1);
                                        }
                                    }
                                    tracks.remove(indexToBeRemoved);
                                    actualTrackIndex.remove(indices[i]-i);
                                    model.remove(indices[i]-i);
                                }
                                if (currentTrackIndex > actualTrackIndex.size()-1){
                                    currentTrackIndex = actualTrackIndex.size()-1;
                                    list.setSelectedTrack(currentTrackIndex);
                                } else {
                                    list.setSelectedTrack(currentTrackIndex);
                                }
                                if (selected) {
                                    if (tracks.size() > 0) {
                                        if (currentTrackIndex > tracks.size()-1) {
                                            currentTrackIndex = tracks.size()-1;
                                        }
                                        TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                                        list.UI.Play(trackToPlay);
                                        if (!playing) {
                                            list.UI.Pause();
                                        }
                                        //list.UI.Play(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                                    } else {
                                        currentTrackIndex = -1;
                                        list.UI.Stop();
                                        list.UI.ClearTrack();
                                        list.UI.ClearGraph();
                                    }
                                }
                                list.setLabel();
                                list.saveState();
                            }
                        };
                        remove.addActionListener(removeListener);
                        menu.add(remove);

                       
                        menu.show(click.getComponent(),click.getX(),click.getY());

                    } else if (clickCount == 2) {
                        int[] rows = list.getSelectedIndices();
                        list.currentTrackIndex = rows[0];
                        list.setSelectedIndex(rows[0]);
                        TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(rows[0])));
                        list.UI.Play(trackToPlay);
                        //list.UI.Play(tracks.get(actualTrackIndex.get(rows[0])));
                    }
                }
            }
        };

        this.addMouseListener(playingListMouseListener);
    }

    /**
     * reloads a specific track
     * @param trackID
     */
    public void reloadTrack(int trackID) {
        boolean removedCurrentSong = false;
        for (int i=0; i<actualTrackIndex.size(); i++) {
            if (tracks.get(actualTrackIndex.get(i))==trackID) {
                TrackObject tempTrack = new TrackObject(tracks.get(i));
                if (tempTrack.getTrackID() != -1) {
                    model.remove(i);
                    model.add(i, tempTrack.getTableData());
                } else {
                    if (currentTrackIndex == i) {
                        UI.ClearTrack();
                        UI.ClearGraph();
                        removedCurrentSong = true;
                    }
                    model.remove(i);
                    int index = actualTrackIndex.get(i);
                    tracks.remove(index);
                    actualTrackIndex.remove(i);
                    if (currentTrackIndex > i) {
                        currentTrackIndex--;
                    }
                    for (int j = i; j < actualTrackIndex.size(); j++) {
                        actualTrackIndex.set(j, actualTrackIndex.get(j)-1);
                    }
                    i--;
                    if (removedCurrentSong) {
                        if (currentTrackIndex<actualTrackIndex.size()) {
                            TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                            UI.Play(trackToPlay);
                        }
                    }
                }
            }
        }
    }

    /**
     * basic reload non-track specific
     */
    public void reload() {
        //this.closePlayingListDbObject();
        //checkThreads();
        boolean removedCurrentSong = false;
        for (int i=0; i<actualTrackIndex.size(); i++) {
            TrackObject tempTrack = new TrackObject(tracks.get(actualTrackIndex.get(i)));
            if (tempTrack.getTrackID() == -1) {
                if (currentTrackIndex == i) {
                    UI.ClearTrack();
                    UI.ClearGraph();
                    removedCurrentSong = true;
                }
                model.remove(i);
                int index = actualTrackIndex.get(i);
                tracks.remove(index);
                actualTrackIndex.remove(i);
                if (currentTrackIndex > i) {
                    currentTrackIndex--;
                }
                for (int j = i; j < actualTrackIndex.size(); j++) {
                    actualTrackIndex.set(j, actualTrackIndex.get(j)-1);
                }
                i--;
            }
        }
        if (removedCurrentSong) {
            if (currentTrackIndex<actualTrackIndex.size()) {
                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                UI.Play(trackToPlay);
            }
        }
        this.setSelectedTrack(currentTrackIndex);
        setLabel();
        this.saveState();
        //resumeThreads();
    }

    /**
     * removes a specific track from the list
     * @param trackID
     */
    public void removeTrack(int trackID) {
        for (int i=0; i<actualTrackIndex.size(); i++) {
            if (tracks.get(actualTrackIndex.get(i)) == trackID) {
                //remove track in this position
                currentOrderIDs.remove(i);
                int index  =actualTrackIndex.get(i);
                tracks.remove(index);
                actualTrackIndex.remove(i);
                this.model.remove(i);
                if (currentTrackIndex == i) {
                    if (currentTrackIndex<actualTrackIndex.size()-1) {
                        TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                        UI.Play(trackToPlay);
                    } else {
                        currentTrackIndex--;
                        TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                        UI.Play(trackToPlay);
                    }
                } else if (currentTrackIndex > i) {
                    currentTrackIndex--;
                }
                for (int j = i; j < actualTrackIndex.size(); j++) {
                    actualTrackIndex.set(j, actualTrackIndex.get(j)-1);
                }
                i--;
            }
        }
        this.reload();
        this.setLabel();
        this.saveState();
    }

    /**
     * sets which track (by index) is selected in the list
     * @param index
     */
    public void setSelectedTrack(int index) {
        if (tracks.size() != 0) {
            if (index != -1) {
                this.setSelectedIndex(index);
                JScrollBar bar = UI.getPlayingListScrollBar();
                int val = Math.round(bar.getMaximum()-bar.getVisibleAmount());
                val =val * index;
                val  = (val/tracks.size());
                //val = val*(index+(index-(currentOrderIDs.size()/2)));
                //val = Math.round(val/currentOrderIDs.size());
                if (val> (bar.getMaximum()-bar.getVisibleAmount())) {
                    val = (bar.getMaximum()-bar.getVisibleAmount());
                }
                bar.setValue(val);
            } else {
                this.setSelectedIndex(-1);
            }
        }
    }

    /**
     * saves the current state of the now playing list so that when the program
     * is started again, it can resume where it left off
     */
    public void saveState() {
        checkThreads();
        ArrayList<Integer> IDs = new ArrayList<Integer>();
        for (int i=0; i<tracks.size(); i++) {
            IDs.add(tracks.get(i));
        }
        db.updatePlaylist(IDs, 1);
        UIUtilities util = new UIUtilities();
        try {
            util.config("playingListPosition", Integer.toString(currentTrackIndex));
        } catch (Exception ex) {
            ArmadilloLog.LogError("PLayng list cannot access config file - "+ex.toString());
            Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        this.resumeThreads();
    }


    /**
     * Calculates and sets the stats label
     */
    public void setLabel() {
        int numberOfTracks = tracks.size();
        String text;
        if (numberOfTracks == 1) {
            text = "1 Track";
            
        } else {
            text = numberOfTracks + " Tracks";
            
        }
        long entireLength = 0;
        for (int i =0; i<tracks.size(); i++) {
            String[] trackDetails = db.getTrack(tracks.get(i));
            entireLength = entireLength + Long.parseLong(trackDetails[4]);
        }
        if (numberOfTracks > 0) {
           text = text + " - " + UIUtilities.FormatTime(entireLength);
        }
        //code to get cumulative length of songs
        UI.setListLabelText(text);
    }

    /**
     * gets the currently playing track
     * @return
     */
    public TrackObject getCurrentTrack() {
        if ((currentTrackIndex < 0) || (currentTrackIndex > actualTrackIndex.size()-1)) {
            return null;
        } else {
            TrackObject trackToReturn = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
            return trackToReturn;
        }
    }
  
    /**
     * returns the track ID of the next song to play, should update currentTrackIndex
     * @return
     */
    public TrackObject getNextTrack() {
        if (actualTrackIndex.size() > 0) {
            if (loopOne) {
                TrackObject trackToReturn = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                return trackToReturn;
            } else {
                if (currentTrackIndex == actualTrackIndex.size()-1) {
                    if (loopAll) {
                        currentTrackIndex = 0;
                        this.setSelectedTrack(0);
                        TrackObject trackToReturn = new TrackObject(tracks.get(actualTrackIndex.get(0)));
                        return trackToReturn;
                    } else {
                        return null;
                    }
                } else {
                    currentTrackIndex++;
                    this.setSelectedTrack(currentTrackIndex);
                    TrackObject trackToReturn = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                    return trackToReturn;
                }
            }
        } else {
            return null;
        }
    }

    /**
     * gets the previous track in the list
     * @return
     */
    public TrackObject getPreviousTrack() {
        if (actualTrackIndex.size() > 0) {
            if (loopOne) {
                TrackObject trackToReturn = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                return trackToReturn;
            } else {
                if (currentTrackIndex == 0) {
                    if (loopAll) {
                        currentTrackIndex = actualTrackIndex.size()-1;
                        this.setSelectedTrack(currentTrackIndex);
                        TrackObject trackToReturn = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                        return trackToReturn;
                    } else {
                        TrackObject trackToReturn = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                        return trackToReturn;
                    }
                } else {
                    currentTrackIndex--;
                    this.setSelectedTrack(currentTrackIndex);
                    TrackObject trackToReturn = new TrackObject(tracks.get(actualTrackIndex.get(currentTrackIndex)));
                    return trackToReturn;
                }
            }
        } else {
            return null;
        }
    }

    /**
     * sets the single track loop on or off
     * @param onOrOff
     */
    public void setLoopSingle(boolean onOrOff) {
        loopOne = onOrOff;
        if (onOrOff == true) {
            loopAll = false;
        }
    }

    /**
     * sets the all track loop on or off
     * @param onOrOff
     */
    public void setLoopAll(boolean onOrOff) {
        loopAll = onOrOff;
        if (onOrOff == true) {
            loopOne = false;
        }
    }

    /**
     * sets shuffle (and shuffles) all tracks, or unshuffles them
     * @param onOrOff
     */
    public void setShuffle(boolean onOrOff) {
        shuffle = onOrOff;
        DefaultListModel newModel = new DefaultListModel();
        ArrayList<Integer> oldTrackIndexes = new ArrayList<Integer>(actualTrackIndex);
        int index3 = oldTrackIndexes.indexOf(new Integer(0));

        
        if (tracks.size() > 0) {
            int currentTrack =0;

            currentTrack = actualTrackIndex.get(currentTrackIndex);

        int resultingTrackHighlight=0;
            if (shuffle) {
                boolean changed= false;
                actualTrackIndex.clear();
                ArrayList<Integer> indexesLeft = new ArrayList();
                for (int i = 0; i<tracks.size(); i++) {
                    indexesLeft.add(i);
                }
                Random generator = new Random(System.currentTimeMillis());
                for (int i = 0; i<tracks.size(); i++) {
                    int index = generator.nextInt(indexesLeft.size());
                    actualTrackIndex.add(indexesLeft.get(index));
                    int nextTrackIndex = indexesLeft.get(index);
                    //model.addElement(tracks.get(indexesLeft.get(index)).getListData());                    
                    int index2 = oldTrackIndexes.indexOf(new Integer(nextTrackIndex));                    
                    newModel.addElement(model.getElementAt(index2));
                    if (indexesLeft.get(index) == currentTrack) {
                        this.setSelectedTrack(i);
                        resultingTrackHighlight = i;
                        currentTrackIndex = i;
                        this.setSelectedIndex(currentTrackIndex);                        
                        changed = true;
                    }

                    indexesLeft.remove(index);
                }
                this.model = newModel;
                this.setModel(newModel);
            } else {
                if (currentTrackIndex >=0) {
                    int currentTrackObjectIndex = actualTrackIndex.get(currentTrackIndex);
                    actualTrackIndex.clear();
                    for (int i=0; i<tracks.size(); i++) {
                        actualTrackIndex.add(i);
                        //oldTrackIndex                        
                        int oldIndex = oldTrackIndexes.indexOf(new Integer(i));
                            newModel.addElement(model.getElementAt(oldIndex));
                        if (currentTrack == i) {

                            resultingTrackHighlight = i;
                            currentTrackIndex = i;
                        }
                    }
                    this.model = newModel;
                    this.setModel(model);
                }
            }
            this.setSelectedTrack(resultingTrackHighlight);
        }
    }

    /**
     * clears the entire contents of the list
     */
    public void clear() {
        model.clear();
        tracks.clear();
        actualTrackIndex.clear();
        UI.clearShuffle();
    }


    /**
     * loads all the songs in the library (unused)
     */
    public void loadAllSongs() {
        clear();        
        addAllSongs();
    }
    /**
     * clears, then loads a single track
     * @param ID
     */
    public void loadTrack(int ID) {
        clear();        
        addTrack(ID);
    }
    /**
     * clears, then loads a playlist
     * @param ID
     */
    public void loadPlaylist(int ID) {
        clear();        
        addPlaylist(ID);
    }
    /**
     * clears, then loads an artist
     * @param ID
     */
    public void loadArtist(int ID) {
        clear();        
        addArtist(ID);
    }
    /**
     * clears, then loads a album
     * @param ID
     */
    public void loadAlbum(int ID) {
        clear();        
        addAlbum(ID);
    }
    /**
     * clears, then loads a genre
     * @param ID
     */
    public void loadGenre(int ID) {
        clear();      
        addGenre(ID);
    }

    /**
     * clears, then loads a BPM range
     * @param BPM
     */
    public void loadBPM(int BPM) {
        clear();
        addBPM(BPM);
    }

    /**
     * loads a file (depreciated)
     * @param location
     */
    public void loadFile(File location) {
        clear();        
        addFile(location);
    }

    /**
     * adds all songs to the queue
     */
    public void addAllSongs() {
        int sizeBeforeAdding =tracks.size();
        try {            
            ArrayList<String[]> contents = db.getLibrary();
            for (int i = 0; i<contents.size(); i++) {
                int bpm = db.getBPM(Integer.parseInt(contents.get(i)[6]));
                //double dec = db.getAverageDec(Integer.parseInt(contents.get(i)[6]));
                double dec =0;
                //TrackObject newTrack = new TrackObject(Integer.parseInt(contents.get(i)[6]), contents.get(i)[0],contents.get(i)[1],contents.get(i)[2],contents.get(i)[3], contents.get(i)[5], bpm,Long.parseLong(contents.get(i)[4]),dec);
                tracks.add(Integer.parseInt(contents.get(i)[6]));
                String listData = contents.get(i)[0] + " - " + contents.get(i)[1];
                model.addElement(listData);
                actualTrackIndex.add(tracks.size()-1);

            }
            
            if (sizeBeforeAdding == 0) {
                UI.playTimer.stop();
                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
                UI.Play(trackToPlay);
                this.setSelectedTrack(0);
                currentTrackIndex=0;
            }
        } catch (Exception error) {
            ArmadilloLog.LogError("Playing list cannot load from DB - "+error.toString());
            new DialogUI(17).setVisible(true);
            error.printStackTrace();
        }
        setLabel();
        this.saveState();    
    }
    /**
     * adds a set of tracks to the queue
     * @param IDs
     */
    public void addTracks(int[] IDs) {
        int sizeBeforeAdding = tracks.size();
        try {
            for (int i=0; i<IDs.length; i++) {
                String[] contents = db.getTrack(IDs[i]);
                int bpm = db.getBPM(Integer.parseInt(contents[6]));
                //double dec = db.getAverageDec(Integer.parseInt(contents[6]));
                double dec =0;
//                TrackObject newTrack = new TrackObject(Integer.parseInt(contents[6]), contents[0],contents[1],contents[2],contents[3], contents[5], bpm,Long.parseLong(contents[4]),dec);
//                tracks.add(newTrack);
                tracks.add(Integer.parseInt(contents[6]));
                String listData = contents[0] + " - " + contents[1];
                model.addElement(listData);
                actualTrackIndex.add(tracks.size()-1);
//                model.addElement(newTrack.getListData());
            }
        } catch (Exception error) {
            ArmadilloLog.LogError("Playing list cannot load from DB - "+error.toString());
            new DialogUI(17).setVisible(true);
            error.printStackTrace();
        }
        if (sizeBeforeAdding == 0) {
            UI.playTimer.stop();
            TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
            UI.Play(trackToPlay);
            this.setSelectedTrack(0);
            currentTrackIndex=0;
        }
        setLabel();
        this.saveState();
    }

    /**
     * adds a track to the queue
     * @param ID
     */
    public void addTrack(int ID) {
        int sizeBeforeAdding = tracks.size();
        try {
            String[] contents = db.getTrack(ID);
            int bpm = db.getBPM(Integer.parseInt(contents[6]));
            //double dec = db.getAverageDec(Integer.parseInt(contents[6]));
            double dec =0;
//            TrackObject newTrack = new TrackObject(Integer.parseInt(contents[6]), contents[0],contents[1],contents[2],contents[3], contents[5], bpm,Long.parseLong(contents[4]),dec);
//            tracks.add(newTrack);
            tracks.add(Integer.parseInt(contents[6]));
            String listData = contents[0] + " - " + contents[1];
            model.addElement(listData);
            actualTrackIndex.add(tracks.size()-1);
//            model.addElement(newTrack.getListData());
        } catch (Exception error) {
            ArmadilloLog.LogError("Playing list cannot load from DB - "+error.toString());
            new DialogUI(17).setVisible(true);
            error.printStackTrace();
        }
        if (sizeBeforeAdding == 0) {
            UI.playTimer.stop();
            TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
            UI.Play(trackToPlay);
            //UI.Play(tracks.get(actualTrackIndex.get(0)));
            this.setSelectedTrack(0);
            currentTrackIndex=0;
        }
        setLabel();
        this.saveState();
    }
    /**
     * adds a playlist to the queue
     * @param ID
     */
    public void addPlaylist(int ID) {
        int sizeBeforeAdding = tracks.size();
        try {
            ArrayList<String[]> contents = db.getPlaylistContents(ID);
            for (int i = 0; i<contents.size(); i++) {
                int bpm = db.getBPM(Integer.parseInt(contents.get(i)[6]));
                //double dec = db.getAverageDec(Integer.parseInt(contents.get(i)[6]));
                double dec =0;
                //TrackObject newTrack = new TrackObject(Integer.parseInt(contents.get(i)[6]), contents.get(i)[0],contents.get(i)[1],contents.get(i)[2],contents.get(i)[3], contents.get(i)[5], bpm,Long.parseLong(contents.get(i)[4]),dec);
                //tracks.add(newTrack);
                tracks.add(Integer.parseInt(contents.get(i)[6]));
                String listData = contents.get(i)[0] + " - " + contents.get(i)[1];
                model.addElement(listData);
                actualTrackIndex.add(tracks.size()-1);
                //model.addElement(newTrack.getListData());
            }

            if (sizeBeforeAdding == 0) {
                UI.playTimer.stop();
                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
                UI.Play(trackToPlay);
                //UI.Play(tracks.get(actualTrackIndex.get(0)));
                this.setSelectedTrack(0);
                currentTrackIndex=0;
            }
        } catch (Exception error) {
            ArmadilloLog.LogError("Playing list cannot load from DB - "+error.toString());
            new DialogUI(17).setVisible(true);
            error.printStackTrace();
        }
        setLabel();
        this.saveState();
    }
    /**
     * adds a artist to the queue
     * @param ID
     */
    public void addArtist(int ID) {
        int sizeBeforeAdding = tracks.size();
        try {
            ArrayList<String[]> contents = db.getTracksByTag(ID);
            for (int i = 0; i<contents.size(); i++) {
                int bpm = db.getBPM(Integer.parseInt(contents.get(i)[6]));
                //double dec = db.getAverageDec(Integer.parseInt(contents.get(i)[6]));
                double dec =0;
                //TrackObject newTrack = new TrackObject(Integer.parseInt(contents.get(i)[6]), contents.get(i)[0],contents.get(i)[1],contents.get(i)[2],contents.get(i)[3], contents.get(i)[5], bpm,Long.parseLong(contents.get(i)[4]),dec);
                //tracks.add(newTrack);
                tracks.add(Integer.parseInt(contents.get(i)[6]));
                String listData = contents.get(i)[0] + " - " + contents.get(i)[1];
                model.addElement(listData);
                actualTrackIndex.add(tracks.size()-1);
                //model.addElement(newTrack.getListData());
            }

            if (sizeBeforeAdding == 0) {
                UI.playTimer.stop();
                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
                UI.Play(trackToPlay);
                //UI.Play(tracks.get(actualTrackIndex.get(0)));
                this.setSelectedTrack(0);
                currentTrackIndex=0;
            }
        } catch (Exception error) {
            ArmadilloLog.LogError("Playing list cannot load from DB - "+error.toString());
            new DialogUI(17).setVisible(true);
            error.printStackTrace();
        }
        setLabel();
        this.saveState();
    }
    /**
     * adds a album to the queue
     * @param ID
     */
    public void addAlbum(int ID) {
        int sizeBeforeAdding = tracks.size();
        try {
            ArrayList<String[]> contents = db.getTracksByTag(ID);
            for (int i = 0; i<contents.size(); i++) {
                int bpm = db.getBPM(Integer.parseInt(contents.get(i)[6]));
                //double dec = db.getAverageDec(Integer.parseInt(contents.get(i)[6]));
                double dec =0;
                //TrackObject newTrack = new TrackObject(Integer.parseInt(contents.get(i)[6]), contents.get(i)[0],contents.get(i)[1],contents.get(i)[2],contents.get(i)[3], contents.get(i)[5], bpm,Long.parseLong(contents.get(i)[4]),dec);
                //tracks.add(newTrack);
                tracks.add(Integer.parseInt(contents.get(i)[6]));
                String listData = contents.get(i)[0] + " - " + contents.get(i)[1];
                model.addElement(listData);
                actualTrackIndex.add(tracks.size()-1);
                //model.addElement(newTrack.getListData());
            }

            if (sizeBeforeAdding == 0) {
                UI.playTimer.stop();
                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
                UI.Play(trackToPlay);
                //UI.Play(tracks.get(actualTrackIndex.get(0)));
                this.setSelectedTrack(0);
                currentTrackIndex=0;
            }
        } catch (Exception error) {
            ArmadilloLog.LogError("Playing list cannot load from DB - "+error.toString());
            new DialogUI(17).setVisible(true);
            error.printStackTrace();
        }
        setLabel();
        this.saveState();
    }

    /**
     * adds a genre to the queue
     * @param ID
     */
    public void addGenre(int ID) {
        int sizeBeforeAdding = tracks.size();
        try {
            ArrayList<String[]> contents = db.getTracksByTag(ID);
            for (int i = 0; i<contents.size(); i++) {
                int bpm = db.getBPM(Integer.parseInt(contents.get(i)[6]));
                //double dec = db.getAverageDec(Integer.parseInt(contents.get(i)[6]));
                double dec =0;
                //TrackObject newTrack = new TrackObject(Integer.parseInt(contents.get(i)[6]), contents.get(i)[0],contents.get(i)[1],contents.get(i)[2],contents.get(i)[3], contents.get(i)[5], bpm,Long.parseLong(contents.get(i)[4]),dec);
                //tracks.add(newTrack);
                tracks.add(Integer.parseInt(contents.get(i)[6]));
                String listData = contents.get(i)[0] + " - " + contents.get(i)[1];
                model.addElement(listData);
                actualTrackIndex.add(tracks.size()-1);
                //model.addElement(newTrack.getListData());
            }

            if (sizeBeforeAdding == 0) {
                UI.playTimer.stop();
                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
                UI.Play(trackToPlay);
                //UI.Play(tracks.get(actualTrackIndex.get(0)));
                this.setSelectedTrack(0);
                currentTrackIndex=0;
            }
        } catch (Exception error) {
            ArmadilloLog.LogError("Playing list cannot load from DB - "+error.toString());
            new DialogUI(17).setVisible(true);
            error.printStackTrace();
        }
        setLabel();
        this.saveState();
    }

    /**
     * adds a bpm range to the queue
     * @param BPM
     */
    public void addBPM(int BPM) {
        int sizeBeforeAdding = tracks.size();
        try {
            ArrayList<Integer> trackIDs = db.getTracks(BPM);
            ArrayList<String[]> contents = new ArrayList();
            for (int i = 0 ; i< trackIDs.size(); i++) {
                contents.add(db.getTrack(trackIDs.get(i)));
            }
            for (int i = 0; i<contents.size(); i++) {
                int bpm = db.getBPM(Integer.parseInt(contents.get(i)[6]));
                //double dec = db.getAverageDec(Integer.parseInt(contents.get(i)[6]));
                double dec =0;
                //TrackObject newTrack = new TrackObject(Integer.parseInt(contents.get(i)[6]), contents.get(i)[0],contents.get(i)[1],contents.get(i)[2],contents.get(i)[3], contents.get(i)[5], bpm,Long.parseLong(contents.get(i)[4]),dec);
                //tracks.add(newTrack);
                tracks.add(Integer.parseInt(contents.get(i)[6]));
                String listData = contents.get(i)[0] + " - " + contents.get(i)[1];
                model.addElement(listData);
                actualTrackIndex.add(tracks.size()-1);
                //model.addElement(newTrack.getListData());
            }

            if (sizeBeforeAdding == 0) {
                UI.playTimer.stop();
                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
                UI.Play(trackToPlay);
                //UI.Play(tracks.get(actualTrackIndex.get(0)));
                this.setSelectedTrack(0);
                currentTrackIndex=0;
            }
        } catch (Exception error) {
            ArmadilloLog.LogError("Playing list cannot load from DB - "+error.toString());
            new DialogUI(17).setVisible(true);
            error.printStackTrace();
        }
        setLabel();
        this.saveState();
    }

    /**
     * adds a file to the queue (not in final release)
     * @param location
     */
    public void addFile(File location) {

    }


    /**
     * adds an arraylist of tracks to the queue
     * @param newTracks
     */
    public void addTracks(ArrayList<Integer> newTracks) {
        if (newTracks.size() >0) {
            int sizeBeforeAdding = tracks.size();
            for (int i=0; i<newTracks.size(); i++) {
                tracks.add(newTracks.get(i));
                actualTrackIndex.add(tracks.size()-1);
                String[] contents = db.getTrack(newTracks.get(i));
                String listData = contents[0] + " - " + contents[1];
                model.addElement(listData);
                //model.addElement(newTracks.get(i).getListData());
            }
            if (sizeBeforeAdding == 0) {
                UI.playTimer.stop();
                TrackObject trackToPlay = new TrackObject(tracks.get(actualTrackIndex.get(0)));
                UI.Play(trackToPlay);
                //UI.Play(tracks.get(actualTrackIndex.get(0)));
                this.setSelectedTrack(0);
                currentTrackIndex=0;
            }
            setLabel();
            this.saveState();
        }
    }

    /**
     * closes the playing list's db connection
     */
    public void closePlayingListDbObject()
    {
        db.closeConnection();
    }

    private void checkThreads()
    {
        // pause the analysing thread (if its active)
        this.isAnalysing = false;
        if (UI.getbpmThread() != null)
        {
            if (UI.getbpmThread().isAlive())
                this.isAnalysing = UI.getbpmThread().pauseAnalysing(false);
        }
    }

    private void resumeThreads()
    {
        if (isAnalysing)
            UI.resumeAnalysing();
        this.isAnalysing = false;
    }
}
