/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Author: Jon
 * This class extends the JTable class with all relevent functions
 * such as loading a playlist given its ID number, and also has
 * all mouse listeners inside aswell
 */

package armadilloUI;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JTable;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableCellRenderer;
import javax.swing.ListSelectionModel;
import java.awt.Component;
import java.util.ArrayList;
import javax.swing.JPopupMenu;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;


/**
 * This class displays low level data from the database, i.e. track information
 * also performs operations on these tracks and passes the to the playing list
 * @author jonathan
 */
public class PlaylistTable  extends JTable {

    private DefaultTableModel playlistTableModel;    
    private ArrayList<Integer> tracks = new ArrayList<Integer>();
    private int currentTrackID;
    private int row;
    private String listType;
    private int currentPlaylistID;
    private String currentPlaylistName="";
    private PlayingList list;
    private NavTree tree;
    private ArmadilloUI UI;
    private boolean mainTable = false;
    private ArmadilloDB dbTable;
    private long totalLength=0;
    int[] selectedRows;
    private boolean isAnalysing;

    /**
     * Constructer, sets up ui, and adds listeners
     */
    public PlaylistTable() {
        ArmadilloLog.LogOperation("Constructing playlist table");
        this.pauseOtherThreads();
        openTableDbObject();
        this.setAutoCreateRowSorter(true);
        KeyListener playlistTableKeyListener = new KeyAdapter() {
            @Override
            @SuppressWarnings({"static-access", "static-access", "static-access"})
            public void keyTyped(KeyEvent e) {
                int key = e.getKeyChar();
                final PlaylistTable table = (PlaylistTable) e.getComponent();
                int[] rows = table.getSelectedRows();
                if (rows.length == 1) {
                    if (rows[0] == 0 ) {
                        rows[0] = table.getRowCount()-1;
                    } else {
                        rows[0] = rows[0]-1;
                    }
                }
                
                int[] ids= new int[rows.length];
                for (int i=0; i<rows.length; i++) {
                    rows[i]=table.convertRowIndexToModel(rows[i]);
                    ids[i]=table.tracks.get(rows[i]);
                }
                if (key==KeyEvent.VK_ENTER) {
                    if (table.list != null) {
                        table.list.addTracks(ids);
                    }
                    //for (int i=0; i<rows.length; i++) {
                    //    table.list.addTrack(table.trackIDs.get(rows[i]));
                    //}
                } else if ((key==KeyEvent.VK_DELETE) || (key==KeyEvent.VK_BACK_SPACE)) {

                    if (listType.equals("playlist")) {
                        if (ids.length==1) {
                            int modelRow = table.convertRowIndexToModel(table.getSelectedRow());
                            dbTable.removeTrackFromPlaylist(tracks.get(modelRow), currentPlaylistID, modelRow);
                        } else {
                            for (int i=0; i<ids.length; i++) {
                                //System.out.println("id ="+table.trackIDs.get(selectedRows[i])+" P ID="+currentPlaylistID+"row="+(selectedRows[i]-i));
                                dbTable.removeTrackFromPlaylist(ids[i], currentPlaylistID, rows[i]-i);
                                //System.out.println("-------");
                                //System.out.println(db.getPlaylistContents(currentPlaylistID));
                            }
                        }
                        table.reload();
//                        for (int i=0; i<rows.length; i++) {
//                            db.removeTrackFromPlaylist(IDs[i], currentPlaylistID, rows[i]-i);
//
//                        }
//                        table.reload();
                    } else {
                        try {
                            if (ids.length==1) {
                                int modelRow = table.convertRowIndexToModel(table.getSelectedRow());
                                int[] deleteTrackID = new int[1];
                                deleteTrackID[0] = tracks.get(modelRow);
                                new DeleteUI(deleteTrackID,UI).setVisible(true);
                            } else {
                                new DeleteUI(ids,UI).setVisible(true);
                            }
                            table.reload();
                        } catch (Exception ex) {
                            new DialogUI(2).setVisible(true);
                        }
                    }
                }
            }
        };
        if (this.list==null) {
            this.addKeyListener(playlistTableKeyListener);
        }
        if (isAnalysing)
                UI.resumeAnalysing();
        isAnalysing = false;
    }

    /**
     * sets whether this object is the main table or one of the table in create playlist
     * UI
     * @param bool
     */
    public void isMainPlaylistTable(boolean bool){
       mainTable = bool;

    }


    /**
     * performs the set up of the table properties (column titles etc.)
     */
    public void setUp() {
        // = new ArrayList();
        playlistTableModel = new DefaultTableModel() {
            @Override
            public boolean isCellEditable(int row, int column){
                //makes the table uneditable
                return false;
            }
        };
        playlistTableModel.addColumn("Track");
        playlistTableModel.addColumn("Artist");
        playlistTableModel.addColumn("Album");
        playlistTableModel.addColumn("Genre");
        playlistTableModel.addColumn("BPM");
        playlistTableModel.addColumn("Length");
        setModel(playlistTableModel);

        //add mouse listener
         MouseListener fileNavTreeListener = new MouseAdapter() {
            @Override
            public void mousePressed(MouseEvent click) {
                int clickCount = click.getClickCount();
                final PlaylistTable table =(PlaylistTable) click.getComponent();
                DefaultTableModel model = (DefaultTableModel) table.getModel();
                Point mousePoint = new Point(click.getX(),click.getY());
                int oldRow = table.rowAtPoint(mousePoint);
                row = table.convertRowIndexToModel(oldRow);
                currentTrackID = tracks.get(row);
                selectedRows=table.getSelectedRows();
                boolean clickedSelection = false;
                for (int i=0; i<selectedRows.length; i++) {
                    if (selectedRows[i]==oldRow) {
                        clickedSelection=true;
                    }
                    selectedRows[i]=table.convertRowIndexToModel(selectedRows[i]);
                }

                if (clickedSelection == false) {
                    selectedRows = new int[1];
                    selectedRows[0] = row;

                    table.setRowSelectionInterval(oldRow, oldRow);
                }
                if ((selectedRows.length > 0) && (selectedRows[0]!=-1)) {
                    //if (click.isPopupTrigger()) {
                    if (click.getButton() == 3) {
                        JPopupMenu menu = new JPopupMenu();
                        if (mainTable) {
                            JMenuItem play = new JMenuItem("Play");
                            ActionListener playListener = new ActionListener() {
                                public void actionPerformed(ActionEvent playEvent) {
                                    list.clear();
                                    for (int i=0; i<selectedRows.length; i++) {
                                        list.addTrack(table.tracks.get(selectedRows[i]));
                                    }
                                }
                            };
                            play.addActionListener(playListener);
                            menu.add(play);

                            JMenuItem queue = new JMenuItem("Add to queue");
                            ActionListener queueListener = new ActionListener() {
                                public void actionPerformed(ActionEvent queueEvent) {
                                    for (int i=0; i<selectedRows.length; i++) {
                                        list.addTrack(table.tracks.get(selectedRows[i]));
                                    }
                                }
                            };
                            queue.addActionListener(queueListener);
                            menu.add(queue);

                            JMenu add = new JMenu("Add to playlist");
                            try {
                                ArrayList<String> playlistsNames = dbTable.getPlaylistsNames();
                                for (int i=0; i<playlistsNames.size(); i++)
                                {
                                    final String playlistName = playlistsNames.get(i);
                                    JMenuItem playlistItem = new JMenuItem(playlistsNames.get(i));
                                    add.add(playlistItem);
                                    playlistItem.addActionListener(new java.awt.event.ActionListener() {
                                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                            int playlistId = dbTable.getPlayListID(playlistName);
                                            int size = dbTable.getPlaylistContents(playlistId).size();
                                            for (int i=0; i<selectedRows.length; i++) {
                                                pauseOtherThreads();
                                                dbTable.addTrackToPlaylist(playlistId, table.tracks.get(selectedRows[i]),size+i);
                                                if (isAnalysing)
                                                    UI.resumeAnalysing();
                                                isAnalysing = false;
                                            }
                                            
                                            //UI.getTree().reload();
                                            UI.getTable().reload();
                                        }
                                    });
                                }
                            } catch (Exception ex) {
                                ArmadilloLog.LogError("Playlist table cannot load from DB - "+ex.toString());
                                Logger.getLogger(PlaylistTable.class.getName()).log(Level.SEVERE, null, ex);
                            }
                            menu.add(add);

                            JSeparator separator = new JSeparator();
                            menu.add(separator);
                            String editText="Edit track";

                            JMenuItem edit = new JMenuItem(editText);
                            ActionListener editListener = new ActionListener() {
                                public void actionPerformed(ActionEvent editEvent) {
                                    for (int i=0; i<selectedRows.length; i++) {
                                        new EditTrackUI(table.tracks.get(selectedRows[i]), getTableObject(),tree,list, UI).setVisible(true);
                                    }

                                }
                            };
                            edit.addActionListener(editListener);
                            menu.add(edit);

                            if (listType.equals("playlist")) {
                                JMenuItem remove = new JMenuItem("Remove song from "+dbTable.getPlayListName(currentPlaylistID)+" playlist");
                                ActionListener removeListener = new ActionListener() {
                                    public void actionPerformed(ActionEvent removeEvent) {
                                        try {
                                            //System.out.println("-------");
                                            //db.showAllPlaylistTracks();
                                            //System.out.println("-------");
                                            //System.out.println(db.getPlaylistContents(currentPlaylistID));
                                            for (int i=0; i<selectedRows.length; i++) {
                                                //System.out.println("id ="+table.trackIDs.get(selectedRows[i])+" P ID="+currentPlaylistID+"row="+(selectedRows[i]-i));
                                                pauseOtherThreads();
                                                dbTable.removeTrackFromPlaylist(table.tracks.get(selectedRows[i]), currentPlaylistID, selectedRows[i]-i);
                                                if (isAnalysing)
                                                    UI.resumeAnalysing();
                                                isAnalysing = false;
                                                //System.out.println("-------");
                                                //System.out.println(db.getPlaylistContents(currentPlaylistID));
                                            }
                                            table.reload();
                                            //System.out.println("-------");
                                        } catch (Exception ex) {
                                            ArmadilloLog.LogError("Playlist table cannot remove from from DB - "+ex.toString());
                                            new DialogUI(2).setVisible(true);
                                        }

                                    }
                                };
                                remove.addActionListener(removeListener);
                                menu.add(remove);
                            } else {
                                JMenuItem delete = new JMenuItem("Delete song from library");
                                ActionListener deleteListener = new ActionListener() {
                                    public void actionPerformed(ActionEvent deleteEvent) {
                                        try {
                                            int [] IDs = new int[selectedRows.length];
                                            for (int i=0; i<selectedRows.length; i++) {
                                                IDs[i]=table.tracks.get(selectedRows[i]);
                                            }
                                            new DeleteUI(IDs,UI).setVisible(true);                                            
                                            table.reload();
                                        } catch (Exception ex) {
                                            new DialogUI(2).setVisible(true);
                                        }
                                    }
                                };
                                delete.addActionListener(deleteListener);
                                menu.add(delete);
                            }
                        } 
                        menu.show(click.getComponent(),click.getX(),click.getY());                    

                    } else if (clickCount == 2) {
                        if (mainTable) {
                            list.addTrack(currentTrackID);
                        } 
                    }
                }
            }
        };
        this.addMouseListener(fileNavTreeListener);



        this.viewPlaylist("playlist", 1, "Last played");
        packColumnWidth();
    }

    /**
     * sets which playing list to pass data to
     * @param passedList
     */
    public void setPlayingList(PlayingList passedList) {
        list = passedList;
    }

    /**
     * sets what UI to call methods in
     * @param UI
     * @param tree
     */
    public void setUI(ArmadilloUI UI, NavTree tree) {
        this.tree = tree;
        this.UI = UI;
    }

    /**
     * returns what UI this class is currently using
     * @return
     */
    public ArmadilloUI getArmadilloUI() {
        return UI;
    }

    /**
     * reloads only the track with trackID
     * @param trackID
     */
    public void reload(int trackID) {
        for (int i=0; i<tracks.size(); i++) {
            if (tracks.get(i)==trackID) {
                TrackObject tempTrack = new TrackObject(tracks.get(i));                
                if (tempTrack.getTrackID() != -1) {
                    playlistTableModel.removeRow(i);
                    playlistTableModel.insertRow(i, tempTrack.getTableData());
                } else {
                    tracks.remove(i);
                    playlistTableModel.removeRow(i);
                    this.reloadStatsText();
                }
            }
        }
    }

    /**
     * reloads the entire table
     */
    public void reload() {
        if (listType != null) {
            viewPlaylist(listType,currentPlaylistID,currentPlaylistName);
        } 
    }

    /**
     * recalculates and displays the stats label
     */
    public void reloadStatsText() {
        String text ="        " + Integer.toString(playlistTableModel.getRowCount());
        if (playlistTableModel.getRowCount() == 1) {
            text = text + " Track";
        } else {
            text = text + " Tracks";
        }

        totalLength = 0;
        for (int i =0; i<tracks.size(); i++) {
            String[] trackDetails = dbTable.getTrack(tracks.get(i));
            if (trackDetails != null) {
                totalLength = totalLength + Long.parseLong(trackDetails[4]);
            }
        }
        if (playlistTableModel.getRowCount() != 0) {
            text = text + " - " + UIUtilities.FormatTime(totalLength);
        }
        if (mainTable){
            UI.setTableLabelText(text);
            UI.setTableTitleText(currentPlaylistName);
        }
    }

    /**
     * adds the passed tracks to the table
     * @param track_ids
     */
    public void addTracks(int[] track_ids) {
        for (int i=0; i<track_ids.length; i++) {
            TrackObject newTrack = new TrackObject(track_ids[i]);
            tracks.add(track_ids[i]);
            playlistTableModel.addRow(newTrack.getTableData());
        }
        packColumnWidth();
        reloadStatsText();
    }

    /**
     * Only use when adding a single track, do not call in forloop, use addTracks instead
     * @param track_id
     */
    public void addTrack(int track_id) {
        TrackObject newTrack = new TrackObject(track_id);
        tracks.add(track_id);
        playlistTableModel.addRow(newTrack.getTableData());
        packColumnWidth();
        reloadStatsText();
    }

    /**
     * views a playlist in the table of the passed type
     * @param type
     * @param playlist_id
     * @param name
     */
    public void viewPlaylist(String type, int playlist_id, String name) {
        ArrayList<String[]> results = null;
            try {
                listType = type;
                currentPlaylistName=name;
                currentPlaylistID = playlist_id;
                if (type.equals("all songs")) {
                    //load library from DB
                                //resets table     

                    results = dbTable.getLibrary();

                } else if (type.equals("artist")) {        

                    results = dbTable.getTracksByTag(playlist_id);
                } else if (type.equals("album")) {

                    results = dbTable.getTracksByTag(playlist_id);
                } else if (type.equals("genre")) {

                    results = dbTable.getTracksByTag(playlist_id);
                } else if (type.equals("playlist")) {

                    results = dbTable.getPlaylistContents(playlist_id);
                } else if (type.equals("bpm")) {

                    ArrayList<Integer> bpmIDs =dbTable.getTracks(currentPlaylistID);
                    results = new ArrayList();
                    for (int i = 0 ; i< bpmIDs.size(); i++) {
                        results.add(dbTable.getTrack(bpmIDs.get(i)));
                    }                    
                }
            } catch (Exception error) {
                ArmadilloLog.LogError("Playlist table cannot display in table- "+error.toString());
                System.err.println("Error - cannot view " + type + " in table");
            }
            if (results != null) {
                resetTable();
                setUpContents(results);
            }        
    }
    

    /**
     * packs the column widths so each column is just the right size for the longest data
     * contained within it
     */
    public void packColumnWidth() {
        DefaultTableColumnModel model =(DefaultTableColumnModel) getColumnModel();
        int totalWidth = 0;
        for (int k=0; k<model.getColumnCount(); k++) {
            TableColumn column = model.getColumn(k);
            int largestwidth = 0;
            TableCellRenderer cell = column.getHeaderRenderer();
            //Component component = cell.getTableCellRendererComponent(this, column.getHeaderValue(), false, false, 0, 0);
            //largestwidth = component.getPreferredSize().getWidth();
            for (int i=0; i<getRowCount(); i++) {
                //cell = getCellRenderer(i, k);
                //component = cell.getTableCellRendererComponent(this, getValueAt(i,k), false, false, k, i);
                if (this.getValueAt(i, k) != null) {
                    int len = this.getValueAt(i,k).toString().length();
                    len = len * 10 ;
                    if (largestwidth < len) {
                        largestwidth = len;
                    }
                }
            }
            if (largestwidth > 170) {
                largestwidth = 170;
            }
            column.setPreferredWidth((int) largestwidth+5);
            column.setWidth(largestwidth+5);            
            totalWidth = totalWidth + largestwidth;
        }
        if (totalWidth < this.getParent().getWidth()) {            
            setAutoResizeMode(AUTO_RESIZE_SUBSEQUENT_COLUMNS);            
        } else {
            this.setAutoResizeMode(AUTO_RESIZE_OFF);
        }
    }

    /**
     * returns all the tracks id currently in the table
     * @return
     */
    public int[] getTrackIDs()
    {
        int[] track_ids = new int[tracks.size()];
        for (int i=0; i<tracks.size(); i++) {
            track_ids[i] = tracks.get(i);
        }
        return track_ids;
    }

    /**
     * resets the table
     */
    public void resetTable()
    {
        playlistTableModel = new DefaultTableModel() {
            @Override
            public boolean isCellEditable(int row, int column){
                //makes the table uneditable
                return false;
            }
        };
        playlistTableModel.addColumn("Track");
        playlistTableModel.addColumn("Artist");
        playlistTableModel.addColumn("Album");
        playlistTableModel.addColumn("Genre");
        playlistTableModel.addColumn("BPM");
        playlistTableModel.addColumn("Length");
        setModel(playlistTableModel);
        tracks.clear();
        currentTrackID = -1;
    }


    /**
     * formats and inserts the passed track data for insertion into the table
     * @param contents
     */
    public void setUpContents(ArrayList<String[]> contents)
    {
        for (int i = 0; i<contents.size(); i++) {
            String track_ID = contents.get(i)[6];
            int trackID = Integer.parseInt(track_ID);
            int bpm = dbTable.getBPM(trackID);
            //double dec = db.getAverageDec(Integer.parseInt(contents.get(i)[6]));
            double dec =0;
//            TrackObject newTrack = new TrackObject(Integer.parseInt(contents.get(i)[6]), contents.get(i)[0],contents.get(i)[1],contents.get(i)[2],contents.get(i)[3], contents.get(i)[5], bpm,Long.parseLong(contents.get(i)[4]),dec);
//            playlistTableModel.addRow(newTrack.getTableData());

            String[] rowData = new String[6];
            rowData[0] = contents.get(i)[0]; //title
            rowData[1] = contents.get(i)[1]; //artist
            rowData[2] = contents.get(i)[2]; //album
            rowData[3] = contents.get(i)[3]; //genre

            if (bpm == -1) {
                rowData[4] = "N/A";
            } else {
                rowData[4] = Integer.toString(bpm);
            }
            double mins = Double.parseDouble(contents.get(i)[4]);
            rowData[5] = UIUtilities.FormatTime((long) mins); //length
            playlistTableModel.addRow(rowData);
            tracks.add(Integer.parseInt(contents.get(i)[6]));
        }
        packColumnWidth();
        reloadStatsText();
    }

    /**
     * returns IDs of the currently selected track, or -1 if nothing is selected
     * @return
     */
    public int[] getCurrentSelection()
    {
        int currentRowID = -1;
        int[] rows = this.getSelectedRows();
        for (int i=0; i<rows.length; i++) {
            rows[i]=this.tracks.get(this.convertRowIndexToModel(rows[i]));
            //rows[i]=this.trackIDs.get(this.convertRowIndexToModel(rows[i]));
        }
        return rows;
    }

    /**
     * tells the playing list to play the contents of this table
     */
    public void playContents() {
        //passes trackID, name, artist and length to playingList
        if (this.mainTable) {
            list.clear();
            list.addTracks(tracks);
        }
    }

    /**
     * return index of selected row
     * @return
     */
    public int[] getCurrentRow() {
        int[] rows = this.getSelectedRows();
        for (int i=0; i<rows.length; i++) {
            rows[i] = this.convertRowIndexToModel(rows[i]);
        }
        return rows;
    }

    /**
     * returns the current playlist type currently in the table
     * @return
     */
    public String getCurrentPlaylistType() {
        return this.listType;
    }

    /**
     * returns the unique identifier of the playlist in the table
     * @return
     */
    public int getCurrentPlaylistID() {
        return currentPlaylistID;
    }

    /**
     *  returns reference to this PlaylistTable object
     * @return
     */
    public PlaylistTable getTableObject()
    {
        return this;
    }

    // open db object used within this class
    private void openTableDbObject()
    {
        try {
            dbTable = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
        } catch (Exception ex) {
            ArmadilloLog.LogError("Playlist table cannot open DB - "+ex.toString());
            System.err.println("dbTable");
            Logger.getLogger(PlaylistTable.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     *  close db object used within this class
     */
    public void closeTableDbObject()
    {
        dbTable.closeConnection();
    }

    private void pauseOtherThreads()
    {
        // first pause the analysing thread (if its active)
        this.isAnalysing = false;
        try
        {
            if (UI.getbpmThread() != null)
            {
                if (UI.getbpmThread().isAlive())
                    this.isAnalysing = UI.getbpmThread().pauseAnalysing(false);
            }
        }
        catch (NullPointerException nullex)
        {
            this.isAnalysing = false;
        }
    }
}
