/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * PreferencesUI.java
 *
 * Created on 02-Feb-2010, 16:15:19
 */

package armadilloUI;

import java.awt.Color;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This class displays a preferences window and deals with the changes in preferences
 * made by the user
 * @author Kate
 */
public class PreferencesUI extends javax.swing.JFrame {

    UIUtilities util;
    ArmadilloUI mainUI;
    PreferencesUI pUI;
    boolean frame;
    // graph defaults
    /**
     * value for high graph detail
     */
    protected static final int gdHigh = 1920;
    /**
     * value for medium graph detail
     */
    protected static final int gdMed = 1024;
    /**
     * value for low graph detal
     */
    protected static final int gdLow = 640;

    /** Creates new form PreferencesUI
     * @param Util
     * @param UI
     */
    public PreferencesUI(UIUtilities Util, ArmadilloUI UI) {
        ArmadilloLog.LogOperation("Constructing Preferences UI");
        this.util = Util;
        this.mainUI = UI;
        this.pUI = this;
        initComponents();
        restartLabel.setVisible(false);
        restartLabel.setForeground(Color.red);
        this.setLocationRelativeTo(null);
        ChangeListener colorchange = new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                String colour =Integer.toString(jColorChooser1.getColor().getRGB());
                pUI.repaint();
                if (jToggleButton1.isSelected()) {
                    canvas1.setBackground(jColorChooser1.getColor());
                    try {
                        util.config("customFontColour", colour);                        
                    } catch (Exception ex) {
                        Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
                    }
                } else if (jToggleButton2.isSelected()) {
                    canvas2.setBackground(jColorChooser1.getColor());
                    try {
                        util.config("customBackgroundColour", colour);                        
                    } catch (Exception ex) {
                        Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
                    }
                } else if (jToggleButton3.isSelected()) {
                    canvas3.setBackground(jColorChooser1.getColor());
                    try {
                        util.config("customBorderColour", colour);                        
                    } catch (Exception ex) {
                        Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
                    }
                } else if (jToggleButton4.isSelected()) {
                    canvas4.setBackground(jColorChooser1.getColor());
                    try {
                        util.config("customPanelBackgroundColour", colour);
                    } catch (Exception ex) {
                        Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
                try {
                    if (util.config("skin").equals("Custom")) {
                        mainUI.skinsMenuItemActionPerformed(null, "Custom");
                        mainUI.repaint();
                    }
                } catch (Exception ex) {
                        ArmadilloLog.LogError("Preferences UI cannot write to config file - "+ex.toString());
                        Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        };
        jColorChooser1.getSelectionModel().addChangeListener(colorchange);        
        try {
            String font = util.config("customFontColour");
            String background = util.config("customBackgroundColour");
            String border = util.config("customBorderColour");
            String panelBackground = util.config("customPanelBackgroundColour");
            //canvas1.setBackground(font);
            java.awt.Color bgcolor = new java.awt.Color(0, 0, 0);
            java.awt.Color fontcolor = new java.awt.Color(255, 255, 255);
            java.awt.Color scrollcolor = new java.awt.Color(32, 32, 32);
            java.awt.Color scrollcolor2 = new java.awt.Color(64, 64, 64);
            if (font == null) {                
                util.config("customFontColour",Integer.toString(fontcolor.getRGB()));
            } else {
                fontcolor = new java.awt.Color(Integer.parseInt(font));
            }
            if (background == null) {                
                util.config("customBackgroundColour",Integer.toString(bgcolor.getRGB()));
            } else {
                bgcolor = new java.awt.Color(Integer.parseInt(background));
            }
            if (border == null) {                
                util.config("customBorderColour",Integer.toString(scrollcolor2.getRGB()));
            } else {
                scrollcolor2 = new java.awt.Color(Integer.parseInt(border));
            }
            if (panelBackground == null) {                
                util.config("customPanelBackgroundColour",Integer.toString(scrollcolor.getRGB()));
            } else {
                scrollcolor = new java.awt.Color(Integer.parseInt(panelBackground));
            }
            canvas1.setBackground(fontcolor);
            canvas2.setBackground(bgcolor);
            canvas3.setBackground(scrollcolor2);
            canvas4.setBackground(scrollcolor);

        } catch (Exception ex) {
            ArmadilloLog.LogError("Preferences UI cannot read/write config file - "+ex.toString());
            Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
        }

        // graph settings

        String strGraphSize = null, strGraphDetail = null;
        float graphSize; int graphDetail;

        try
        {
            strGraphSize = util.config("graphSize");
            strGraphDetail = util.config("graphDetail");
        }
        catch (Exception e)
        {
            ArmadilloLog.LogError("Preferences UI cannot read config file - "+e.toString());
            e.printStackTrace();
        }


        if (strGraphSize == null)
            strGraphSize = "0.25";
        if (strGraphDetail == null)
            strGraphDetail = Integer.toString(gdMed);
        
        try
        {
            graphSize = Float.parseFloat(strGraphSize);
        }
        catch (NumberFormatException e)
        {
            graphSize = 0.25f;
        }
        sliderGraph.setValue(Math.round(100 * graphSize));
        
        try
        {
            graphDetail = Integer.parseInt(strGraphDetail);
        }
        catch (NumberFormatException e)
        {
            graphDetail = gdMed;
        }
        
        switch (graphDetail)
        {
            case gdLow:
                rGraphLow.setSelected(true);
                break;
            case gdMed:
                rGraphMed.setSelected(true);
                break;
            case gdHigh:
                rGraphHigh.setSelected(true);
                break;
        }

        sliderGraph.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                float size = (float)sliderGraph.getValue() / (float)100;
                mainUI.drawGraph1.setInnerSize(size);
                
                try
                {
                    util.config("graphSize", Float.toString(size));
                }
                catch (Exception e)
                {
                    ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
                    e.printStackTrace();
                }
            }
        });

        rGraphLow.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                mainUI.graphDetail = gdLow;
                if (rGraphLow.isSelected())
                {
                    try
                    {
                        util.config("graphDetail", Integer.toString(gdLow));
                    }
                    catch (Exception e)
                    {
                        ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
                        e.printStackTrace();
                    }
                }
            }
        });

        rGraphMed.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                mainUI.graphDetail = gdMed;
                if (rGraphMed.isSelected())
                {
                    try
                    {
                        util.config("graphDetail", Integer.toString(gdMed));
                    }
                    catch (Exception e)
                    {
                        ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
                        e.printStackTrace();
                    }
                }
            }
        });

        rGraphHigh.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                mainUI.graphDetail = gdHigh;
                if (rGraphHigh.isSelected())
                {
                    try
                    {
                        util.config("graphDetail", Integer.toString(gdHigh));
                    }
                    catch (Exception e)
                    {
                        ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
                        e.printStackTrace();
                    }
                }
            }
        });

        try {
            String mood = util.config("moodDetail");
            if (mood != null) {
                int moodInt = Integer.parseInt(mood);
                if (moodInt == 1)
                        lowAnalysisButton.setSelected(true);
                if (moodInt == 2)
                        mediumAnalysisButton.setSelected(true);
                if (moodInt == 3)
                        highAnalysisButton.setSelected(true);                
            } else {
                lowAnalysisButton.setSelected(true);
            }
        } catch (Exception e) {
            ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
            e.printStackTrace();
        }

        this.lowAnalysisButton.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                if (lowAnalysisButton.isSelected()) {
                    mainUI.moodDetail=1;
                    try {
                        util.config("moodDetail","1");
                    } catch (Exception e) {
                        ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
                        e.printStackTrace();
                    }
                }
            }

        });
        this.mediumAnalysisButton.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                if (mediumAnalysisButton.isSelected()) {
                    mainUI.moodDetail=2;
                    try {
                        util.config("moodDetail","2");
                    } catch (Exception e) {
                        ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
                        e.printStackTrace();
                    }
                }
            }

        });
        this.highAnalysisButton.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                if (highAnalysisButton.isSelected()) {
                    mainUI.moodDetail=3;
                    try {
                        util.config("moodDetail","3");
                    } catch (Exception e) {
                        ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
                        e.printStackTrace();
                    }
                }
            }

        });

        //setting up sliders
        try {
            String moodValue = util.config("moodSlider");
            String BPMValue = util.config("BPMSlider");
            String artistValue = util.config("artistSlider");
            String albumValue = util.config("albumSlider");
            String genreValue = util.config("genreSlider");
            String volumeValue = util.config("volumeSlider");
            if (moodValue == null) {
                util.config("moodSlider","90");
                moodSlider.setValue(90);
            } else {
                moodSlider.setValue(Integer.parseInt(moodValue));
            }

            if (BPMValue == null) {
                util.config("BPMSlider","85");
                BPMSlider.setValue(85);
            } else {
                BPMSlider.setValue(Integer.parseInt(BPMValue));
            }

            if (artistValue == null) {
                util.config("artistSlider","50");
                artistSlider.setValue(50);
            } else {
                artistSlider.setValue(Integer.parseInt(artistValue));
            }

            if (albumValue == null) {
                util.config("albumSlider","50");
                albumSlider.setValue(50);
            } else {
                albumSlider.setValue(Integer.parseInt(albumValue));
            }

            if (genreValue == null) {
                util.config("genreSlider","55");
                genreSlider.setValue(55);
            } else {
                genreSlider.setValue(Integer.parseInt(genreValue));
            }

            if (volumeValue == null) {
                util.config("volumeSlider","75");
                volumeSlider.setValue(75);
            } else {
                volumeSlider.setValue(Integer.parseInt(volumeValue));
            }

        } catch (Exception e) {
            ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
            e.printStackTrace();
        }
        this.moodSlider.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    util.config("moodSlider",Integer.toString(moodSlider.getValue()));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
        this.BPMSlider.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    util.config("BPMSlider",Integer.toString(BPMSlider.getValue()));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
        this.artistSlider.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    util.config("artistSlider",Integer.toString(artistSlider.getValue()));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

        this.albumSlider.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    util.config("albumSlider",Integer.toString(albumSlider.getValue()));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

        this.genreSlider.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    util.config("genreSlider",Integer.toString(genreSlider.getValue()));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
        this.volumeSlider.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    util.config("volumeSlider",Integer.toString(volumeSlider.getValue()));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
        try {
            String decorated = util.config("undecorated");
            if (decorated == null) {
                frameCheckBox.setSelected(false);
                util.config("undecorated","true");
            } else {
                frame = Boolean.parseBoolean(decorated);
                if (frame) {
                    frameCheckBox.setSelected(false);
                } else {
                    frameCheckBox.setSelected(true);
                }
            }
            restartLabel.setVisible(false);
        } catch (Exception e) {
            ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
            e.printStackTrace();
        }


        this.frameCheckBox.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    if (frameCheckBox.isSelected() == frame) {
                        restartLabel.setVisible(true);
                    } else {
                        restartLabel.setVisible(false);
                    }
                    if (frameCheckBox.isSelected()) {
                        util.config("undecorated","false");
                    } else {
                        util.config("undecorated","true");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

         try {
            String volEq = util.config("volEq");
            if (volEq == null) {
                volEqCheckBox.setSelected(false);
                util.config("volEq","false");
            } else {
                boolean Eq = Boolean.parseBoolean(volEq);
                if (Eq) {
                    volEqCheckBox.setSelected(true);
                } else {
                    volEqCheckBox.setSelected(false);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }


        this.volEqCheckBox.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    if (volEqCheckBox.isSelected()) {
                        util.config("volEq","true");
                    } else {
                        util.config("volEq","false");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

        try {
            String importMood = util.config("moodOnImport");
            if (importMood == null) {
                moodImportCheckBox.setSelected(true);
                util.config("moodOnImport","true");
            } else {
                boolean boolImportMood = Boolean.parseBoolean(importMood);
                if (boolImportMood) {
                    moodImportCheckBox.setSelected(true);
                } else {
                    moodImportCheckBox.setSelected(false);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }


        this.moodImportCheckBox.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    if (moodImportCheckBox.isSelected()) {
                        util.config("moodOnImport","true");
                    } else {
                        util.config("moodOnImport","false");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

        try {
            String launchMood = util.config("moodOnLaunch");
            if (launchMood == null) {
                moodLaunchCheckBox.setSelected(true);
                util.config("moodOnLaunch","true");
            } else {
                boolean boolLaunchMood = Boolean.parseBoolean(launchMood);
                if (boolLaunchMood) {
                    moodLaunchCheckBox.setSelected(true);
                } else {
                    moodLaunchCheckBox.setSelected(false);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }


        this.moodLaunchCheckBox.addChangeListener(new ChangeListener()
        {
            @Override
            public void stateChanged(ChangeEvent evt)
            {
                try {
                    if (moodLaunchCheckBox.isSelected()) {
                        util.config("moodOnLaunch","true");
                    } else {
                        util.config("moodOnLaunch","false");
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        bgGraphDetail = new javax.swing.ButtonGroup();
        buttonGroup2 = new javax.swing.ButtonGroup();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        volEqCheckBox = new javax.swing.JCheckBox();
        frameCheckBox = new javax.swing.JCheckBox();
        moodImportCheckBox = new javax.swing.JCheckBox();
        jButton4 = new javax.swing.JButton();
        sliderGraph = new javax.swing.JSlider();
        rGraphLow = new javax.swing.JRadioButton();
        rGraphMed = new javax.swing.JRadioButton();
        rGraphHigh = new javax.swing.JRadioButton();
        jLabel8 = new javax.swing.JLabel();
        lowAnalysisButton = new javax.swing.JRadioButton();
        mediumAnalysisButton = new javax.swing.JRadioButton();
        highAnalysisButton = new javax.swing.JRadioButton();
        sliderReset = new javax.swing.JButton();
        restartLabel = new javax.swing.JLabel();
        moodLaunchCheckBox = new javax.swing.JCheckBox();
        jPanel2 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jColorChooser1 = new javax.swing.JColorChooser();
        jButton2 = new javax.swing.JButton();
        jButton3 = new javax.swing.JButton();
        jPanel5 = new javax.swing.JPanel();
        jToggleButton1 = new javax.swing.JToggleButton();
        jToggleButton2 = new javax.swing.JToggleButton();
        jToggleButton3 = new javax.swing.JToggleButton();
        jToggleButton4 = new javax.swing.JToggleButton();
        canvas1 = new java.awt.Canvas();
        canvas2 = new java.awt.Canvas();
        canvas3 = new java.awt.Canvas();
        canvas4 = new java.awt.Canvas();
        jPanel4 = new javax.swing.JPanel();
        moodSlider = new javax.swing.JSlider();
        BPMSlider = new javax.swing.JSlider();
        artistSlider = new javax.swing.JSlider();
        volumeSlider = new javax.swing.JSlider();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        jLabel9 = new javax.swing.JLabel();
        genreSlider = new javax.swing.JSlider();
        jLabel10 = new javax.swing.JLabel();
        albumSlider = new javax.swing.JSlider();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Preferences");
        setIconImage(util.getWindowIcon());
        setLocationByPlatform(true);
        setResizable(false);
        getContentPane().setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jTabbedPane1.setName("jTabbedPane1"); // NOI18N

        jPanel1.setName("jPanel1"); // NOI18N

        volEqCheckBox.setText("Volume Equalization");
        volEqCheckBox.setName("volEqCheckBox"); // NOI18N

        frameCheckBox.setText("Windows Frame On");
        frameCheckBox.setName("frameCheckBox"); // NOI18N

        moodImportCheckBox.setText("Analyse Mood on Import");
        moodImportCheckBox.setName("moodImportCheckBox"); // NOI18N

        jButton4.setText("Close");
        jButton4.setName("jButton4"); // NOI18N
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton4ActionPerformed(evt);
            }
        });

        sliderGraph.setName("sliderGraph"); // NOI18N

        bgGraphDetail.add(rGraphLow);
        rGraphLow.setText("Low Graph Detail");
        rGraphLow.setName("rGraphLow"); // NOI18N

        bgGraphDetail.add(rGraphMed);
        rGraphMed.setText("Medium Graph Detail");
        rGraphMed.setName("rGraphMed"); // NOI18N

        bgGraphDetail.add(rGraphHigh);
        rGraphHigh.setText("High Graph Detail");
        rGraphHigh.setName("rGraphHigh"); // NOI18N

        jLabel8.setText("Amount of Graph Colour");
        jLabel8.setName("jLabel8"); // NOI18N

        buttonGroup2.add(lowAnalysisButton);
        lowAnalysisButton.setText("Low Mood Analysis");
        lowAnalysisButton.setName("lowAnalysisButton"); // NOI18N

        buttonGroup2.add(mediumAnalysisButton);
        mediumAnalysisButton.setText("Medium Mood Analysis");
        mediumAnalysisButton.setName("mediumAnalysisButton"); // NOI18N

        buttonGroup2.add(highAnalysisButton);
        highAnalysisButton.setText("High Mood Analysis");
        highAnalysisButton.setName("highAnalysisButton"); // NOI18N

        sliderReset.setText("Default");
        sliderReset.setName("sliderReset"); // NOI18N
        sliderReset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sliderResetActionPerformed(evt);
            }
        });

        restartLabel.setText("Armadillo must be restarted for this to take effect");
        restartLabel.setName("restartLabel"); // NOI18N

        moodLaunchCheckBox.setText("Analyse Remaining Tracks on Launch");
        moodLaunchCheckBox.setName("moodLaunchCheckBox"); // NOI18N

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(volEqCheckBox)
                    .addComponent(moodImportCheckBox)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(rGraphMed)
                            .addComponent(rGraphLow)
                            .addComponent(rGraphHigh))
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addGap(74, 74, 74)
                                .addComponent(sliderGraph, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(sliderReset))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addGap(96, 96, 96)
                                .addComponent(jLabel8))))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(frameCheckBox)
                        .addGap(18, 18, 18)
                        .addComponent(restartLabel)))
                .addContainerGap(67, Short.MAX_VALUE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap(524, Short.MAX_VALUE)
                .addComponent(jButton4)
                .addContainerGap())
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(lowAnalysisButton)
                .addContainerGap(422, Short.MAX_VALUE))
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(mediumAnalysisButton)
                .addContainerGap(398, Short.MAX_VALUE))
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(highAnalysisButton)
                .addContainerGap(419, Short.MAX_VALUE))
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(moodLaunchCheckBox)
                .addContainerGap(305, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(41, 41, 41)
                .addComponent(volEqCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(frameCheckBox)
                    .addComponent(restartLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(moodImportCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(moodLaunchCheckBox)
                .addGap(15, 15, 15)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(rGraphLow)
                    .addComponent(jLabel8))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(sliderReset)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(rGraphMed)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(rGraphHigh)
                        .addGap(35, 35, 35)
                        .addComponent(lowAnalysisButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(mediumAnalysisButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(highAnalysisButton)
                        .addGap(40, 40, 40)
                        .addComponent(jButton4))
                    .addComponent(sliderGraph, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(27, 27, 27))
        );

        jTabbedPane1.addTab("General", jPanel1);

        jPanel2.setName("jPanel2"); // NOI18N

        jPanel3.setName("jPanel3"); // NOI18N

        jColorChooser1.setName("jColorChooser1"); // NOI18N

        jButton2.setText("Reset to Default");
        jButton2.setName("jButton2"); // NOI18N
        jButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton2ActionPerformed(evt);
            }
        });

        jButton3.setText("Close");
        jButton3.setName("jButton3"); // NOI18N
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton3ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(jColorChooser1, javax.swing.GroupLayout.PREFERRED_SIZE, 511, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(81, Short.MAX_VALUE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel3Layout.createSequentialGroup()
                .addContainerGap(379, Short.MAX_VALUE)
                .addComponent(jButton2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jButton3)
                .addGap(44, 44, 44))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(jColorChooser1, javax.swing.GroupLayout.PREFERRED_SIZE, 278, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jButton3)
                    .addComponent(jButton2))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel5.setName("jPanel5"); // NOI18N

        jToggleButton1.setSelected(true);
        jToggleButton1.setText("Text");
        jToggleButton1.setName("jToggleButton1"); // NOI18N
        jToggleButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton1ActionPerformed(evt);
            }
        });

        jToggleButton2.setText("Window Background"); // NOI18N
        jToggleButton2.setName("jToggleButton2"); // NOI18N
        jToggleButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton2ActionPerformed(evt);
            }
        });

        jToggleButton3.setText("Borders");
        jToggleButton3.setName("jToggleButton3"); // NOI18N
        jToggleButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton3ActionPerformed(evt);
            }
        });

        jToggleButton4.setText("Panel Background"); // NOI18N
        jToggleButton4.setName("jToggleButton4"); // NOI18N
        jToggleButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jToggleButton4ActionPerformed(evt);
            }
        });

        canvas1.setName("canvas1"); // NOI18N

        canvas2.setName("canvas2"); // NOI18N

        canvas3.setName("canvas3"); // NOI18N

        canvas4.setName("canvas4"); // NOI18N

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel5Layout.createSequentialGroup()
                        .addGap(12, 12, 12)
                        .addComponent(canvas1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel5Layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jToggleButton1, javax.swing.GroupLayout.DEFAULT_SIZE, 86, Short.MAX_VALUE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(canvas2, 0, 0, Short.MAX_VALUE)
                    .addComponent(jToggleButton2, javax.swing.GroupLayout.DEFAULT_SIZE, 148, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(canvas3, 0, 0, Short.MAX_VALUE)
                    .addComponent(jToggleButton3, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(canvas4, 0, 0, Short.MAX_VALUE)
                    .addComponent(jToggleButton4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap(28, Short.MAX_VALUE))
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jToggleButton1)
                    .addComponent(jToggleButton2)
                    .addComponent(jToggleButton3)
                    .addComponent(jToggleButton4))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(canvas4, javax.swing.GroupLayout.DEFAULT_SIZE, 18, Short.MAX_VALUE)
                    .addComponent(canvas3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 18, Short.MAX_VALUE)
                    .addComponent(canvas2, javax.swing.GroupLayout.DEFAULT_SIZE, 18, Short.MAX_VALUE)
                    .addComponent(canvas1, javax.swing.GroupLayout.DEFAULT_SIZE, 18, Short.MAX_VALUE))
                .addContainerGap())
        );

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        jTabbedPane1.addTab("Custom Skin Colours", jPanel2);

        jPanel4.setName("jPanel4"); // NOI18N
        jPanel4.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        moodSlider.setName("moodSlider"); // NOI18N
        jPanel4.add(moodSlider, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 60, 540, -1));

        BPMSlider.setName("BPMSlider"); // NOI18N
        jPanel4.add(BPMSlider, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 110, 540, -1));

        artistSlider.setName("artistSlider"); // NOI18N
        jPanel4.add(artistSlider, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 160, 540, -1));

        volumeSlider.setName("volumeSlider"); // NOI18N
        jPanel4.add(volumeSlider, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 310, 540, -1));

        jLabel1.setText("Low");
        jLabel1.setName("jLabel1"); // NOI18N
        jPanel4.add(jLabel1, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 50, -1, -1));

        jLabel2.setText("High");
        jLabel2.setName("jLabel2"); // NOI18N
        jPanel4.add(jLabel2, new org.netbeans.lib.awtextra.AbsoluteConstraints(520, 50, -1, -1));

        jLabel3.setText("Songs matched on ");
        jLabel3.setName("jLabel3"); // NOI18N
        jPanel4.add(jLabel3, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 20, -1, -1));

        jLabel4.setText("Mood");
        jLabel4.setName("jLabel4"); // NOI18N
        jPanel4.add(jLabel4, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 50, -1, -1));

        jLabel5.setText("Beat (BPM)");
        jLabel5.setName("jLabel5"); // NOI18N
        jPanel4.add(jLabel5, new org.netbeans.lib.awtextra.AbsoluteConstraints(250, 100, -1, -1));

        jLabel6.setText("Artist");
        jLabel6.setName("jLabel6"); // NOI18N
        jPanel4.add(jLabel6, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 150, -1, -1));

        jLabel7.setText("Volume");
        jLabel7.setName("jLabel7"); // NOI18N
        jPanel4.add(jLabel7, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 300, -1, -1));

        jButton1.setText("Reset to Defaults");
        jButton1.setName("jButton1"); // NOI18N
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        jPanel4.add(jButton1, new org.netbeans.lib.awtextra.AbsoluteConstraints(440, 360, -1, -1));

        jButton5.setText("Close");
        jButton5.setName("jButton5"); // NOI18N
        jButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton5ActionPerformed(evt);
            }
        });
        jPanel4.add(jButton5, new org.netbeans.lib.awtextra.AbsoluteConstraints(510, 400, -1, -1));

        jLabel9.setText("Genre");
        jLabel9.setName("jLabel9"); // NOI18N
        jPanel4.add(jLabel9, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 250, -1, -1));

        genreSlider.setName("genreSlider"); // NOI18N
        jPanel4.add(genreSlider, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 260, 540, -1));

        jLabel10.setText("Album");
        jLabel10.setName("jLabel10"); // NOI18N
        jPanel4.add(jLabel10, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 200, -1, -1));

        albumSlider.setName("albumSlider"); // NOI18N
        jPanel4.add(albumSlider, new org.netbeans.lib.awtextra.AbsoluteConstraints(20, 210, 540, -1));

        jTabbedPane1.addTab("Mood Model", jPanel4);

        getContentPane().add(jTabbedPane1, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 12, 595, -1));

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jToggleButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton4ActionPerformed
        if (jToggleButton4.isSelected()) {
            jToggleButton1.setSelected(false);
            jToggleButton2.setSelected(false);
            jToggleButton3.setSelected(false);
        } else {
            jToggleButton4.setSelected(true);
        }
}//GEN-LAST:event_jToggleButton4ActionPerformed

    private void jToggleButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton3ActionPerformed
        if (jToggleButton3.isSelected()) {
            jToggleButton1.setSelected(false);
            jToggleButton2.setSelected(false);
            jToggleButton4.setSelected(false);
        } else {
            jToggleButton3.setSelected(true);
        }
}//GEN-LAST:event_jToggleButton3ActionPerformed

    private void jToggleButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton2ActionPerformed
        if (jToggleButton2.isSelected()) {
            jToggleButton1.setSelected(false);
            jToggleButton3.setSelected(false);
            jToggleButton4.setSelected(false);
        } else {
            jToggleButton2.setSelected(true);
        }
}//GEN-LAST:event_jToggleButton2ActionPerformed

    private void jToggleButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jToggleButton1ActionPerformed
        if (jToggleButton1.isSelected()) {

            jToggleButton2.setSelected(false);
            jToggleButton3.setSelected(false);
            jToggleButton4.setSelected(false);
        } else {
            jToggleButton1.setSelected(true);
        }
}//GEN-LAST:event_jToggleButton1ActionPerformed

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
        java.awt.Color bgcolor = new java.awt.Color(0, 0, 0);        
	java.awt.Color scrollcolor = new java.awt.Color(32, 32, 32);
	java.awt.Color scrollcolor2 = new java.awt.Color(64, 64, 64);
        pUI.repaint();
        if (jToggleButton1.isSelected()) {
            canvas1.setBackground(Color.BLUE);
            try {
                util.config("customFontColour",Integer.toString(Color.BLUE.getRGB()));
            } catch (Exception ex) {
                Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else if (jToggleButton2.isSelected()) {
            canvas2.setBackground(bgcolor);
            try {
                util.config("customBackgroundColour",Integer.toString(bgcolor.getRGB()));
            } catch (Exception ex) {
                Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else if (jToggleButton3.isSelected()) {
            canvas3.setBackground(scrollcolor2);
            try {
                util.config("customBorderColour",Integer.toString(scrollcolor2.getRGB()));
            } catch (Exception ex) {
                Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        } else if (jToggleButton4.isSelected()) {
            canvas4.setBackground(scrollcolor);
            try {
                util.config("customPanelBackgroundColour", Integer.toString(scrollcolor.getRGB()));
            } catch (Exception ex) {
                Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        try {
            if (util.config("skin").equals("Custom")) {
                mainUI.skinsMenuItemActionPerformed(null, "Custom");
            }
        } catch (Exception ex) {
                Logger.getLogger(ArmadilloUI.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_jButton2ActionPerformed

    private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
        this.dispose();
    }//GEN-LAST:event_jButton5ActionPerformed

    private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
        this.dispose();
    }//GEN-LAST:event_jButton3ActionPerformed

    private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
        this.dispose();
    }//GEN-LAST:event_jButton4ActionPerformed

    private void sliderResetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sliderResetActionPerformed
        sliderGraph.setValue(25);
    }//GEN-LAST:event_sliderResetActionPerformed

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        try {
                util.config("moodSlider","90");
                moodSlider.setValue(90);
                util.config("BPMSlider","85");
                BPMSlider.setValue(85);
                util.config("artistSlider","50");
                artistSlider.setValue(50);
                util.config("albumSlider","50");
                albumSlider.setValue(50);
                util.config("genreSlider","55");
                genreSlider.setValue(55);
                util.config("volumeSlider","75");
                volumeSlider.setValue(75);
        } catch (Exception e) {
            ArmadilloLog.LogError("Preferences UI cannot access config file - "+e.toString());
            e.printStackTrace();
        }
    }//GEN-LAST:event_jButton1ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JSlider BPMSlider;
    private javax.swing.JSlider albumSlider;
    private javax.swing.JSlider artistSlider;
    private javax.swing.ButtonGroup bgGraphDetail;
    private javax.swing.ButtonGroup buttonGroup2;
    private java.awt.Canvas canvas1;
    private java.awt.Canvas canvas2;
    private java.awt.Canvas canvas3;
    private java.awt.Canvas canvas4;
    private javax.swing.JCheckBox frameCheckBox;
    private javax.swing.JSlider genreSlider;
    private javax.swing.JRadioButton highAnalysisButton;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JColorChooser jColorChooser1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JToggleButton jToggleButton1;
    private javax.swing.JToggleButton jToggleButton2;
    private javax.swing.JToggleButton jToggleButton3;
    private javax.swing.JToggleButton jToggleButton4;
    private javax.swing.JRadioButton lowAnalysisButton;
    private javax.swing.JRadioButton mediumAnalysisButton;
    private javax.swing.JCheckBox moodImportCheckBox;
    private javax.swing.JCheckBox moodLaunchCheckBox;
    private javax.swing.JSlider moodSlider;
    private javax.swing.JRadioButton rGraphHigh;
    private javax.swing.JRadioButton rGraphLow;
    private javax.swing.JRadioButton rGraphMed;
    private javax.swing.JLabel restartLabel;
    private javax.swing.JSlider sliderGraph;
    private javax.swing.JButton sliderReset;
    private javax.swing.JCheckBox volEqCheckBox;
    private javax.swing.JSlider volumeSlider;
    // End of variables declaration//GEN-END:variables

}
