/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

package armadilloUI;

import java.awt.Color;
import java.util.concurrent.atomic.AtomicBoolean;
import javax.swing.GroupLayout;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;
import java.awt.Image;

/**
 * UI and functionality for the Splash screen
 * @author Mark Tolley
 */
public class SplashScreen
{
    /**
     * the splash screen jframe
     */
    protected JFrame jf;

    /**
     * the atomic boolean used by the splashscreen
     */
    protected final AtomicBoolean go = new AtomicBoolean();
    /**
     * the text display ontop the splash screen image
     */
    protected final JLabel txt;

    /**
     * constructor for the splash screen, sets up the UI
     */
    public SplashScreen()
    {
        jf = new JFrame("Armadillo Music Player");
        jf.setResizable(false);
        jf.setUndecorated(true);

        JPanel panel = new JPanel();
        panel.setBorder(LineBorder.createBlackLineBorder());
        panel.setBackground(new Color(250, 251, 246));
        jf.add(panel);
        ImageIcon icon = new javax.swing.ImageIcon(getClass().getResource("/armadilloUI/graphics/babyIcon.gif"));
        Image iconImage = icon.getImage();
        jf.setIconImage(iconImage);
        JLabel img = new JLabel();
        img.setIcon(new ImageIcon(getClass().getResource("graphics/About_screen.jpg")));

        txt = new JLabel("  Please wait, loading");

        GroupLayout layout = new GroupLayout(panel);
        panel.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup()
            .addComponent(img)
            .addComponent(txt)
        );
        layout.setVerticalGroup(
            layout.createSequentialGroup()
            .addComponent(img)
            .addComponent(txt)
        );

        jf.pack();
        jf.setLocationRelativeTo(null);
    }

    /**
     * Display the splash screen, and alters the loading text
     */
    public void show()
    {
        jf.setVisible(true);
        go.set(true);

        Thread t = new Thread(new Runnable()
        {
           @Override
           public void run()
           {
               while (go.get())
               {
                   try
                   {
                       Thread.sleep(500);
                   }
                   catch (InterruptedException e)
                   {
                       continue;
                   }

                   txt.setText(txt.getText() + ".");
               }
           }
        });

        t.start();
    }

    /**
     * hides the splash screen
     */
    public void hide()
    {
        jf.setVisible(false);
        go.set(false);
    }
}
