/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * tapBPMUI.java
 *
 * Created on 16-Apr-2010, 14:49:05
 */

package armadilloUI;

/**
 *
 * @author jonathan
 */

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import java.io.File;
import org.gstreamer.State;


/**
 * this class provides the UI and functionality for the user to tap along
 * to the beat of a track and use the result as the value for the bpm field
 * @author jonathan
 */
public class TapBPMUI extends javax.swing.JFrame {

    private ArmadilloDB db;
    private ArmadilloUI UI;
    private Timer tapBPMTimer;
    private MusicPlayer tapBPMPlayer;
    private int trackID;
    private long beatTime;
    private long lastBeatTime;
    private int beatCount;
    private long startBeatTime;
    private long bpm;
    private boolean from_edit_track_window;
    private EditTrackUI editTrackUI;
    private boolean isAnalysing;
    /** Creates new form tapBPMUI
     * @param track_id
     * @param from_edit_track
     * @param passedUI
     */
    public TapBPMUI(int track_id, boolean from_edit_track, ArmadilloUI passedUI) {
        ArmadilloLog.LogOperation("Constructing TapBPM UI");
        this.trackID = track_id;
        this.from_edit_track_window = from_edit_track;
        if (track_id == -1) {            
            //Error no track selected
            new DialogUI(8).setVisible(true);
            this.dispose();
        } else {
            try {
                db = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
            } catch (Exception ex) {
                ArmadilloLog.LogError("TapBPM UI cannot access DB - " + ex.toString());
                Logger.getLogger(EditTrackUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            UI = passedUI;
            initComponents();
            this.setLocationRelativeTo(null);
            String[] trackDetails = db.getTrack(track_id);
            trackLabel.setText(trackDetails[0] + "  -  " + trackDetails[1]);
            tapBPMPlayer = new MusicPlayer();
            if (UI.playTimer.isRunning()) {
                UI.Pause();
            }
            tapBPMPlayer.playbin.setState(State.READY);
            File trackFile = new File(trackDetails[5]);
            tapBPMPlayer.Play(trackFile);
            tapBPMPlayer.Pause();
            tapBPMTimer = new Timer(1000/2, new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    long time = tapBPMPlayer.GetTimeElapsed();
                    long dur = tapBPMPlayer.GetDuration();
                    trackSlider.setValue(Math.round((time*500)/dur));
                }
            });
            //Add mouselistener to seekbar
            MouseListener seekBarListener = new MouseAdapter() {
                @Override
                public void mouseClicked(MouseEvent click) {
                    long mousePosition = click.getX();
                    long width = trackSlider.getWidth();
                    long percentage = (mousePosition*100)/width;
                    long dur = tapBPMPlayer.GetDuration();
                    long seek = (dur/100)*percentage;                    
                    tapBPMPlayer.Seek(seek);
                    trackSlider.setValue(Math.round(5*percentage));
                }

                @Override
                public void mouseReleased(MouseEvent click) {
                    long mousePosition = click.getX();
                    long width = trackSlider.getWidth();
                    long percentage = (mousePosition*100)/width;
                    long dur = tapBPMPlayer.GetDuration();
                    long seek = (dur/100)*percentage;                    
                    tapBPMPlayer.Seek(seek);
                    trackSlider.setValue(Math.round(5*percentage));
                }
            };
            trackSlider.addMouseListener(seekBarListener);
            WindowListener windowListener = new WindowAdapter() {
                @Override
                public void windowClosing(WindowEvent e) {                    
                    tapBPMTimer.stop();
                    tapBPMPlayer.Stop();
                    tapBPMPlayer.playbin.setState(State.NULL);
                    tapBPMPlayer.playbin.dispose();
                    e.getWindow().dispose();
                }
            };
            this.addWindowListener(windowListener);
        }
    }

    /**
     * Sets the edit track ui object which spawn this object
     * @param passedEditTrackUI
     */
    public void setEditTrackUI(EditTrackUI passedEditTrackUI) {
        this.editTrackUI = passedEditTrackUI;
    }

    /**
     * closes the UI
     */
    public void close() {

        tapBPMTimer.stop();
        tapBPMPlayer.Stop();
        tapBPMPlayer.playbin.setState(State.NULL);
        tapBPMPlayer.playbin.dispose();
        db.closeConnection();
        if (!from_edit_track_window) {
            UI.reloadTable(trackID);
            UI.reloadNavTreeBPM();
        }
        this.dispose();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        playPauseButton = new javax.swing.JButton();
        trackSlider = new javax.swing.JSlider();
        tapButton = new javax.swing.JButton();
        bpmTextField = new javax.swing.JTextField();
        instructionsLabel1 = new javax.swing.JLabel();
        instructionsLabel2 = new javax.swing.JLabel();
        acceptButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        trackLabel = new javax.swing.JLabel();
        bpmLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Manual BPM Estimation");
        setResizable(false);

        playPauseButton.setText(">");
        playPauseButton.setName("playPauseButton"); // NOI18N
        playPauseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                playPauseButtonActionPerformed(evt);
            }
        });

        trackSlider.setMaximum(500);
        trackSlider.setValue(0);
        trackSlider.setName("trackSlider"); // NOI18N

        tapButton.setText("Tap Beat");
        tapButton.setName("tapButton"); // NOI18N
        tapButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                tapButtonActionPerformed(evt);
            }
        });

        bpmTextField.setEditable(false);
        bpmTextField.setText("0");
        bpmTextField.setName("bpmTextField"); // NOI18N

        instructionsLabel1.setText("Tap the button in time with the beat to estimate the BPM");
        instructionsLabel1.setName("instructionsLabel1"); // NOI18N

        instructionsLabel2.setText("Once happy with the estimation press Accept or Cancel");
        instructionsLabel2.setName("instructionsLabel2"); // NOI18N

        acceptButton.setText("Accept");
        acceptButton.setName("acceptButton"); // NOI18N
        acceptButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                acceptButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        trackLabel.setText("Track name - Artist Name");
        trackLabel.setName("trackLabel"); // NOI18N

        bpmLabel.setText("Estimated BPM:");
        bpmLabel.setName("bpmLabel"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(instructionsLabel1)
                    .addComponent(instructionsLabel2)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(acceptButton)
                        .addGap(11, 11, 11)
                        .addComponent(cancelButton))
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(playPauseButton, javax.swing.GroupLayout.PREFERRED_SIZE, 51, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGap(18, 18, 18)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(trackSlider, javax.swing.GroupLayout.DEFAULT_SIZE, 297, Short.MAX_VALUE)
                                    .addComponent(trackLabel)))
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                .addComponent(tapButton, javax.swing.GroupLayout.PREFERRED_SIZE, 105, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(17, 17, 17)
                                .addComponent(bpmLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(bpmTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 59, javax.swing.GroupLayout.PREFERRED_SIZE)))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(instructionsLabel1)
                .addGap(12, 12, 12)
                .addComponent(trackLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(trackSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(tapButton)
                            .addComponent(bpmTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(bpmLabel)))
                    .addComponent(playPauseButton, javax.swing.GroupLayout.PREFERRED_SIZE, 51, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(instructionsLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(acceptButton)
                    .addComponent(cancelButton))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void playPauseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_playPauseButtonActionPerformed
        if (tapBPMTimer.isRunning()) {
            playPauseButton.setText(">");
            tapBPMPlayer.Pause();
            tapBPMTimer.stop();
        } else {
            playPauseButton.setText("||");
            tapBPMPlayer.Pause();
            tapBPMTimer.start();
        }
    }//GEN-LAST:event_playPauseButtonActionPerformed

    private void tapButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_tapButtonActionPerformed
        beatTime = System.currentTimeMillis();
        //if the first beat for 10 seconds (i.e. 6BPM or less) take it as a new try
        long gap = (beatTime-lastBeatTime);        
        if (gap>2000) {
            startBeatTime = beatTime;
            lastBeatTime = beatTime;            
            beatCount = 0;
            bpm = 0;
        } else {
            //60bpm = 1BPS = gap of 1000 milliseconds
            beatCount++;                        
            lastBeatTime = beatTime;
            gap = beatTime - startBeatTime;
            long BPM = (60000*beatCount)/gap;
            bpmTextField.setText(Integer.toString(Math.round(BPM)));
        }
        /*
        if (flag == 0)
        {
            StartTime = System.currentTimeMillis();
            BPMString = ("First beat");
            flag = 1;
        } else {
            BeatTime = System.currentTimeMillis();
            BPMcompare = (int)BPM;
            BPM = ((double)60000 / ((double)BeatTime - (double)StartTime)) * (double)count;
            BPMString = ("" + BPM);
        }
        count++;
        if (BPMcompare == (int)BPM) BPMconverge++;
        else BPMconverge = 0;
        if (BPMconverge >= 3) updateBPM.setEnabled(true);
        else updateBPM.setEnabled(false);
        addBPMToBox();
        */
    }//GEN-LAST:event_tapButtonActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        this.close();
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void acceptButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_acceptButtonActionPerformed
        if (from_edit_track_window) {
            editTrackUI.setBPMField(Integer.parseInt(bpmTextField.getText()));
            this.close();
            //code to insert generated number into edit track window
        } else {
            this.pauseOtherThreads();
            db.editTrackTag(trackID, 5, bpmTextField.getText());
            if (isAnalysing)
                UI.resumeAnalysing();
            isAnalysing = false;
            close();
        }
    }//GEN-LAST:event_acceptButtonActionPerformed

    private void pauseOtherThreads()
    {
        // first pause the analysing thread (if its active)
        this.isAnalysing = false;
        if (UI.getbpmThread() != null)
        {
            if (UI.getbpmThread().isAlive())
                this.isAnalysing = UI.getbpmThread().pauseAnalysing(false);
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton acceptButton;
    private javax.swing.JLabel bpmLabel;
    private javax.swing.JTextField bpmTextField;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel instructionsLabel1;
    private javax.swing.JLabel instructionsLabel2;
    private javax.swing.JButton playPauseButton;
    private javax.swing.JButton tapButton;
    private javax.swing.JLabel trackLabel;
    private javax.swing.JSlider trackSlider;
    // End of variables declaration//GEN-END:variables

}
