/*   This file is part of Armadillo Music Player.
*
*   Armadillo is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Armadillo is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Armadillo.  If not, see <http://www.gnu.org/licenses/>.
*/

package armadilloUI;

/**
 * Track object data structure
 * @author jonathan
 */

public class TrackObject {

    private ArmadilloDB dbTrackObject;
    private int trackID;
    private String[] trackDetails = null;
    private String trackName;
    private String artist;
    private String album;
    private String genre;
    private String location;
    private int BPM;
    private long length;
    private double Dec;


    /**
     * This class provides a standard data structure for handling a music file
     * within the program, along with all the data associated with it
     * @param TrackID
     */
    public TrackObject(int TrackID) {
        try {
            trackID=TrackID;
            getDataFromDB(2);
            if (this.trackDetails == null) {
                this.trackID = -1;
            } else {
                this.trackName = trackDetails[0];
                this.artist = trackDetails[1];
                this.album = trackDetails[2];
                this.genre = trackDetails[3];
                this.length = Long.parseLong(trackDetails[4]);
                this.location = trackDetails[5];
            }
        } catch (Exception ex) {
            ArmadilloLog.LogError("TrackObject - cannot load data -" + ex.toString() + " ");
            new DialogUI(16).setVisible(true);
        }
    }

    /**
     * sets the decibels of the trackobject
     * @param passedDec
     */
    public void setDec(double passedDec) {
        this.Dec=passedDec;
    }

    /**
     * returns the decibels of the trackobject
     * @return
     */
    public double getDec() {
        return this.Dec;
    }

    /**
     * returns an array of data ready to put in table
     * @return
     */
    public String[] getTableData() {
        String[] trackData= new String[6];
        trackData[0] = this.trackName;
        trackData[1] = this.artist;
        trackData[2] = this.album;
        trackData[3] = this.genre;
        if (BPM == -1) {
            trackData[4] = "N/A";
        } else {
            trackData[4] = Integer.toString(this.BPM);
        }
        trackData[5] = UIUtilities.FormatTime(length);
        return trackData;
    }

    /**
     * returns a string to be displayed in the table for this track
     * @return
     */
    public String getListData() {
        String listData = this.trackName + " - " + this.artist;
        return listData;
    }

    /**
     * returns a string ready to be displayed in the track info on the main UI
     * @return
     */
    public String getPlayingData() {
        return "    " + this.trackName + " - " + this.album + " - " + this.artist + "    ";
    }
    
    /**
     * returns true if reloaded and still exists in DB, false if it doesnt
     * and also updates all the field from the Db if it exists
     * @return
     */
    public boolean refresh() {
        try {
            getDataFromDB(2);
            if (trackDetails[0] == null) {
                return false;
            }
            this.trackName = trackDetails[0];
            this.artist = trackDetails[1];
            this.album = trackDetails[2];
            this.genre = trackDetails[3];
            this.length = Long.parseLong(trackDetails[4]);
            this.location = trackDetails[5];
        } catch (Exception ex) {
            ArmadilloLog.LogError("TrackObject - cannot reload data -" + ex.toString() + " ");
            new DialogUI(16).setVisible(true);
        }       
        return true;
    }

    /**
     * returns the track id
     * @return
     */
    public int getTrackID() {
        return this.trackID;
    }

    /**
     * returns the length of the track
     * @return
     */
    public long getLength() {
        return this.length;
    }

    /**
     * returns the file name
     * @return
     */
    public String getFileName() {
        return this.location;
    }

    private void getDataFromDB(int constructorNo)
    {
        try {
            dbTrackObject = new ArmadilloDB("org.sqlite.JDBC", "jdbc:sqlite:armadillo.db");
            if (constructorNo == 1) {
                this.Dec = dbTrackObject.getAverageDec(trackID);
            }
            else if (constructorNo == 2) {
                this.trackDetails =dbTrackObject.getTrack(trackID);
                this.BPM = dbTrackObject.getBPM(trackID);
                this.Dec = dbTrackObject.getAverageDec(trackID);
            }
            dbTrackObject.closeConnection();
        } catch (Exception ex) {
            ArmadilloLog.LogError("TrackObject cannot access DB - " + ex.toString());

            ArmadilloLog.LogError("TrackObject cannot access DB");
        }
    }
}
