
package armadilloUI.runcheck;

import java.rmi.AlreadyBoundException;
import java.rmi.ConnectException;
import java.rmi.NotBoundException;
import java.rmi.registry.Registry;
import java.rmi.registry.LocateRegistry;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * performs a check to see if another armadillo is running
 * @author Mark Tolley
 */
public class RunCheck
{
    /**
     * sets up the code base
     */
    protected static void setupCodebase()
    {
        if (System.getProperty("java.rmi.server.codebase") == null)
        {
            //File codebase = new File(System.getProperty("user.dir"), "build/classes");
            //System.setProperty("java.rmi.server.codebase", codebase.toURI().toString());
            
            String codebase = RunCheck.class.getProtectionDomain().getCodeSource().getLocation().toString();
            System.setProperty("java.rmi.server.codebase", codebase);

            //System.out.println(codebase);
        }
    }

    /**
     * starts a registry
     */
    protected static void startRegistry()
    {
        try
        {
            LocateRegistry.createRegistry(Registry.REGISTRY_PORT);
        }
        catch (RemoteException re)
        {
            re.printStackTrace();
        }
    }

    /**
     * sets the primary instance
     * @return
     */
    public static boolean setPrimaryInstance()
    {
        setupCodebase();
        
        try
        {
            Server srv = new Server();
            Checker stub = (Checker)UnicastRemoteObject.exportObject(srv, 0);

            Registry reg = LocateRegistry.getRegistry();
            try
            {
                reg.bind("RunCheck", stub);
            }
            catch (AlreadyBoundException e)
            {
                reg.rebind("RunCheck", stub);
            }
            catch (ConnectException e)
            {
                startRegistry();
                return false;
            }

            System.out.println("This is the primary Armadillo instance.");
        }
        catch (RemoteException re)
        {
            re.printStackTrace();
        }

        return true;
    }

    /**
     * checks if it is running
     * @return
     */
    public static boolean isRunning()
    {
        int result = 0;

        setupCodebase();

        try
        {
            Registry reg = LocateRegistry.getRegistry();

            Checker stub;
            try
            {
                stub = (Checker)reg.lookup("RunCheck");
            }
            catch (NotBoundException e)
            {
                return false;
            }
            catch (ConnectException e)
            {
                // registry is not running yet
                return false;
            }

            try
            {
                result = stub.isRunning();
            }
            catch (ConnectException e)
            {
                return false;
            }
        }
        catch (RemoteException re)
        {
            re.printStackTrace();
        }
        
        if (result == 1)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * shutdowns
     */
    public static void shutDown()
    {
        try
        {
            Registry reg = LocateRegistry.getRegistry();
            reg.unbind("RunCheck");
        }
        catch (Exception e)
        {
        }
    }
}
