/*
 * FinishScreen.java
 *
 * Created on 20-Apr-2010, 18:45:22
 */

package armadillosetup;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 *
 * @author Mark Tolley
 */
class FinishScreen extends javax.swing.JPanel implements WizardScreen
{

    protected WizardFrame wizard;

    protected boolean windows = false;

    /** Creates new form FinishScreen */
    public FinishScreen()
    {
        String os = System.getProperty("os.name");
        if (os.startsWith("Windows"))
        {
            windows = true;
        }

        initComponents();
    }

    @Override
    public void display(WizardFrame wizard)
    {
        this.wizard = wizard;

        wizard.setContent(this);
        wizard.setFinished();
    }

    @Override
    public void hide()
    {
        if (cbDesktop.isSelected())
        {
            createDesktopShortcut();
        }

        if (cbLaunch.isSelected())
        {
            launchArmadillo();
        }
    }

    protected void createDesktopShortcut()
    {
        System.out.println("Creating desktop shortcut...");

        String path = System.getProperty("user.dir");

        if (windows)
        {
            try
            {
                Process p = new ProcessBuilder("setup/shortcut.exe").start();
            }
            catch (IOException e)
            {
                System.err.println("Error: createDesktopShortcut(): Could not create shortcut.");
                e.printStackTrace();
            }

            return;
        }

        File home = new File(System.getProperty("user.home"));
        File desktop = new File(home, "Desktop");
        File shortcut = new File(desktop, "armadillo.desktop");
        
        File app = new File(path, "armadillo-launch.sh");
        File icon = new File(path, "setup/armadillo-icon.png");

        if (desktop.exists())
        {
            if (shortcut.exists())
            {
                if (!shortcut.delete())
                {
                    System.err.println("Error: createDesktopShortcut(): Shortcut already exists and cannot be removed.");
                    return;
                }
            }

            FileWriter w;
            try
            {
                w = new FileWriter(shortcut);
            }
            catch (IOException e)
            {
                System.err.println("Error: createDesktopShortcut(): Cannot open file for writing.");
                e.printStackTrace();
                return;
            }
            
            StringBuilder sb = new StringBuilder();
            sb.append("[Desktop Entry]\n")
                    .append("Type=Application\n")
                    .append("Version=1.0\n")
                    .append("Name=Armadillo Music Player\n")
                    .append("GenericName=Media Player\n")
                    .append("Comment=Play & categorise your music\n")
                    .append("Categories=AudioVideo;Audio;Player;Music;Java\n")
                    .append("Icon=").append(icon.toString()).append("\n")//.append("armadillo-icon.png")
                    .append("Exec=").append(app.toString()).append("\n")
                    .append("Path=").append(path).append("\n");

            try
            {
                w.write(sb.toString());

                w.close();
            }
            catch (IOException e)
            {
                System.err.println("Error: createDesktopShortcut(): Could not create shortcut.");
                e.printStackTrace();
                return;
            }

            if (!shortcut.setExecutable(true))
            {
                System.err.println("Warning: createDesktopShortcut(): Could not make shortcut executable.");
            }
        }
        else
        {
            System.err.println("Error: createDesktopShortcut(): Cannot find desktop.");
        }

        /*
        System.out.println("Installing icon...");

        File iconDir = new File(home, ".icon");
        if (!iconDir.exists())
        {
            if (!iconDir.mkdir())
            {
                System.err.println("Error: createDesktopShortcut(): Cannot create .icon directory.");
                return;
            }
        }

        File iconSrc = new File(path, "setup/armadillo-icon.png");
        File iconDest = new File(iconDir, "armadillo-icon.png");

        if (!Util.copyFile(iconSrc, iconDest))
        {
            System.err.println("Error: createDesktopShortcut(): Could not copy icon.");
        }
        */
    }

    protected void launchArmadillo()
    {
        System.out.println("Launching Armadillo...");

        try
        {
            if (windows)
            {
                Runtime.getRuntime().exec("armadillo-launch.exe");
            }
            else
            {
                Runtime.getRuntime().exec("./armadillo-launch.sh");
            }
        }
        catch (IOException e)
        {
            System.err.println("Error: launchArmadillo(): Cannot exec process.");
            e.printStackTrace();
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        cbDesktop = new javax.swing.JCheckBox();
        cbLaunch = new javax.swing.JCheckBox();

        jLabel1.setFont(new java.awt.Font("Tahoma", 1, 12));
        jLabel1.setText("Setup complete");

        jLabel2.setText("Setup has finished configuring Armadillo Music Player.");

        jLabel3.setText("You can close this wizard now by clicking Finish.");

        cbDesktop.setSelected(true);
        cbDesktop.setText("Add a shortcut for Armadillo to my Desktop");

        cbLaunch.setSelected(true);
        cbLaunch.setText("Launch Armadillo");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(cbLaunch)
                    .addComponent(cbDesktop)
                    .addComponent(jLabel1)
                    .addComponent(jLabel2)
                    .addComponent(jLabel3))
                .addContainerGap(278, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addGap(18, 18, 18)
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel3)
                .addGap(18, 18, 18)
                .addComponent(cbDesktop)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(cbLaunch)
                .addContainerGap(190, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox cbDesktop;
    private javax.swing.JCheckBox cbLaunch;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    // End of variables declaration//GEN-END:variables

}
