/*
 * GStreamerScreen.java
 *
 * Created on 20-Apr-2010, 18:55:14
 */

package armadillosetup;

import java.awt.event.ItemEvent;
import org.gstreamer.ElementFactory;
import org.gstreamer.Gst;
import org.gstreamer.Version;

/**
 *
 * @author Mark Tolley
 */
class GStreamerScreen extends javax.swing.JPanel implements WizardScreen
{
    // graph defaults
    protected static final int gdHigh = 1920;
    protected static final int gdMed = 1024;
    protected static final int gdLow = 640;

    protected WizardFrame wizard;

    protected boolean done = false;
    protected boolean perfUpdate = true;

    class Tester extends Thread
    {
        String helpText = "";

        public Tester()
        {
        }

        protected boolean gstCheck()
        {
            try
            {
                Gst.init();
            }
            catch(UnsatisfiedLinkError e)
            {
                e.printStackTrace();
                return false;
            }

            gstFound.setText("Found");

            return true;
        }

        protected boolean gstVerCheck()
        {
            gstVersion.setText(Gst.getVersionString());

            Version v = Gst.getVersion();

            if ((v.getMajor() == 0 && v.getMinor() == 10 && v.getMicro() >= 10)
                    || (v.getMajor() == 0 && v.getMinor() >= 10)
                    || (v.getMajor() > 0))
            {
                return true;
            }

            return false;
        }

        protected boolean gstPluginCheck()
        {
            try
            {
                ElementFactory.make("appsink", "testsink");
            }
            catch (IllegalArgumentException e)
            {
                e.printStackTrace();

                gstPlugins.setText("Missing AppSink");
                helpText += "<p>Your installation of GStreamer is missing the AppSink plugin.</p>"
                        + "<p>Please either install the gst-plugins-bad package or download the Armadillo GStreamer bundle from our website.</p>";

                return false;
            }

            gstPlugins.setText("OK");
            return true;
        }

        @Override
        public void run()
        {
            bar.setValue(0);

            status.setText("Checking for GStreamer...");
            if (!gstCheck())
            {
                gstFound.setText("Not found");
                helpText += "<p>The GStreamer Multimedia Framework is required to use Armadillo.</p>"
                        + "<p>Please either install it via your Operating System's package management system or download the bundled installer from our website.</p>";
                done();
                return;
            }

            bar.setValue(30);

            status.setText("Checking GStreamer version...");
            if (!gstVerCheck())
            {
                unsupported.setVisible(true);
                helpText += "<p>The version of the GStreamer Multimedia Framework installed on your computer is out of date.</p>"
                        + "<p>You must update it if you wish to use Armadillo.</p>";

                done();
                return;
            }

            bar.setValue(60);

            status.setText("Checking GStreamer plugins...");
            if (!gstPluginCheck())
            {
                done();
                return;
            }

            bar.setValue(100);

            done();

            done = true;
            wizard.setButtons(true, true);
        }

        public void done()
        {
            bar.setValue(100);
            
            status.setText("Finished.");
            if (!helpText.equals(""))
                help.setText("<html>" + helpText + "</html>");

            testAgain.setEnabled(true);
        }
    }

    /** Creates new form GStreamerScreen */
    public GStreamerScreen()
    {
        initComponents();

        status.setText("");
        gstFound.setText("");
        gstVersion.setText("");
        gstPlugins.setText("");

        help.setText("");
        unsupported.setVisible(false);
    }

    public void display(WizardFrame wizard)
    {
        this.wizard = wizard;

        wizard.setContent(this);

        if (!done)
        {
            wizard.setButtons(false, false);

            runTests();
        }
        else
        {
            wizard.setButtons(true, true);
        }
    }

    @Override
    public void hide()
    {
        return;
    }

    protected void runTests()
    {
        testAgain.setEnabled(false);

        Tester t = new Tester();
        t.start();

        // config stuff

        Config config;
        try
        {
            config = new Config("armadillo.config");
        }
        catch (Exception e)
        {
            System.err.println("Error: runTests(): Cannot open config.");

            return;
        }

        String gd_s = config.getKey("graphDetail");
        int gd;

        if (gd_s == null)
        {
            gd = gdMed;
            writeConfig();
        }
        else
        {
            gd = Integer.parseInt(gd_s);
        }

        perfUpdate = false;
        switch (gd)
        {
            case gdHigh:
                gstPerf.setSelectedIndex(2);
                break;
            case gdMed:
                gstPerf.setSelectedIndex(1);
                break;
            case gdLow:
                gstPerf.setSelectedIndex(0);
                break;
        }
        perfUpdate = true;
    }

    protected void writeConfig()
    {
        Config config;

        try
        {
            config = new Config("armadillo.config");
        }
        catch (Exception e)
        {
            System.err.println("Error: writeConfig(): Cannot open config.");

            return;
        }

        String perf = gstPerf.getSelectedItem().toString();
        String gd;
        String md;

        if (perf.equals("High"))
        {
            gd = Integer.toString(gdHigh);
            md = "3";
        }
        else if (perf.equals("Medium"))
        {
            gd = Integer.toString(gdMed);
            md = "2";
        }
        else
        {
            gd = Integer.toString(gdLow);
            md = "1";
        }

        config.setKey("graphDetail", gd);
        config.setKey("moodDetail", md);

        if (!config.save())
        {
            System.err.println("Error: writeConfig(): Cannot save config.");
        }

        System.out.format("Writing new config:\ngraphDetail=%s\nmoodDetail=%s\n", gd, md);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        bar = new javax.swing.JProgressBar();
        jLabel3 = new javax.swing.JLabel();
        status = new javax.swing.JLabel();
        testAgain = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        gstVersion = new javax.swing.JLabel();
        gstFound = new javax.swing.JLabel();
        help = new javax.swing.JLabel();
        unsupported = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        gstPlugins = new javax.swing.JLabel();
        gstPerf = new javax.swing.JComboBox();

        jLabel1.setFont(new java.awt.Font("Tahoma", 1, 12));
        jLabel1.setText("Testing");

        jLabel2.setText("Please wait while setup tests the multimedia capabilities of your computer.");

        jLabel3.setText("Status:");

        status.setText("jLabel4");

        testAgain.setText("Test again");
        testAgain.setEnabled(false);
        testAgain.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                testAgainActionPerformed(evt);
            }
        });

        jLabel4.setText("GStreamer:");

        jLabel5.setText("Version:");

        jLabel6.setText("Performance:");

        gstVersion.setText("jLabel8");

        gstFound.setText("jLabel9");

        help.setText("jLabel10");

        unsupported.setFont(new java.awt.Font("Tahoma", 1, 11));
        unsupported.setForeground(java.awt.Color.red);
        unsupported.setText("This version of GStreamer is not supported! Please upgrade.");

        jLabel7.setText("Plugins:");

        gstPlugins.setText("jLabel8");

        gstPerf.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Low", "Medium", "High" }));
        gstPerf.setSelectedIndex(1);
        gstPerf.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                gstPerfItemStateChanged(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel4)
                            .addComponent(jLabel5)
                            .addComponent(jLabel7)
                            .addComponent(jLabel6))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(gstPerf, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(gstPlugins)
                            .addComponent(gstVersion)
                            .addComponent(gstFound)))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(help))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(unsupported))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jLabel1)
                            .addComponent(jLabel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(bar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(testAgain, javax.swing.GroupLayout.Alignment.TRAILING)))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(jLabel3)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(status)))
                .addContainerGap(243, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addGap(18, 18, 18)
                .addComponent(jLabel2)
                .addGap(18, 18, 18)
                .addComponent(bar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(status))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(testAgain)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(gstFound))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(gstVersion))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(gstPlugins))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(gstPerf, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(unsupported)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(help)
                .addContainerGap(103, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void testAgainActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_testAgainActionPerformed
        gstFound.setText("");
        gstVersion.setText("");
        gstPlugins.setText("");

        runTests();
    }//GEN-LAST:event_testAgainActionPerformed

    private void gstPerfItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_gstPerfItemStateChanged
        if (evt.getStateChange() == ItemEvent.SELECTED
                && perfUpdate)
        {
            //System.out.println(gstPerf.getSelectedItem().toString());

            writeConfig();
        }
    }//GEN-LAST:event_gstPerfItemStateChanged


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JProgressBar bar;
    private javax.swing.JLabel gstFound;
    private javax.swing.JComboBox gstPerf;
    private javax.swing.JLabel gstPlugins;
    private javax.swing.JLabel gstVersion;
    private javax.swing.JLabel help;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel status;
    private javax.swing.JButton testAgain;
    private javax.swing.JLabel unsupported;
    // End of variables declaration//GEN-END:variables

}
