/*
 * JavaScreen.java
 *
 * Created on 20-Apr-2010, 18:16:09
 */

package armadillosetup;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.JFileChooser;

/**
 *
 * @author Mark Tolley
 */
class JavaScreen extends javax.swing.JPanel implements WizardScreen
{

    protected WizardFrame wizard;

    /** Creates new form JavaScreen */
    public JavaScreen()
    {
        initComponents();

        verUnsupported.setVisible(false);
        vendUnsupported.setVisible(false);
        archUnsupported.setVisible(false);

        help.setVisible(false);
        help.setText("");
    }

    @Override
    public void display(WizardFrame wizard)
    {
        this.wizard = wizard;

        wizard.setButtons(true, true);

        gatherData();

        wizard.setContent(this);
    }

    @Override
    public void hide()
    {
        return;
    }

    protected void gatherData()
    {
        boolean badVer = false, badVend = false, badArch = false;
        String helpStr = "";

        javaPath.setText(System.getProperty("java.home"));

        String ver = System.getProperty("java.version");
        String vend = System.getProperty("java.vendor");
        String arch = System.getProperty("os.arch");

        Pattern p = Pattern.compile("(\\d+)\\.(\\d+)\\.(\\d+)_(\\d+)");
        Matcher m = p.matcher(ver);

        if (m.matches())
        {
            int major = Integer.parseInt(m.group(1));
            int minor = Integer.parseInt(m.group(2));
            int micro = Integer.parseInt(m.group(3));
            int update = Integer.parseInt(m.group(4));

            if (!((major == 1 && minor == 6 && update > 0)
                    || (major == 1 && minor > 6)
                    || (major > 1)))
            {
                badVer = true;
            }
        }
        else
        {
            badVer = true;
        }

        if (badVer)
        {
            verUnsupported.setVisible(true);

            helpStr = helpStr
                    + "<p>You are running an old version of Java. You need to update it.</p>";
        }

        if (!vend.equals("Sun Microsystems Inc."))
        {
            badVend = true;
            vendUnsupported.setVisible(true);

            helpStr = helpStr
                    + "<p>Only the official Java Runtime Environment available from Sun Microsystems Inc. is supported by Armadillo.</p>";
        }
        
        if (!(arch.equals("x86") || arch.equals("i386")))
        {
            badArch = true;
            archUnsupported.setVisible(true);

            helpStr = helpStr + "<p>Only the 32bit version of Java is supported by Armadillo. Please install it alongside your current 64bit JRE.</p>";
        }

        if (badVer || badVend || badArch)
        {
            helpStr = helpStr + "<p>Please visit <a href=\"http://www.java.com\">http://www.java.com</a> for more information.</p>";
            help.setText("<html>" + helpStr + "</html>");
            help.setVisible(true);

            wizard.setButtons(true, false);
        }
        
        javaVersion.setText(ver);
        javaVendor.setText(vend);
        javaArch.setText(System.getProperty("os.arch"));
    }

    protected void changeJava()
    {
        int ret;

        JFileChooser file = new JFileChooser(javaPath.getText());

        file.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        ret = file.showDialog(wizard, "Select");
        if (ret == JFileChooser.APPROVE_OPTION)
        {
            Util.writeln("setup/armadillo.setup.jre", file.getSelectedFile().toString());
            System.exit(2);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        javaPath = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        javaVersion = new javax.swing.JLabel();
        javaVendor = new javax.swing.JLabel();
        javaArch = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        verUnsupported = new javax.swing.JLabel();
        vendUnsupported = new javax.swing.JLabel();
        help = new javax.swing.JLabel();
        archUnsupported = new javax.swing.JLabel();

        jLabel1.setFont(new java.awt.Font("Tahoma", 1, 12));
        jLabel1.setText("Java Settings");

        jLabel2.setText("A Java installation has been found in:");

        javaPath.setEditable(false);

        jLabel3.setText("Version:");

        jLabel4.setText("Vendor:");

        jLabel5.setText("Arch:");

        javaVersion.setText("jLabel6");

        javaVendor.setText("jLabel7");

        javaArch.setText("jLabel8");

        jButton1.setText("Change");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jLabel6.setText("Please review the settings below and ensure they are correct.");

        jLabel7.setText("If you are unsure of what to do, it is probably ok to skip this step.");

        verUnsupported.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        verUnsupported.setForeground(java.awt.Color.red);
        verUnsupported.setText("This version of Java is not supported! Please upgrade.");

        vendUnsupported.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        vendUnsupported.setForeground(java.awt.Color.red);
        vendUnsupported.setText("This version of Java is not supported!");

        help.setText("jLabel10");

        archUnsupported.setFont(new java.awt.Font("Tahoma", 1, 11)); // NOI18N
        archUnsupported.setForeground(java.awt.Color.red);
        archUnsupported.setText("This version of Java is not supported!");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel3)
                            .addComponent(jLabel4)
                            .addComponent(jLabel5))
                        .addGap(18, 18, 18)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(javaVersion)
                            .addComponent(javaVendor)
                            .addComponent(javaArch))
                        .addGap(18, 18, 18)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(verUnsupported)
                            .addComponent(vendUnsupported)
                            .addComponent(archUnsupported)))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(javaPath, javax.swing.GroupLayout.PREFERRED_SIZE, 365, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(jLabel2))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(jLabel7))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(jLabel6))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(10, 10, 10)
                        .addComponent(jLabel1))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jButton1))
                    .addGroup(layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(help)))
                .addContainerGap(256, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel1)
                .addGap(18, 18, 18)
                .addComponent(jLabel6)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel7)
                .addGap(18, 18, 18)
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(javaPath, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(javaVersion)
                    .addComponent(verUnsupported))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(javaVendor)
                    .addComponent(vendUnsupported))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(javaArch)
                    .addComponent(archUnsupported))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jButton1)
                .addGap(18, 18, 18)
                .addComponent(help)
                .addContainerGap(157, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        changeJava();
    }//GEN-LAST:event_jButton1ActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel archUnsupported;
    private javax.swing.JLabel help;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel javaArch;
    private javax.swing.JTextField javaPath;
    private javax.swing.JLabel javaVendor;
    private javax.swing.JLabel javaVersion;
    private javax.swing.JLabel vendUnsupported;
    private javax.swing.JLabel verUnsupported;
    // End of variables declaration//GEN-END:variables

}
