/*
 * WizardFrame.java
 *
 * Created on 20-Apr-2010, 16:01:08
 */

package armadillosetup;

import java.awt.Image;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 *
 * @author Mark Tolley
 */
class WizardFrame extends javax.swing.JFrame
{
    protected int screen = 0;
    protected boolean finished = false;

    protected ArrayList<WizardScreen> screens;
    protected JPanel content = null;

    /** Creates new form WizardFrame */
    public WizardFrame(String title, ArrayList<WizardScreen> screens)
    {
        this.screens = screens;

        initComponents();

        setLocationRelativeTo(null);
        setTitle(title);

        ImageIcon myIcon = new javax.swing.ImageIcon(getClass().getResource("/armadillosetup/graphics/ScribbleArmadilloBanner.jpg"));
        Image img = myIcon.getImage();
        Image newimg = img.getScaledInstance(441, 157,  java.awt.Image.SCALE_SMOOTH);
        //newIcon = new ImageIcon(newimg);
        ImageIcon myIconNew = new ImageIcon(newimg);

        banner.setText("");
        banner.setIcon(myIconNew);

        invalidate();

        displayScreen(screen);
    }

    public void setContent(JPanel content)
    {
        if (this.content != null)
        {
            body.remove(this.content);
        }
        
        body.add(content);
        body.validate();

        this.content = content;

        this.repaint();
    }

    public void setButtons(boolean back, boolean next)
    {
        this.back.setEnabled(back);
        this.next.setEnabled(next);
    }

    public void setFinished()
    {
        back.setEnabled(false);
        next.setEnabled(true);

        exit.setEnabled(false);

        next.setText("Finish");
        finished = true;
    }

    protected void displayScreen(int num)
    {
        WizardScreen wc = screens.get(num);
        wc.display(this);
    }

    protected void hideScreen(int num)
    {
        WizardScreen wc = screens.get(num);
        wc.hide();
    }

    public void advance(int x)
    {
        screen += x;
        displayScreen(screen);
    }

    public void advance()
    {
        advance(1);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        header = new javax.swing.JPanel();
        jSeparator2 = new javax.swing.JSeparator();
        banner = new javax.swing.JLabel();
        body = new javax.swing.JPanel();
        footer = new javax.swing.JPanel();
        next = new javax.swing.JButton();
        back = new javax.swing.JButton();
        exit = new javax.swing.JButton();
        jSeparator1 = new javax.swing.JSeparator();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(509, 497));
        setResizable(false);

        header.setBackground(java.awt.Color.white);
        header.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        header.add(jSeparator2, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 174, 509, -1));

        banner.setText("jLabel1");
        banner.setMaximumSize(new java.awt.Dimension(441, 157));
        banner.setMinimumSize(new java.awt.Dimension(441, 157));
        header.add(banner, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 11, 441, 157));

        body.setLayout(new java.awt.GridLayout(1, 0));

        next.setText("Next >");
        next.setEnabled(false);
        next.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                nextActionPerformed(evt);
            }
        });

        back.setText("< Back");
        back.setEnabled(false);
        back.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backActionPerformed(evt);
            }
        });

        exit.setText("Exit");
        exit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout footerLayout = new javax.swing.GroupLayout(footer);
        footer.setLayout(footerLayout);
        footerLayout.setHorizontalGroup(
            footerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(footerLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(exit)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 282, Short.MAX_VALUE)
                .addComponent(back)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(next)
                .addContainerGap())
            .addComponent(jSeparator1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 509, Short.MAX_VALUE)
        );

        footerLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {back, exit, next});

        footerLayout.setVerticalGroup(
            footerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, footerLayout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addComponent(jSeparator1, javax.swing.GroupLayout.DEFAULT_SIZE, 11, Short.MAX_VALUE)
                .addGap(0, 0, 0)
                .addGroup(footerLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(next)
                    .addComponent(back)
                    .addComponent(exit))
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(footer, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(header, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(body, javax.swing.GroupLayout.DEFAULT_SIZE, 509, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(header, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, 0)
                .addComponent(body, javax.swing.GroupLayout.DEFAULT_SIZE, 446, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(footer, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void exitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitActionPerformed
        int ret;

        ret = JOptionPane.showConfirmDialog(this, "Setup is not complete. If you exit now, the program may not be installed correctly.\n\n" +
                "You may run Setup again at another time to complete the installation.\n\n" +
                "Exit Setup?", "Exit Setup", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);

        if (ret == JOptionPane.YES_OPTION)
        {
            System.exit(0);
        }
    }//GEN-LAST:event_exitActionPerformed

    private void nextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_nextActionPerformed
        if (!finished)
        {
            ++screen;
            displayScreen(screen);
        }
        else
        {
            hideScreen(screen);
            System.exit(0);
        }
    }//GEN-LAST:event_nextActionPerformed

    private void backActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backActionPerformed
        --screen;
        displayScreen(screen);
    }//GEN-LAST:event_backActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton back;
    private javax.swing.JLabel banner;
    private javax.swing.JPanel body;
    private javax.swing.JButton exit;
    private javax.swing.JPanel footer;
    private javax.swing.JPanel header;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JButton next;
    // End of variables declaration//GEN-END:variables

}
